#include "TBCompliance.h"
#include "../../src/aMain.h"

//extern "C" void __cxa_pure_virtual () {}

TBCompliance::TBCompliance(AEEApplet* app,byte localizationCode){
	DEBUG(("TBCompliance(): Initializing"));

	myMain = (aMain*)app;
	strings = new TBStringFactory(localizationCode);

	moduleState = MODULE_STATE_OFF;
	menuState = MENU_OFF;
	isGameRunning = false;

	animationState = 0;
	dynamicTextCounter = -1;
	dynamicTextIndex = 0;
	dynamicTextOptions.append(FP_STR_CONST_ID_TEST_TEXT_DYNAMIC_1);
	dynamicTextOptions.append(FP_STR_CONST_ID_TEST_TEXT_DYNAMIC_2);
	sizeOptions.append(4);
	sizeOptions.append(8);
	sizeOptions.append(12);
	sizeOptionIndexSelected = 2;
	
	// PROGRESS TEST controlls
	progressState = MENU_ELEMENT_PROGRESSBAR_INITIAL_STATE;
	progressRateOptions.append(2);
	progressRateOptions.append(10);
	int progressRateIndexSelected = 0;
	
	fpClientControlLock = 0;
	
	TBEvents::registerEventHandler(strings);
	TBEvents::registerEventHandler((ITBMenuControls*)this);
	TBEvents::registerEventHandler((ITBClientControls*)this);
	
	clientTrigger = TBEvents::getClientEventListener();
	menuTrigger = TBEvents::getMenuEventListener();
	
	titleImage = ((aMain*)myMain)->myUtils->fetchImage(String("/tbg_logo_100w_anim04"));
	animationImages[0] = ((aMain*)myMain)->myUtils->fetchImage(String("/tbg_logo_100w_anim00"));
	animationImages[1] = ((aMain*)myMain)->myUtils->fetchImage(String("/tbg_logo_100w_anim01"));
	animationImages[2] = ((aMain*)myMain)->myUtils->fetchImage(String("/tbg_logo_100w_anim02"));
	animationImages[3] = ((aMain*)myMain)->myUtils->fetchImage(String("/tbg_logo_100w_anim03"));
	animationImages[4] = ((aMain*)myMain)->myUtils->fetchImage(String("/tbg_logo_100w_anim04"));
	animationImages[5] = ((aMain*)myMain)->myUtils->fetchImage(String("/tbg_logo_100w_anim03"));
	animationImages[6] = ((aMain*)myMain)->myUtils->fetchImage(String("/tbg_logo_100w_anim02"));
	animationImages[7] = ((aMain*)myMain)->myUtils->fetchImage(String("/tbg_logo_100w_anim01"));
	
}

void TBCompliance::Release(){
	if(strings)
		delete(strings);
	heapFill._delete();
	if(titleImage){
		titleImage->release();
		delete(titleImage);
		titleImage=NULL;
	}
	for(int i = 0; i < 8; i++){
		animationImages[i]->release();
		delete(animationImages[i]);
		animationImages[i]=NULL;
	}
	inputText.destruct();
	dynamicTextOptions._delete();
	sizeOptions._delete();
	progressRateOptions._delete();
}


bool TBCompliance::notifyModule(int code, int data){
	DEBUG(("TBCompliance:notifyModule(%d): Processing",moduleState));
	
	Bbyte tmp;
	int oldModuleState = moduleState;
	
	if (code > 0) {
		moduleState |= code;
			
	} else {
		moduleState &= code;
	}
	
	if (moduleState == oldModuleState) return false;
	
	if ((moduleState & MODULE_STATE_FLAG_GAME_ON) != 0) {
		if (!isGameRunning) {
			isGameRunning = true;
			moduleState &= ~MODULE_STATE_FLAG_MENU_ON;
		}
	} else {
		if (isGameRunning) {
			isGameRunning = false;
			curScore = data;
			moduleState |= MODULE_STATE_FLAG_MENU_ON;
			menuState = MENU_SCREEN_GAME_RESULTS;
			clientTrigger->notifyClient(CLIENT_START_MENU,tmp);
		}
	}
	
	if ((moduleState & MODULE_STATE_FLAG_MENU_ON) != 0) {
		
		if (heapFill.size() == 0) {
			heapFill._new(HEAP_FILL_SIZE, 0);
		}
		
		if (menuState == MENU_OFF) {
			clientTrigger->notifyClient(CLIENT_START_MENU,tmp);
			if ((moduleState & MODULE_STATE_FLAG_HELP_ONLY) != 0) {
				setMenu(MENU_SCREEN_HELP);
			} else {
				setMenu(MENU_SCREEN_START);
			}	
		} else {
			setMenu(menuState);
		}
		
	} else {
		heapFill.removeAll();
		if (menuState != MENU_OFF) {
			menuState = MENU_OFF;
			moduleState &= ~MODULE_STATE_FLAG_HELP_ONLY;
			clientTrigger->notifyClient(CLIENT_END_MENU,tmp);
		}
	}
	
	return true;
}

bool TBCompliance::menuCommand(int menuCommandData){
	if ((menuCommandData | MENU_FLAG_ACTION)  != 0) {
		DEBUG(("TBCompliance:menuCommand(%d) in menu: %s",(menuCommandData & MENU_MASK_FLAGS),getScreenName(menuState).toCharArray()));
	} 

	Bbyte tmp;
	
	bool toReturn = true;
	
	switch (menuCommandData) {
	
		case MENU_ACTION_START_GAME: {
//#ifdef DEBUG_ON
			DEBUG(("FP:MENU_ACTION_START_GAME"));
//#endif
			if (TBEvents::getSeedGenerator() ==NULL) DEBUG(("No Seed Generator Registered.  Bailing"));
			Bbyte seed = TBEvents::getSeedGenerator()->getSeed();
			if (seed.size() != 0) {
				notifyModule(MODULE_NOTIFY_START_GAME,0);
				if (!clientTrigger->notifyClient(CLIENT_GAME_START,seed)) {
					DEBUG(("FP:startGame() - Critical Error - CLIENT_GAME_START Not Handled"));
					return false;
				}
			} else {
				toReturn = setMenu(MENU_SCREEN_OUT_OF_SEEDS);
			}
		} break;
		
		case MENU_ACTION_END_GAME: {
//#ifdef DEBUG_ON
			DEBUG(("FP:MENU_ACTION_END_GAME"));
//#endif
			clientTrigger->notifyClient(CLIENT_START_MENU,tmp);
			toReturn = setMenu(MENU_SCREEN_GAME_RESULTS);
		} break;
		
		case MENU_ACTION_FORE: switch (menuState) {
			
			case MENU_SCREEN_TEST_PROGRESS: {
				if (progressState >= 100) {
					progressState = MENU_ELEMENT_PROGRESSBAR_INITIAL_STATE;
					update(menuState);
					break;
				}
			} case MENU_SCREEN_TEST_IMAGE: {
				setRun(!isRunning);
				update(menuState);
			} break;
					
			case MENU_SCREEN_TEST_TEXT: {
				inputText = "";
				update(menuState);
			} break;
			
			case MENU_SCREEN_GAME_RESULTS: {
				menuCommand(MENU_ACTION_START_GAME);
			} break;
			
			default: {
//#ifdef DEBUG_ON
				DEBUG(("menuCommand(MENUENTRY_COMMAND_FORE): In MenuId %d: Unknown",  (menuState & MENU_MASK_FLAGS)));	
//#endif
			} return false;
			
		} break;
		
		case MENU_ACTION_BACK: switch (menuState) {
			
			case MENU_SCREEN_FP_MAIN: 
			case MENU_SCREEN_HELP: {
				notifyModule(MODULE_NOTIFY_STOP_MENU,0);
			} break;
			
			case MENU_SCREEN_HELP_1:
			case MENU_SCREEN_HELP_2:
			case MENU_SCREEN_HELP_3: {
				toReturn = setMenu(MENU_SCREEN_HELP);
			} break;
				
			case MENU_SCREEN_EXERCISE_UI: 
			case MENU_SCREEN_OUT_OF_SEEDS: 
			case MENU_SCREEN_GAME_RESULTS: {
				toReturn = setMenu(MENU_SCREEN_FP_MAIN);		
			} break;
			
			case MENU_SCREEN_TEST_PROGRESS:
			case MENU_SCREEN_TEST_UNSUPPORTED_1:
			case MENU_SCREEN_TEST_UNSUPPORTED_2:
			case MENU_SCREEN_TEST_TEXT:
			case MENU_SCREEN_TEST_IMAGE: {
				toReturn = setMenu(MENU_SCREEN_EXERCISE_UI);		
			} break;
				
			default: {	
//#ifdef DEBUG_ON
				DEBUG(("menuCommand(MENUENTRY_COMMAND_BACK): In MenuId %d: Unknown",(menuState & MENU_MASK_FLAGS)));	
//#endif
			} return false;
				
		} break;
			
		/*
		 * This return is short circuited because if setScreen handles the input, 
		 * then actionState should not be updated
		 */
		default: return setMenu(menuCommandData);
		
	}

	return toReturn;
	
}

bool TBCompliance::menuInput(int menuId, int menuEntryIndex, int menuEntryType, int entryData, String entryText){
	DEBUG(("FP:menuInput(%s, %d, %d, %d, %s)",getScreenName(menuId).toCharArray(), menuEntryIndex, menuEntryType, entryData, entryText.toCharArray()));
	
	bool toReturn = true;;
	
	switch (menuId) {
		
		case MENU_SCREEN_TEST_IMAGE: { // We don't need to check index because there is only one input on this screen
			setRun(entryData == MENU_ELEMENT_SWITCH_ON);
			update(menuState);
		} break;
		
		case MENU_SCREEN_TEST_PROGRESS: {
			setRun(false);
			progressRateIndexSelected = entryData;
			progressState = MENU_ELEMENT_PROGRESSBAR_INITIAL_STATE;
			update(menuState);
		} break;
		
		case MENU_SCREEN_TEST_TEXT: switch (menuEntryIndex) {
			case 2: { // The Index of the text Input
				inputText = entryText;
			} break;
			case 3: { // The Index of the size control switch
				sizeOptionIndexSelected = entryData;
			} break;
			default: {
				DEBUG(("menuInput(%s, %d): Index is unknown Input Entry, did you update them?",getScreenName(menuId).toCharArray(), menuEntryIndex));
				return false;
			}
		} 
		update(menuState);
		break;
		
		default: {
			DEBUG(("menuInput(%d, %d, %d, %d, %s): Unknown Input Field", menuId , menuEntryIndex, menuEntryType, entryData, entryText.toCharArray()));
		} return false;
	}
	
	return toReturn;
	
}

String TBCompliance::getScreenName(int screenState){
	switch (screenState) {
		case MENU_SCREEN_FP_MAIN			: return strings->getString(FP_STR_CONST_ID_TITLE);
		case MENU_SCREEN_EXERCISE_UI		: return strings->getString(FP_STR_CONST_ID_TITLE_MENU_EXERCISE);
		case MENU_SCREEN_TEST_IMAGE			: return strings->getString(FP_STR_CONST_ID_TEST_IMAGE);
		case MENU_SCREEN_TEST_TEXT			: return strings->getString(FP_STR_CONST_ID_TEST_TEXT);
		case MENU_SCREEN_TEST_PROGRESS 		: return strings->getString(FP_STR_CONST_ID_TEST_PROGRESS);
		case MENU_SCREEN_TEST_UNSUPPORTED_1	: return strings->getString(FP_STR_CONST_ID_TEST_UNSUPPORTED) + ": Replacement Menu";
		case MENU_SCREEN_TEST_UNSUPPORTED_2 : return strings->getString(FP_STR_CONST_ID_TEST_UNSUPPORTED) + ": Replacement Entry";
		case MENU_SCREEN_HELP				: return strings->getString(FP_STR_CONST_ID_SCREEN_HELP);
		case MENU_SCREEN_OUT_OF_SEEDS 		: return strings->getString(FP_STR_CONST_ID_SCREEN_SEEDS);
		case MENU_SCREEN_GAME_RESULTS 		: return strings->getString(FP_STR_CONST_ID_SCREEN_RESULTS);
		case MENU_SCREEN_CLIENT				: return strings->getString(FP_STR_CONST_ID_SCREEN_CLIENT);
		default: return "Unknown State";
	}
};

bool TBCompliance::setMenu(int newScreenState){
	DEBUG(("%s --> %s",getScreenName(menuState).toCharArray(), getScreenName(newScreenState).toCharArray()));
//#endif
		// Stop Animations if they are running
	setRun(false);
	
	if ((newScreenState & MENU_FLAG_SCREEN) == MENU_NULL ) {
//#ifdef DEBUG_ON
		DEBUG(("setScreen(%d, %s): INVALID Menu Screen State", (newScreenState & MENU_MASK_FLAGS), getScreenName(newScreenState)));
//#endif
		return false;
	}
	
	bool toReturn = true;
	
	int oldScreenState = this->menuState;
	this->menuState = newScreenState;
	
	switch (newScreenState) {
	
		case MENU_SCREEN_FP_MAIN: {
			menuTrigger->setMenu(newScreenState,1,3,strings->getString(FP_STR_CONST_ID_TITLE));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			menuTrigger->setMenuEntry(newScreenState,0,
					MENUENTRY_DYNAMIC_IMAGE,
					"",-1,"",
					titleImage);
			menuTrigger->setMenuEntry(newScreenState,1,
					MENUENTRY_COMMAND,
					strings->getString(FP_STR_CONST_ID_TITLE_MENU_START),
					MENU_ACTION_START_GAME,
					"",NULL);
			menuTrigger->setMenuEntry(newScreenState,2,
					MENUENTRY_COMMAND,
					strings->getString(FP_STR_CONST_ID_TITLE_MENU_EXERCISE),
					MENU_SCREEN_EXERCISE_UI,
					"",NULL);
			menuTrigger->endSetMenu(newScreenState);
		} break;
		
		case MENU_SCREEN_OUT_OF_SEEDS: {
			menuTrigger->setMenu(newScreenState,1,1,strings->getString(FP_STR_CONST_ID_TITLE_MENU_EXERCISE));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			menuTrigger->setMenuEntry(newScreenState,0,
					MENUENTRY_STATIC_TEXT,
					"",
					MENU_SCREEN_TEST_IMAGE,
					strings->getString(FP_STR_CONST_ID_OUT_OF_SEEDS),
					NULL);	
			menuTrigger->endSetMenu(newScreenState);
		} break;
		
		case MENU_SCREEN_GAME_RESULTS: {
			menuTrigger->setMenu(newScreenState,1,2,strings->getString(FP_STR_CONST_ID_TITLE_MENU_EXERCISE));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_FORE,
					strings->getString(FP_STR_CONST_ID_REPLAY),
					-1,"",NULL);
			menuTrigger->setMenuEntry(newScreenState,0,
					MENUENTRY_STATIC_TEXT,
					"",0,
					strings->getString(FP_STR_CONST_ID_END_GAME_TEXT),
					NULL);	
			char str[10];
			menuTrigger->setMenuEntry(newScreenState,1,
					MENUENTRY_DYNAMIC_TEXT,
					"",0,
					strings->getString(FP_STR_CONST_ID_END_GAME_SCORE) + _itoa(curScore,str,10),
					NULL);	
			menuTrigger->endSetMenu(newScreenState);
		} break;
		
		case MENU_SCREEN_EXERCISE_UI: {
			menuTrigger->setMenu(newScreenState,1,4,strings->getString(FP_STR_CONST_ID_TITLE_MENU_EXERCISE));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			menuTrigger->setMenuEntry(newScreenState,0,
					MENUENTRY_COMMAND,
					strings->getString(FP_STR_CONST_ID_TEST_IMAGE),
					MENU_SCREEN_TEST_IMAGE,
					"",NULL);
			menuTrigger->setMenuEntry(newScreenState,1,
					MENUENTRY_COMMAND,
					strings->getString(FP_STR_CONST_ID_TEST_TEXT),
					MENU_SCREEN_TEST_TEXT,
					"",NULL);
			menuTrigger->setMenuEntry(newScreenState,2,
					MENUENTRY_COMMAND,
					strings->getString(FP_STR_CONST_ID_TEST_PROGRESS),
					MENU_SCREEN_TEST_PROGRESS,
					"",NULL);
			menuTrigger->setMenuEntry(newScreenState,3,
					MENUENTRY_COMMAND,
					strings->getString(FP_STR_CONST_ID_TEST_UNSUPPORTED),
					MENU_SCREEN_TEST_UNSUPPORTED_1,
					"",NULL);
			menuTrigger->endSetMenu(newScreenState);
		} break;
		
		case MENU_SCREEN_TEST_IMAGE: {
			menuTrigger->setMenu(newScreenState,1,2,strings->getString(FP_STR_CONST_ID_TEST_IMAGE));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			update(newScreenState);
			menuTrigger->endSetMenu(newScreenState);
			setRun(true);
		} break;
		
		case MENU_SCREEN_TEST_TEXT: {
			menuTrigger->setMenu(newScreenState,1,4,strings->getString(FP_STR_CONST_ID_TEST_TEXT));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_FORE,
					strings->getString(FP_STR_CONST_ID_TEST_TEXT_INPUT_CLEAR),
					-1,"",NULL);
			menuTrigger->setMenuEntry(newScreenState,0,
					MENUENTRY_DYNAMIC_TEXT,
					"",-1,
					strings->getString(FP_STR_CONST_ID_TEST_TEXT_DYNAMIC_1),
					NULL);
			menuTrigger->setMenuEntry(newScreenState,1,
					MENUENTRY_STATIC_TEXT,
					"",0,
					strings->getString(FP_STR_CONST_ID_TEST_TEXT_OUTPUT),
					NULL);
			if (inputText.length() == 0) inputText = strings->getString(FP_STR_CONST_ID_TEST_TEXT_INPUT_DEFAULT);
			update(newScreenState);
			menuTrigger->endSetMenu(newScreenState);
			setRun(true);
		} break;
		
		case MENU_SCREEN_TEST_PROGRESS: {
			menuTrigger->setMenu(newScreenState,1,2,strings->getString(FP_STR_CONST_ID_TEST_IMAGE));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			update(newScreenState);
			menuTrigger->endSetMenu(newScreenState);
			setRun(true);
		} break;
		
		
		case MENU_SCREEN_TEST_UNSUPPORTED_1: {
			menuTrigger->setMenu(newScreenState,1,2,strings->getString(FP_STR_CONST_ID_TEST_UNSUPPORTED));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			menuTrigger->setMenuEntry(newScreenState,0,
					MENUENTRY_STATIC_TEXT,
					"",	-1,
					strings->getString(FP_STR_CONST_ID_TEST_UNSUPPORTED_TEXT),
					NULL);
			if (menuTrigger->setMenuEntry(newScreenState,1,
					MENU_MASK_FLAGS, // An intentionally Bogus Menu Entry Type
					strings->getString(FP_STR_CONST_ID_TEST_UNSUPPORTED_TEXT),
					-1,"",NULL)) {
				DEBUG(("Unsupported Menu Entry Type Accepted: Replacement Menu Attempt"));
			} else {
				setMenu(MENU_SCREEN_TEST_UNSUPPORTED_2);
			}
			menuTrigger->endSetMenu(newScreenState);
		} break;
		
		case MENU_SCREEN_TEST_UNSUPPORTED_2: {
			menuTrigger->setMenu(newScreenState,1,1,strings->getString(FP_STR_CONST_ID_TEST_UNSUPPORTED));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			if (menuTrigger->setMenuEntry(newScreenState,0,
					MENU_MASK_FLAGS, // An intentionally Bogus Menu Entry Type
					strings->getString(FP_STR_CONST_ID_TEST_UNSUPPORTED_TEXT),
					-1,"",NULL)) {
				DEBUG(("Unsupported Menu Entry Type Accepted: Replacement Entry Attempt"));
			} else {
				menuTrigger->setMenuEntry(newScreenState,0,
						MENUENTRY_STATIC_TEXT,
						"", -1,
						strings->getString(FP_STR_CONST_ID_TEST_UNSUPPORTED_TEXT),
						NULL);
			}
			menuTrigger->endSetMenu(newScreenState);
		} break;
		
		case MENU_SCREEN_HELP: {
			menuTrigger->setMenu(newScreenState,1,3,strings->getString(FP_STR_CONST_ID_SCREEN_HELP));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			menuTrigger->setMenuEntry(newScreenState,0,
					MENUENTRY_COMMAND,
					strings->getString(FP_STR_CONST_ID_SCREEN_HELP_1),
					MENU_SCREEN_HELP_1,
					"",NULL);
			menuTrigger->setMenuEntry(newScreenState,1,
					MENUENTRY_COMMAND,
					strings->getString(FP_STR_CONST_ID_SCREEN_HELP_2),
					MENU_SCREEN_HELP_2,
					"",NULL);
			menuTrigger->setMenuEntry(newScreenState,2,
					MENUENTRY_COMMAND,
					strings->getString(FP_STR_CONST_ID_SCREEN_HELP_3),
					MENU_SCREEN_HELP_3,
					"",NULL);
			menuTrigger->endSetMenu(newScreenState);
		} break;
		
		case MENU_SCREEN_HELP_1: {
			menuTrigger->setMenu(newScreenState,1,1,strings->getString(FP_STR_CONST_ID_TITLE_MENU_EXERCISE));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			menuTrigger->setMenuEntry(newScreenState,0,
					MENUENTRY_STATIC_TEXT,
					"",0,
					strings->getString(FP_STR_CONST_ID_TEST_TEXT_OUTPUT),
					NULL);	
			menuTrigger->endSetMenu(newScreenState);
		} break;
		
		case MENU_SCREEN_HELP_2: {
			menuTrigger->setMenu(newScreenState,1,1,strings->getString(FP_STR_CONST_ID_TITLE_MENU_EXERCISE));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			menuTrigger->setMenuEntry(newScreenState,0,
					MENUENTRY_STATIC_TEXT,
					"",0,
					strings->getString(FP_STR_CONST_ID_TEST_UNSUPPORTED_TEXT),
					NULL);	
			menuTrigger->endSetMenu(newScreenState);
		} break;
		
		case MENU_SCREEN_HELP_3: {
			menuTrigger->setMenu(newScreenState,1,1,strings->getString(FP_STR_CONST_ID_TITLE_MENU_EXERCISE));
			menuTrigger->setMenuEntry(newScreenState,-1,
					MENUENTRY_COMMAND_BACK,
					strings->getString(FP_STR_CONST_ID_BACK),
					-1,"",NULL);
			menuTrigger->setMenuEntry(newScreenState,0,
					MENUENTRY_STATIC_TEXT,
					"",0,
					strings->getString(FP_STR_CONST_ID_OUT_OF_SEEDS),
					NULL);	
			menuTrigger->endSetMenu(newScreenState);
		} break;
		
		default: {
			DEBUG(("setScreen(%d): UNKNOWN Menu Screen State, reverting too: %s",(newScreenState & MENU_MASK_FLAGS), getScreenName(oldScreenState).toCharArray()));
			this->menuState = oldScreenState;
		}	return false;
		
	}
	
	return toReturn;
}

void TBCompliance::update(int menuState){
	String caption;
	switch (menuState) {
			
		case MENU_SCREEN_TEST_PROGRESS: {
			
			caption = "";//strings->getString(FP_STR_CONST_ID_TEST_PROGRESS_STATE);
			
			if (progressState >= 100) {
				setRun(false);
				caption += strings->getString(FP_STR_CONST_ID_TEST_PROGRESS_COMPLETE);
				menuTrigger->setMenuEntry(menuState,-1,
						MENUENTRY_COMMAND_FORE,
						strings->getString(FP_STR_CONST_ID_TEST_PROGRESS_COMMAND_RESET),
						-1,"",NULL);
				progressState = 100;
			} else if (isRunning) {
				progressState += 100/(progressRateOptions[progressRateIndexSelected] * ANIMATION_TICKS_PER_SECOND);
				caption+=strings->getString(FP_STR_CONST_ID_TEST_PROGRESS_RUNNING);
				menuTrigger->setMenuEntry(menuState,-1,
						MENUENTRY_COMMAND_FORE,
						strings->getString(FP_STR_CONST_ID_TEST_PROGRESS_COMMAND_PAUSE),
						-1,"",NULL);
			} else if (progressState > 0) {
				caption+=strings->getString(FP_STR_CONST_ID_TEST_PROGRESS_PAUSED);
				menuTrigger->setMenuEntry(menuState,-1,
						MENUENTRY_COMMAND_FORE,
						strings->getString(FP_STR_CONST_ID_TEST_PROGRESS_COMMAND_RUN),
						-1,"",NULL);
			} else {
				caption+=strings->getString(FP_STR_CONST_ID_TEST_PROGRESS_READY);
				menuTrigger->setMenuEntry(menuState,-1,
						MENUENTRY_COMMAND_FORE,
						strings->getString(FP_STR_CONST_ID_TEST_PROGRESS_COMMAND_RUN),
						-1,"",NULL);
			}
			
			menuTrigger->setMenuEntry(menuState,0,
					MENUENTRY_DYNAMIC_PROGRESS,
					caption,
					progressState,
					"",
					NULL);
			
			caption = "";
			
			for (UINT progressOptionsIndex = 0; progressOptionsIndex < progressRateOptions.size(); progressOptionsIndex++ ){
				//caption.append(progressRateOptions[progressOptionsIndex]).append(strings->getString(FP_STR_CONST_ID_TEST_PROGRESS_RUN));
				if(progressRateOptions[progressOptionsIndex] == 2)
					caption += "2";
				else
					caption += "10";
				caption += strings->getString(FP_STR_CONST_ID_TEST_PROGRESS_RUN);
				if (progressOptionsIndex < progressRateOptions.size() - 1) caption+=",";
			}

			menuTrigger->setMenuEntry(menuState,1,
				MENUENTRY_INPUT_SWITCH,
				strings->getString(FP_STR_CONST_ID_TEST_PROGRESS_RUN_QUESTION),
				progressRateIndexSelected,
				caption,
				NULL);
			
		} break;
	
		case MENU_SCREEN_TEST_IMAGE: {
			
			caption = strings->getString(FP_STR_CONST_ID_TEST_IMAGE_ANIMATION);
			
			if (isRunning) {
				caption+=strings->getString(FP_STR_CONST_ID_ON);
				if (++animationState == 8) animationState = 0;
				menuTrigger->setMenuEntry(menuState,-1,
						MENUENTRY_COMMAND_FORE,
						strings->getString(FP_STR_CONST_ID_TOGGLE) + strings->getString(FP_STR_CONST_ID_OFF),
						-1,"",NULL);
			} else {
				caption+=strings->getString(FP_STR_CONST_ID_OFF);
				menuTrigger->setMenuEntry(menuState,-1,
						MENUENTRY_COMMAND_FORE,
						strings->getString(FP_STR_CONST_ID_TOGGLE) + strings->getString(FP_STR_CONST_ID_ON),
						-1,"",NULL);
			}
			
			menuTrigger->setMenuEntry(menuState,0,
					MENUENTRY_DYNAMIC_IMAGE,
					caption,
					animationState,
					"",
					animationImages[animationState]);
			
			caption = "";
			caption += strings->getString(FP_STR_CONST_ID_NO);
			caption += ",";
			caption += strings->getString(FP_STR_CONST_ID_YES);
			
			int selectIndex = MENU_ELEMENT_SWITCH_OFF;
			if (isRunning) selectIndex = MENU_ELEMENT_SWITCH_ON;
			
			menuTrigger->setMenuEntry(menuState,1,
				MENUENTRY_INPUT_SWITCH,
				strings->getString(FP_STR_CONST_ID_TEST_IMAGE_ANIMATION_QUESTION),
				selectIndex,
				caption,
				NULL);
		} break;
		
		case MENU_SCREEN_TEST_TEXT: {
			
			// Dynamic text screen state test
			if (++dynamicTextCounter >= ANIMATION_TEXT_CYCLE_TIME * ANIMATION_TICKS_PER_SECOND) {
				dynamicTextIndex = ++dynamicTextIndex < dynamicTextOptions.size() ? dynamicTextIndex : 0;
				dynamicTextCounter = 0;
				menuTrigger->setMenuEntry(menuState,0,
							MENUENTRY_DYNAMIC_TEXT,
							"",-1,
							strings->getString(dynamicTextOptions[dynamicTextIndex])
							,NULL);
					
			}
				
			if (sizeOptions[sizeOptionIndexSelected] < (int)inputText.length()) {
				inputText = inputText.substring(0,sizeOptions[sizeOptionIndexSelected]);
			}
			
			menuTrigger->setMenuEntry(menuState,2,
					MENUENTRY_INPUT_TEXT,
					strings->getString(FP_STR_CONST_ID_TEST_TEXT_INPUT_TITLE),
					sizeOptions[sizeOptionIndexSelected],
					inputText,
					NULL);
			
			String sizeOptionBuffer = "";
			for (UINT sizeOptionIndex = 0; sizeOptionIndex < sizeOptions.size(); sizeOptionIndex++) {
				char str[10];
				sizeOptionBuffer += _itoa(sizeOptions[sizeOptionIndex],str,10);
				sizeOptionBuffer += strings->getString(FP_STR_CONST_ID_TEST_TEXT_INPUT_SIZE);
				if (sizeOptionIndex < sizeOptions.size() -1) sizeOptionBuffer += ",";
			}
			
			menuTrigger->setMenuEntry(menuState,3,
					MENUENTRY_INPUT_SWITCH,
					strings->getString(FP_STR_CONST_ID_TEST_TEXT_INPUT_SIZE_QUESTION),
					sizeOptionIndexSelected,
					sizeOptionBuffer,
					NULL);
			
		} break;
		
	}
}

void TBCompliance::setRun(bool state){
	if (state && !isRunning) {
		isRunning = true;
		run(this);
	} else if (!state){
		isRunning = false;	
		//update(menuState);
	}
}

void TBCompliance::run(TBCompliance *pTBC){
	if(pTBC->isRunning){
		pTBC->update(pTBC->menuState);
		ISHELL_SetTimer(((aMain*)pTBC->myMain)->m_pIShell,(1000 / ANIMATION_TICKS_PER_SECOND), ( PFNNOTIFY )run, pTBC);
	}
}