#ifndef _ITB_CLIENT_CONTROLS_H_
#define _ITB_CLIENT_CONTROLS_H_

/**
 * <p>ClientControls Provides a mechanism for the client to pass updates on major
 * game events back to the FP module for information/processing.  These game events
 * All revolve around the state of the <b>current For-Prizes Related Gameplay</b>.  
 * If these methods are called while the FPModule is not currently involved in a For
 * Prizes Session, they will return false and perform no action.<p>
 * 
 * <p>The interface implementation used to invoke these commands should be gathered 
 * by calling: </p> 
 * <pre>TBEvents.getClientControls()</pre>
 * <p>During the initialization portion of the client</p>
 * 
 * <p>The client is not allowed to implement or register ITBClientControls, that is the sole
 * responsibility of the ForPrize Module.  This interface is provided simply as a reporting
 * mechanism for the client to the Module.</p>
 * 
 * <p>This Interface Provides for Client ---> Module Communication.  For more information about
 * Module ---> Client communication, see <i>ITBClientEventListener</i>.</p>
 *  
 * @author E Scott Cragg. Twistbox Client Core Engineering.
 *
 * and Tiago Teixeira. Twistbox Client Core Engineering.
 */
class ITBClientControls {
	
	/**
	 * <p>Notify the module that a given event has occurred, with supporting data if necessary</p>
	 * 
	 * <p>For More information about the specific events that can be passed, see the MODULE_ events 
	 * in Interface FPC.</p>
	 * 
	 * @param code See documentation for TBC
	 * @param data See documentation for TBC 
	 * @return true if the event is known and handled.  false if otherwise.
	 */
public:
	virtual bool notifyModule(int code, int data) = 0;
	
};

#endif //_ITB_CLIENT_CONTROLS_H_
