#ifndef _ITB_CLIENT_EVENT_LISTENER_H_
#define _ITB_CLIENT_EVENT_LISTENER_H_

#include "..\..\src\BrewVector.h"
/**
 * <p>ITBClientEventListener Provides a mechanism for the module to pass updates on major
 * game events back to the FP module for information/processing.  These game events
 * All revolve around the state of the <b>current For-Prizes Related Gameplay</b>.  
 *
 * <p>These events will only be called with the FP Module has control of the clients
 * User Interface</p>
 * 
 * <p>The client should register all classes that which to recieve these evetns by calling: </p> 
 * <pre>TBEvents.registerEventListener(IFPClientEventListener)</pre>
 * <p>During the initialization portion of the client.  Multiple listeners can be registered.
 * Listeners will be called in Last-in-first-out order (LI-FO) starting with the most
 * recently registered listener until the first listener returns true.  Once added, listeners cannot 
 * be removed and no checking is done for duplicate listeners</p>
 * 
 * <p>The client is not allowed to gain access to the ITBClientEventListener trigger, Client events
 * are the sole responsibility of the ForPrize Module.</p>
 * 
 * <p>This Interface Provides for Module ---> Client Communication.  For more information about
 * Client ---> Module communication, see <i>ITBClientControls</i>.</p>
 *  
 * @author E Scott Cragg. Twistbox Client Core Engineering.
 *
 * and Tiago Teixeira. Twistbox Client Core Engineering.
 */
class ITBClientEventListener {

	/**
	 * <p>A Method for the client to implement (possibly within multiple client classes), to recieve
	 * updates and events from the Module.</p>
	 * 
	 * <p>For More information about the specific events that can be passed, see the CLIENT_ events 
	 * in Interface TBC.</p>
	 * 
	 * @param code See documentation for TBC
	 * @param data See documentation for TBC 
	 * @return true if the event is known and handled.  false otherwise.
	 */
public:
	virtual bool notifyClient(int code, Bbyte data) = 0;
	
};

#endif// _ITB_CLIENT_EVENT_LISTENER_H_
