/*=============================================================================

  Atlas Mobile Tournament Platform

  Copyright (c) 2003-2004 Atlas Mobile, Inc. All Rights Reserved.

  This software is the confidential and proprietary information of Atlas Mobile
  ("Confidential Information").  You shall not disclose such Confidential
  Information and shall use it only in accordance with the terms of the license
  agreement you entered into with Atlas Mobile.

  ATLAS MOBILE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
  NONINFRINGEMENT. ATLAS MOBILE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
  LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
  DERIVATIVES.

=============================================================================*/

#include "AMTPTournament.h"

//-----------------------------------------------------------------------------
AMTPTournament::AMTPTournament()
	: _descriptionStr(NULL)
	, _prizeStr(NULL)
	, _statusStr(NULL)
	, _pSeed(NULL)
	, _lenSeed(0)
	, _pParam(NULL)
	, _lenParam(0)
	, _dateStart(0)
	, _dateEnd(0)
{
	_id[0] = '\0';

	_prizeBitmapInfo.bitmap = NULL;
	_prizeBitmapInfo.memAlloc = FALSE;
}
//-----------------------------------------------------------------------------
AMTPTournament::AMTPTournament(const AMTPTournament& copy)
{
	CopyID(copy._id);
	CopySeed(copy._pSeed, copy._lenSeed);
	CopyParam(copy._pParam, copy._lenParam);
	_dateStart = copy._dateStart;
	_dateEnd = copy._dateEnd;
}
//-----------------------------------------------------------------------------
AMTPTournament& AMTPTournament::operator=(const AMTPTournament& copy)
{
	CopyID(copy._id);
	CopySeed(copy._pSeed, copy._lenSeed);
	CopyParam(copy._pParam, copy._lenParam);
	_dateStart = copy._dateStart;
	_dateEnd = copy._dateEnd;
	return *this;
}
//-----------------------------------------------------------------------------
AMTPTournament::~AMTPTournament()
{
	Reset();
}
//-----------------------------------------------------------------------------
void AMTPTournament::Reset()
{
	_id[0] = '\0';
	if (_descriptionStr)
	{
		MEMFREE(_descriptionStr);
		_descriptionStr = NULL;
	}
	if (_prizeStr)
	{
		MEMFREE(_prizeStr);
		_prizeStr = NULL;
	}

	TBUTIL_ReleaseBitmap(&_prizeBitmapInfo);

	if (_statusStr)
	{
		MEMFREE(_statusStr);
		_statusStr = NULL;
	}
	if (_pSeed)
	{
		MEMFREE(_pSeed);
		_pSeed = NULL;
		_lenSeed = 0;
	}
	if (_pParam)
	{
		MEMFREE(_pParam);
		_pParam = NULL;
		_lenParam = 0;
	}
}


char * AMTPTournament::GetDescription() 
{ 
	return _descriptionStr; 
}

char * AMTPTournament::GetPrizeStr() 
{ 
	return _prizeStr; 
}

ConvertedBitmapInfo * AMTPTournament::GetPrizeImg() 
{ 
	return &_prizeBitmapInfo; 
}

char * AMTPTournament::GetStatusStr() 
{ 
	return _statusStr; 
}

byte * AMTPTournament::GetSeed(int32* lenSeed) 
{ 
	*lenSeed = _lenSeed; 
	return _pSeed; 
}


//-----------------------------------------------------------------------------
void AMTPTournament::CopyID(const char* pID)
{
	if (!pID)
	{
		_id[0] = '\0';
	}
	else
	{
		STRNCPY(_id, pID, LEN_ID-1);
		_id[LEN_ID-1] = '\0';
	}
}
//-----------------------------------------------------------------------------
void AMTPTournament::CopyDescription(const char* descriptionStr, int32 len)
{
	if (_descriptionStr) {
		MEMFREE(_descriptionStr);
		_descriptionStr = NULL;
	}

	if (!descriptionStr) return;

	ASSERT((uint32)len == STRLEN(descriptionStr));

	if (len > 0)
	{
		_descriptionStr = (char*)MEMALLOC(len+1);
		if (_descriptionStr)
		{
			STRNCPY(_descriptionStr, descriptionStr, len);
			_descriptionStr[len] = '\0';
		}
	}
}
//-----------------------------------------------------------------------------
void AMTPTournament::CopyPrizeStr(const char* prizeStr, int32 len)
{
	if (_prizeStr) {
		MEMFREE(_prizeStr);
		_prizeStr = NULL;
	}

	if (!prizeStr) return;

	ASSERT((uint32)len == STRLEN(prizeStr));

	if (len > 0)
	{
		_prizeStr = (char*)MEMALLOC(len+1);
		if (_prizeStr)
		{
			STRNCPY(_prizeStr, prizeStr, len);
			_prizeStr[len] = '\0';
		}
	}
}
//-----------------------------------------------------------------------------
void AMTPTournament::CreatePrizeImg(byte* rawImgData, int32 rawImgSize)
{
	TBUTIL_ReleaseBitmap(&_prizeBitmapInfo);
	TBUTIL_CreateBitmap(&_prizeBitmapInfo, rawImgData, rawImgSize);
}
//-----------------------------------------------------------------------------
void AMTPTournament::CopyStatusStr(const char* statusStr, int32 len)
{
	if (_statusStr) {
		MEMFREE(_statusStr);
		_statusStr = NULL;
	}

	if (!statusStr) return;

	ASSERT((uint32)len == STRLEN(statusStr));

	if (len > 0)
	{
		_statusStr = (char*)MEMALLOC(len+1);
		if (_statusStr)
		{
			STRNCPY(_statusStr, statusStr, len);
			_statusStr[len] = '\0';
		}
	}
}
//-----------------------------------------------------------------------------
void AMTPTournament::CopySeed(const byte* pSeed, int32 lenSeed)
{
	if (pSeed) {
		MEMFREE(_pSeed);
		_pSeed = NULL;
	}

	_lenSeed = 0;

	if (lenSeed > 0)
	{
		_pSeed = (byte*)MEMALLOC(lenSeed);
		if (_pSeed)
		{
			MEMCPY(_pSeed, pSeed, lenSeed);
			_lenSeed = lenSeed;
		}
	}
}

//-----------------------------------------------------------------------------
#ifdef PARAM_IS_BINARY
void AMTPTournament::CopyParam(const byte* pParam, int32 lenParam)
{
	MEMFREE(_pParam);
	_pParam = NULL;
	_lenParam = 0;

	if (lenParam > 0)
	{
		_pParam = (byte*)MEMALLOC(lenParam);
		if (_pParam)
		{
			MEMCPY(_pParam, pParam, lenParam);
			_lenParam = lenParam;
		}
	}
}
#else
void AMTPTournament::CopyParam(const char* pParam, int32 lenParam)
{
	MEMFREE(_pParam);
	_pParam = NULL;
	_lenParam = lenParam;

	if (!pParam) return;

	int len = STRLEN(pParam);
	if (len > 0)
	{
		_pParam = (char*)MEMALLOC(len+1);
		if (_pParam)
		{
			MEMCPY(_pParam, pParam, len);
			_pParam[len] = '\0';
		}
	}
}
#endif //PARAM_IS_BINARY
