#ifndef _FOR_PRIZES_ADAPTER_H_
#define _FOR_PRIZES_ADAPTER_H_

#include "AEE.h"
#include "AEEappGen.h"

#include "..\..\Core\ITBStringFactory.h"
#include "..\..\Std\InputStream.h"
//#include "..\..\Std\OutputStream.h"
#include "ForPrizesAdapterConstantDefinitions.h"
#include "ForPrizeTestbed.h"
#include "TBModuleEvents.h"
#include "..\..\..\src\aImage.h"
#include "..\..\..\src\BrewString.h"
#include "..\..\..\src\BrewVector.h"
#include "..\..\Std\Helpers.h"

#define DATASIZE 3

class ForPrizesAdapter
{
public:

	//int requestResponse;
	/**
	 * Record Store Names
	 */
	String _accountRecordName;
	String _billingRecordName;
	
	/**
	 * Specifies which carrier we are dealing with
	 */
	byte carrierID;

	/**
	 * Specifies which carrier we are dealing with
	 */
	String _countryCode;

	/**
	 * Specifies tournament server
	 */
	String _serverURL;

	/**
	 * Subscription time defaults to 'now'. If the subscription time was different
	 * some place, the server or RMS will re-populate it.
	 */
	long _subscriptionStartTime;

	/**
	 * True while the subscription process is active
	 */
	bool _isNewSubscription;

	/**
	 * The subscription price - a string which should populated by the server or JAD file
	 */
	//static String* _subscriptionPrice;

	/**
	 * Subscription length in ms. Subscription time defaults to 60 days when
	 * the application starts.  If the subscription is different some place,
	 * the server or RMS will re-populate it.
	 * <p/>
	 * (24 hr / 1 day) * (60 min / hr) * (60 s / min) * (1000 ms / s) = ms / day
	 * Setting to 100000 days for now since we're not using the subscription model currently
	 */
	//static long _subscriptionLength;

	/**
	 * only relevant for T-mobile subscription which requires a premium SMS.
	 * This is the short code that verifies the user.
	 */
	String _subscriptionRequestURL;// = "+16503059625:test";

	/**
	 * Error number returned in last response
	 */
	uint32 _errorNum;

	/**
	 * Error string returned in last response
	 */
	String _errorStr;

	/**
	 * Unique mobile information number of user's phone
   * Could set to "05" for T-Mobile + "001" for USA = 05001 + 10 digit phone number
	 */
	String min;

	/**
	 * Certification received after grant from Cingular
	 */
	String _cert;

	/**
	 * User name
	 */
	String _userName;

	/**
	 * Used when registering player
	 */
	String _phoneType;

	/**
	 * User's current SMS opt-in preference
	 */
	bool _isSMSOptIn;

	/**
   * The pin user entered.  Used only in PIN entering stage for T-Mobile USA.
	 */
	String _PIN;

	/**
	 * Game product id
	 */
	int32 _productID;

	/**
	 * Game type
	 */
	int16 _gameType;

	/**
	 * Game name
	 */
	String _gameName;

	/**
	 * Game version
	 */
	String _gameVersion;

	/**
	 *	Promo ID
	 */
	int32 _promoID;
	
	/**
	 * Type of tournament currently being accessed
	 * (PARAM_TOURNAMENT_TYPE_PRACTICE, PARAM_TOURNAMENT_TYPE_H2H, PARAM_TOURNAMENT_TYPE_PROGRESSIVE)
	 */
	byte _currentTournamentType;

	/**
	 * which tournament is being played for each tournament type. _currentTournamentIndex[_enteredTournamentIndex]
	 */
	byte _currentTournamentIndex[PARAM_TOURNAMENT_TYPE_COUNT];

	int _percentComplete;
	class ForPrizeTestbed* _fpc;
	bool hasMin;

	// **************************************************************************
	// Protected fields
public:

	/**
	 * Host MIDlet
	 *
	 * @noinspection FieldCanBeLocal
	 */
	AEEApplet* _midlet;

	/**
	 * 0 head to head, 1 progressive
	 */
	byte _enteredTournamentIndex;

	/**
	 * Match results
	 */
	String _matchSummary;

	/**
	 * Ranking summary
	 */
	String _rankSummary;

	/**
	 * Scores summary
	 */
	String _scoreSummary;

	/**
	 * Prompt message (error result)
	 */
	String _promptMsg;

	/**
	 * List of tournament history headers
	 */
	VoidVector _tournamentHistoryHeaders;

	/**
	 * List of tournament history summaries
	 */
	VoidVector _tournamentHistorySummaries;

	int _assetClass;

	/**
	 * Score of last game played
	 */
	long _lastGameScore;

	/**
	 * Time last game started
	 */
	long _lastGameStartTime;

	/**
	 * Time last game ended
	 */
	long _lastGameEndTime;

	/**
	 * Results data blob from last game played
	 */
	Bbyte _lastGameResultsData;

	/**
	 * Flag indicating that flow should skip to registration
	 */
	bool _skipToRegister;

	/**
	 * ID[0] = head to head. _ID[0][1] = second head to head game. _ID[0][1][0] = id number of second head to head game
	 */
	int _ID[PARAM_TOURNAMENT_TYPE_COUNT][PARAM_TOURNAMENT_CACHE_SIZE][2];

	/**
	 * [0] name, [1] description, [2] statusStr
	 */
	String _text[PARAM_TOURNAMENT_TYPE_COUNT][PARAM_TOURNAMENT_CACHE_SIZE][3];

	/**
	 * [0] start, [1] end
	 */
	long _time[PARAM_TOURNAMENT_TYPE_COUNT][PARAM_TOURNAMENT_CACHE_SIZE][2];

	/**
	 * [0][] seed, [1][] params
	 */
	byte _data[PARAM_TOURNAMENT_TYPE_COUNT][PARAM_TOURNAMENT_CACHE_SIZE][2][3];

	/**
	 * promo notification text
	 */
	String _promoText;

	/**
	 * promo notification image
	 */
	//Bbyte _promoImgBuffer;

	/**
	 * promo image
	 */
	//Image* _promoImage;
	ConvertedBitmapInfo _promoImage;
	/**
	 * Winner notification text
	 */
	String _winnerText;

	/**
	 * Winner notification image
	 */
//	Bbyte _winnerImgBuffer;

	/**
	 * Winner image
	 */
	ConvertedBitmapInfo _winnerImage;

	/**
	 * News/promo text
	 */
	String _newsText;

	/**
	 * News/promo image
	 */
//	Bbyte _newsImgBuffer;

	/**
	 * News image
	 */
	ConvertedBitmapInfo _newsImage;

	/**
	 * [0] h2h, [1] progressive, Current prize description
	 */
	String _prizeText[PARAM_TOURNAMENT_TYPE_COUNT];

	/**
	 * [0] h2h, [1] progressive, Current prize image
	 */
	//byte _prizeImgBuffer[PARAM_TOURNAMENT_TYPE_COUNT][3];

	/**
	 * Prize image
	 */
	ConvertedBitmapInfo _prizeImage[2];

	/**
	 * Carrier adapter
	 */
private:
	void* carrierAdapter;

	/**************************************************************************/
	/************************** Methods ***************************************/
	/**************************************************************************/

	/**
	 * ForPrizesAdapter Constructor
	 * @param module TODO
	 * @param midlet	 the host MIDlet instance
	 * @param game		 the game ID
	 * @param gameName the game name
	 * @param version	the game version
	 * @param adapter	the CarrierAdapter
	 */
public:
	ForPrizesAdapter(void* module, ITBStringFactory* strings, AEEApplet* midlet, String fpCarrier, String fpProductID, String fpProductVersion, String fpServerURL, String fpAssetClass, String fpMin, String fpCountryCode, String fpSubPrice, String fpSubRequestURL, String fpAccountRecordName, String fpBillingRecordName, String fpReg, String fpSms);

	void destroy() {
		release();
	};

	/**
	 * Release all resources. Forces the request thread, network
	 * buffers, image, and text to free.
	 */
	void release();

	/**
	 * Gets the most recent error number that was sent down from the server.
	 *
	 * @return int Current error number
	 */
	uint32 getErrorNum() {
		return _errorNum;
	};

	/**
	 * Sets the error number to PARAM_ERR_NONE
	 */
	void clearError() {
		_errorNum = PARAM_ERR_NONE;
		_errorStr = "";
	};

	/**
	 * Gets the most recent error string that was sent down from the server.
	 *
	 * @return int Current error string
	 */
	String getErrorStr() {
		return _errorStr;
	};

	/**
	 * Gets product name.
	 *
	 * @return String Name of "for prizes" product
	 */
	String getProductName() {
		return _gameName;
	};
	
	/**
	 * Get promo ID
	 *
	 */
	int32 getCurrentPromoID()
	{
	   return _promoID;
	};

	
	/**
	 * Set promo ID
	 *
	 */
	void setPromoID(int32 id)
	{
	   _promoID = id;	 
	};

	/**
	 * Gets progress of network transactions
	 *
	 * @return int percentage complete of network transactions
	 */
	int getProgress() {
		return _percentComplete;
	};

	/**
	 * Gets subscription length for use when displaying subscription info to user.
	 *
	 * @return Long Length of subscription.
	 */
	//long getSubscriptionLength() {
	//	return _subscriptionLength;
	//};

	//-----------------------------------------------------------------------------------


	/**
	 * Gets subscription price for use when displaying subscription info to user.
	 *
	 * @return Long Length of subscription.
	 */
	//String getSubscriptionPrice() {
	//	return *_subscriptionPrice;
	//};

	//-----------------------------------------------------------------------------------


	/**
	 * Returns the SMS Option
	 *
	 * @return boolean _isSMSOptin Whether or not user wants server generated SMS
	 */
	bool isSMSOptIn() {
		return _isSMSOptIn;
	};

	bool _hasNextTournaments;
	bool ValidateTournament();

	/**
	 * Sets the currently entered tournament to the type specified (Head to Head or Progressive).
	 * Should be called when wants to play tournament game.
	 * Called internally.
	 *
	 * @param tournamentType int Type of tournament (Head to Head or Progressive)
	 */
	void setCurrentTournamentType(byte tournamentType);

	/**
	 * Given a phone number, return a correctly-formatted MIN
	 *
	 * @param phoneNumber
	 * @return a MIN
	 */
	String toMIN(String phoneNumber);

	/**
	 * Sets the _min. Checks to make sure the MIN is 15 digits.
	 *
	 * @param MIN String 15 Digit MIN<br>
	 *            Should be of form:<br>
	 *            (2 digit carrier ID) + (3 digit country code) + (10 digit phone number)
	 */
	void setMIN(String _MIN);

	/**
	 * Return the current MIN
	 * @return
	 */
	String getMIN();

	/**
	 * Sets the phone identifier string - a very short description of the device.
	 *
	 * @param phone String Description of device
	 */
	void setPhoneType(String phone);

	/**
	 * Set the registration state for this client.
	 *
	 * @param state byte possible values = {PARAM_REGISTER_PHONE_NEEDED,
	 *              PARAM_REGISTER_PIN_NEEDED, PARAM_REGISTER_PIN_VALIDATED, PARAM_REGISTER_DONE}
	 */
	void setRegisterState(byte state);

	/**
	 * Set whether or not user wants to receive server generated SMS news.
	 *
	 * @param in boolean
	 */
	void setSMSOptIn(bool in) {
		_isSMSOptIn = in;
		//savePreferences();
	};


	/**
	 * Get whether or not user wants to receive server generated SMS news.
	 */
	bool getSMSOptIn() {
		return _isSMSOptIn;
	};

	/**
	 * Subscription length should ideally be read in from the .jad and then set using this method upon start of app.
	 *
	 * @param ms long Subscription Length in ms
	 */
//	void setSubscriptionLength(long ms);

	/**
	 * Sets the _username. Checks to make sure that the username is at least 3 characters long.
	 *
	 * @param username String the user's name in the system
	 */
	void setUsername(String username);

	/**
	 * Aborts current network transaction
	 */
	void abortTransaction();

	/**
	 * Load preferences from RMS
	 *
	 * @return success (true) or failure (false)
	 */
	bool loadPreferences();

	/**
	 * Saves preferences to RMS
	 *
	 * @return success (true) or failure (false)
	 */
	bool savePreferences();

	/**
	 * Save subscription info into RMS
	 *
	 * @return success (true) or failure (false)
	 */
	//bool saveSubscription();

	/**
	 * Returns the current tournament type
	 *
	 * @return byte Current tournament type
	 */
	byte getCurrentTournamentType() {
		return _currentTournamentType;
	};

	/**
	 * Return the tournament ID for the active tournament of the given tournament type
	 *
	 * @param tType
	 * @return int current tournament ID for the given tournament type
	 */
	int getCurrentTournamentID(int tType) {
		return _ID[_enteredTournamentIndex][_currentTournamentIndex[_enteredTournamentIndex]][PARAM_TOURNAMENT_ID_IDX];
	};

	/**
	 * Return the message sequencing number for the active tournament of the given tournament type
	 *
	 * @param tType
	 * @return int current message sequence number for the given tournament type
	 */
	int getCurrentMessageSequenceNumber(int tType) {
		return _ID[_enteredTournamentIndex][_currentTournamentIndex[_enteredTournamentIndex]][PARAM_TOURNAMENT_SEQUENCE_IDX];
	};

	/**
	 * Return the name of the active tournament of the given tournament type
	 *
	 * @param tType
	 * @return String name of the current tournament for the given type
	 */
	String getCurrentTournamentName(int tType) {
		return _text[_enteredTournamentIndex][_currentTournamentIndex[_enteredTournamentIndex]][PARAM_TOURNAMENT_NAME_IDX];
	};

	/**
	 * Return a description of the active tournament of the given tournament type
	 *
	 * @param tType
	 * @return String description of the current tournament for the given type
	 */
	String getCurrentTournamentDescription(int tType) {
		return _text[_enteredTournamentIndex][_currentTournamentIndex[_enteredTournamentIndex]][PARAM_TOURNAMENT_DESCRIPTION_IDX];
	};

	/**
	 * Return the status message for the active tournament of the given tournament type
	 *
	 * @param tType
	 * @return String status text for the current tournament of the given type
	 */
	String getCurrentTournamentStatus(int tType) {
		return _text[_enteredTournamentIndex][_currentTournamentIndex[_enteredTournamentIndex]][PARAM_TOURNAMENT_STATUS_IDX];
	};

	/**
	 * Return the start time for the active tournament of the given tournament type
	 *
	 * @param tType
	 * @return long start time for the current tournament of the given type
	 */
	long getCurrentTournamentStartTime(int tType) {
		return _time[_enteredTournamentIndex][_currentTournamentIndex[_enteredTournamentIndex]][PARAM_TOURNAMENT_START_TIME_IDX];
	};

	/**
	 * Return the end time for the active tournament of the given tournament type
	 *
	 * @param tType
	 * @return long endtime for the current tournament of the given type
	 */
	long getCurrentTournamentEndTime(int tType) {
		return _time[_enteredTournamentIndex][_currentTournamentIndex[_enteredTournamentIndex]][PARAM_TOURNAMENT_END_TIME_IDX];
	};

	/**
	 * Returns the seed sent from the server. For H2H tournaments, this is a common
	 * random seed. To guarantee fairness, both players who compete H2H will receive the
	 * same random seed so that the random tiles, cards, etc. generated have the same sequence.
	 * This value should be stored in the client after a successful enter/getTournament call
	 * & is generated by the server. The application developer should use this random seed when
	 * the game starts to generate all of its random values. Appends tournament parameters
	 * (sent by server) into referenced StringBuffer
	 *
	 * @param params StringBuffer params A referenced data structure to be
	 *               populated with tournament parameters provided by the server
	 * @return byte[] a seed: the random seed byte array received by the server.
	 */
	//byte* getGameSeed(String params);
	void getGameSeed(const byte** seed, int32* seedLen, const char ** params, int32* paramsLen);

	/**
	 * Returns the match results from the most recent H2H match.
	 *
	 * @return String _matchSummary The match reuslts.
	 */
	String getMatchSummary();

	/**
	 * Returns the most current score summary.
	 *
	 * @return String The most current score summary.
	 */
	String getScoreSummary();

	/**
	 * Returns the most current rank summary.
	 *
	 * @return String The most current rank summary.
	 */
	String getRankSummary();

	/**
	 * Returns the most current prompt text.
	 *
	 * @return String The most current prompt text.
	 */
	String getPromptText() {
		return _promptMsg;
	};

	/**
	 * Gets winner image.
	 *
	 * @return Image
	 */
	ConvertedBitmapInfo* getWinnerImage();

	/**
	 * Returns the winner display text
	 *
	 * @return String Winner Screen Text
	 */
	String getWinnerText() {
		return _winnerText;
	};

	/**
	 * Gets promo image.
	 *
	 * @return Image
	 */
	ConvertedBitmapInfo* getPromoImage();

	/**
	 * Returns the promo display text
	 *
	 * @return String Promo Screen Text
	 */
	String getPromoText() {

		return _promoText;
	};

	/**
	 * Returns the news image
	 *
	 * @return Image News Screen Image
	 */
	ConvertedBitmapInfo* getNewsImage();

	/**
	 * Returns the news display text
	 *
	 * @return String News Screen Text
	 */
	String getNewsText() {
		return _newsText;
	};

	/**
	 * Gets prize icon for the selected tournament type
	 *
	 * @param tournamentType
	 * @return Image
	 */
	ConvertedBitmapInfo* getPrizeImage(int tournamentType);

	String getPrizeText(int tournamentType);
	/**
	 * Returns list of currently entered tournaments
	 *
	 * @return Vector _tournamentHistoryHeaders
	 */
	String getTournamentHistoryHeader(int idx);

	/**
	 * Returns summaries of currently entered tournaments
	 *
	 * @return Vector _tournamentHistorySummaries
	 */
	String getTournamentHistorySummary(int idx);

	/**
	 * Return whether or not the given username is valid.
   * Does not take in to account obscene language or existing usernames.
   * This method only considers the length of the username.
	 *
	 * @return boolean
	 */
	bool isValidUsername(String name) {
		return (name.equals("")) && (name.trim().length() > 3) && (name.trim().length() < 12);
	};

	/**
	 * Return whether or not the given registration PIN is valid.
   * Does not validate correctness of PIN, only length.
	 *
	 * @return boolean
	 */
	bool isValidRegistrationPIN(String pin) {
		return (pin.equals("")) && (pin.trim().length() >= 4);
	};

	/**
   * Return whether or not the given registration phone number is valid.
   * Only considers length of phone number for validity. Must be 10 digits.
	 *
	 * @return boolean
	 */
	bool isValidPhoneNumber(String phone) {
		return (phone.equals("")) && (phone.trim().length() == 10);
	};

	/**
	 * Return whether or not the user must enter their phone number during
	 * the registration process.  The default implementation returns false,
	 * but some (e.g., T-Mobile USA) will return true.
	 *
	 * @return boolean
	 */
	bool isPhoneNeededForRegistration();

	/**
	 * Checks for the registration status of the player before network
	 * transactions.  It will give back the registration status of client.
	 * Should be invoked before network transactions to tell client whether
	 * or not it needs to register.
	 * Returns either:<br>
	 * PARAM_REGISTER_NOT_DONE,
	 * PARAM_REGISTER_PIN_NEEDED,
	 * PARAM_REGISTER_PIN_VALIDATED, or
	 * PARAM_REGISTER_DONE
	 *
	 * @return byte Registration state
	 */
	byte getRegistrationState() ;

	/**
	 * Returns the number of milliseconds till the users subscription ends.
	 * Convenient to put this info in the about section.
	 *
	 * @return long milliseconds
	 */
	//long getSubscriptionTimeLeft();

	/**
	 * Should be invoked when the splash screens are over so a subscription check can be done. Returns either: PARAM_SUBSCRIPTION_EXPIRED, PARAM_SUBSCRIPTION_NEED_CONFIRM, or PARAM_SUBSCRIPTION_OK.
	 * If subscription is expired, then user should be prompted to resubscribe.
	 * If subscription needs confirmation, then subscription data needs to be sent to our servers again.
	 * If the subscription is ok, then advance to main menu.
	 *
	 * @return byte One of 3 values corresponding to the subscription status. Either PARAM_SUBSCRIPTION_EXPIRED,
	 *         PARAM_SUBSCRIPTION_NEED_CONFIRM, or PARAM_SUBSCRIPTION_OK
	 */
	byte checkSubscription();

	/**
	 * Sends a subscription request, makes note of the re-subscription time
	 * (now), and saves subscription data using SaveSubscription().<br>
	 * Should be invoked after user has given permission to resubscribe.
	 * Does carrier specific resubscribe functionality.
	 * Should be invoked after getSubscriptionInfo(true);
	 */
	void resubscribe();

	/**
	 * Retrieves the subscription information for the application.
	 *
	 * @param confirm boolean true if confirming subscription. false if
	 *                checking subsctiption.
	 * @return PARAM_ERR_NONE if the request was successfully generated and sent to the
	 *         server and the response was successfully received and processed.
	 *         If a failure occurred, an error code will be returned. Error codes
	 *         can either generated locally by the client or network layer or
	 *         remotely by the AMTP server.
	 * @throws Exception if a catastrophic error occurred sending the request or
	 *                   processing the response.
	 */
	int getSubscriptionInfo(bool confirm);

	/**
	 * Notification method that marks the start time of the game. (important for score reporting)
	 * Should be called by client upon start of game.
	 */
	void gameStarted();

	/**
	 * Gets the current H2H & progressive tournaments and their associated data (news, winner, prize info & images)
	 * from the AMTP server.
	 * Should be called when user wants to play a tournament game, but before any information is displayed.
	 *
	 * @return PARAM_ERR_NONE if the request was successfully generated and sent to the
	 *         server and the response was successfully received and processed.
	 *         If a failure occurred, an error code will be returned. Error codes
	 *         can either generated locally by the client or network layer or
	 *         remotely by the AMTP server.
	 * @throws Exception if a catastrophic error occurred sending the request or
	 *                   processing the response.
	 */
	int getTournaments();

	/**
	 * Should be called when starting a tournament game in order to get matched up
	 * and to let the server know that user is entered in this tournamnet.
	 *
	 * @param tournamentType byte The current tournament type which we are entering.
	 *                       Either PARAM_TOURNAMENT_TYPE_H2H or PARAM_TOURNAMENT_TYPE_PROGRESSIVE
	 * @return int Returns PARAM_ERR_NONE if successful
	 */
	int enterTournament(byte tournamentType);

	/**
	 * Used to get scores from AMTP at any time after registration.
	 *
	 * @param tournamentType Type of tournament. Value can be one of <ul>
	 *                       <li>PARAM_TOURNAMENT_TYPE_H2H</li>
	 *                       <li>PARAM_TOURNAMENT_TYPE_PROGRESSIVE</li></ul>
	 * @param scoreType			Type of score. Value can be one of <ul>
	 *                       <li>PARAM_SCORES_TYPE_MY_CURRENT_RANK</li>
	 *                       <li>PARAM_SCORES_TYPE_MY_RECENT_MATCHES</li>
	 *                       <li>PARAM_SCORES_TYPE_MY_TOURNAMENTS</li>
	 *                       <li>PARAM_SCORES_TYPE_GLOBAL_RANKS</li></ul>
	 * @return PARAM_ERR_NONE if the request was successfully generated and sent to the
	 *         server and the response was successfully received and processed.
	 *         If a failure occurred, an error code will be returned. Error codes
	 *         can be either generated locally by the client or network layer or
	 *         remotely by the AMTP server.
	 * @throws Exception if a catastrophic error occurred sending the request or
	 *                   processing the response.
	 */
	int getScores(byte tournamentType, int scoreType);

	/**
	 * Register the user with the AMTP server.  Preset:
	 * <p/>
	 * <ul>
	 * <li> username
	 * <li> phoneType
	 * <li> forceRegister
	 * <li> smsOptin
	 * <li> PIN (only used by T-Mobile)
	 * </ul>
	 *
	 * @return PARAM_ERR_NONE if the request was successfully generated and sent to the
	 *         server and the response was successfully received and processed.
	 *         If a failure occurred, an error code will be returned. Error codes
	 *         are either generated locally by the client or network layer or
	 *         remotely by the AMTP server.
	 * @throws Exception if a catastrophic error occurred sending the request or
	 *                   processing the response.
	 */
	int registerPlayer();

	/**
	 * The game created by the developer should invoke this function
	 * when the final score is known.  This will trigger the start of the
	 * network report score call.  This can be called during the high score
	 * display screen of the game for example so that the actual network
	 * delay can be somewhat hidden from the user.
	 * Used to report the last games score that was stored in the client.
	 *
	 * @param score			 int The final score
	 * @param gameResults byte[] Any game specific data to send.
	 */
	int reportGameScore(int score, byte gameResults[]);

	/**
	 * Get promo data from server
	 * @param tournamentType			
	 * @param id
	 */
	int sendGetPromoRequest(int tournamentType, int id);

	/**
	 * Sends SMS option to server
	 *
	 * @return transaction error status
	 */
	int setOptions();

	/**
	 * Parse tournment quarks from the provided data stream.
	 *
	 * @param tournamentType tournament Type
	 * @param dis						DataInputStream to read quarks from.
	 * @throws Exception
	 */
	//void parseTournamentQuarks(int tournamentType, InputStream dis);

	int getCurrentTournamentIndex(int tType) {
		return ((_currentTournamentIndex[tType - 1] + 1) % PARAM_TOURNAMENT_CACHE_SIZE);
	};

	/**
	 * Load a record store and return the data as a byte array.
	 *
	 * @param name Name of the record store (no more than 32 characters)
	 * @return record store data, or null if the record store could not be found
	 */
	Bbyte getRecord(String name);

	/**
	 * Save a record store. <br>
	 * This will never return an error!
	 *
	 * @param rsName Name of the record store to save to
	 * @param data	 Data to store
	 */
	void putRecord(String rsName, byte data[]);

	bool									_outOfSeeds;

	bool DisplayWinner() ;
	bool DisplayNews(bool clearNews) ;
	bool									_hasNews;

	bool _isGettingScore;
	bool CancelWebRequests();


};

#endif _FOR_PRIZES_ADAPTER_H_
