/*=============================================================================

  Atlas Mobile Tournament Platform

  Copyright (c) 2003-2004 Atlas Mobile, Inc. All Rights Reserved.

  This software is the confidential and proprietary information of Atlas Mobile
  ("Confidential Information").  You shall not disclose such Confidential
  Information and shall use it only in accordance with the terms of the license
  agreement you entered into with Atlas Mobile.

  ATLAS MOBILE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
  NONINFRINGEMENT. ATLAS MOBILE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
  LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
  DERIVATIVES.

=============================================================================*/

#include "IOStreamBuf.h"

#if 0
#define DEBUGBUF(expr) DEBUG(expr)
#else
#define DEBUGBUF(expr)
#endif

#if 0
#define DEBUGI64(expr) DEBUG(expr)
#else
#define DEBUGI64(expr)
#endif

//-----------------------------------------------------------------------------
IOStreamBuf::IOStreamBuf()
	: _pcBuf(NULL)
	, _offset(0)
	, _length(0)
	, _max(0)
{
}

//-----------------------------------------------------------------------------
bool IOStreamBuf::Start(uint32 bufsize)
{
	if (_pcBuf && _max<bufsize)
	{
		Stop();
	}
	if (!_pcBuf) _pcBuf = (uint8*)MEMALLOC(bufsize);
	if (!_pcBuf)
	{
		ASSERT(!("COULD NOT ALLOCATE IOSTREAMBUF"));
		return false;
	}
	_offset = 0;
	_length = 0;
	_max = bufsize;

	DEBUG(( "IOStreamBuf::Start(): head=0x%08x, end=0x%08x",_pcBuf,_pcBuf+bufsize ));

	return true;
}
//-----------------------------------------------------------------------------
void IOStreamBuf::Stop()
{
	TRACE(( "IOStreamBuf::Stop()" ));

	if (_pcBuf)
	{
		MEMFREE(_pcBuf);
		_pcBuf = NULL;
	}
	_offset = 0;
	_length = 0;
	_max = 0;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::Seek(uint32 offset)
{
	DEBUGBUF(( "IOStreamBuf::Seek(%d)", offset ));

	ASSERT(_pcBuf); ASSERT(_length <= _max); ASSERT(_offset <= _length);

	if (_pcBuf && offset<_length)
	{
		_offset = offset;
		return true;
	}
	return false;
}

//-----------------------------------------------------------------------------
//	INPUT
//-----------------------------------------------------------------------------
bool IOStreamBuf::ReadBytes(uint8* data, int len)
{
	if (!len) return true;		//this is ok, it says do nothing
	if (!data) return false;	//this is an error because len!=0 at this point
	ASSERT(_pcBuf); ASSERT(_length <= _max); ASSERT(_offset <= _length);

	*data = 0;	//init w/terminator

	PACKED char* ptr = (PACKED char*)_pcBuf + _offset;
	PACKED char* end = (PACKED char*)_pcBuf + _length;

	while (len-- && ptr<end)
	{
		*data++ = *ptr++;
		_offset++;
	}
	return true;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::ReadChars(char* data)
{
	ASSERT(data);
	ASSERT(_pcBuf); ASSERT(_length <= _max); ASSERT(_offset <= _length);
	if (!data) return false;

	*data = 0;	//init w/terminator

	PACKED char* ptr = (PACKED char*)_pcBuf + _offset;
	PACKED char* end = (PACKED char*)_pcBuf + _length;

	while (ptr < end)
	{
		*data++ = *ptr++;
		_offset++;
		if (!*ptr) break; //found null terminator?
	}
	return true;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::ReadBool(bool& data)
{
	ASSERT(_pcBuf); ASSERT(_length <= _max); ASSERT(_offset <= _length);

	uint8 bByte = (uint8)data;
	return ReadUint8(bByte);

	/*
	if (_offset+sizeof(bool) <= _length)
	{
		data = *((bool*)(_pcBuf + _offset));
		if (sizeof(bool) == 2)
			NTOHS(data);
		else if (sizeof(bool) == 4)
			NTOHL(data);
		_offset += sizeof(bool);
		return true;
	}
	ASSERT(!("ATTEMPTED TO READ PAST END OF BUFFER"));
	Reset();
	return false;
	*/
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::ReadInt8(int8& data)
{
	ASSERT(_pcBuf); ASSERT(_length <= _max); ASSERT(_offset <= _length);

	if (_offset < _length)
	{
		data = *((PACKED int8*)(_pcBuf + _offset));
		_offset++;
		return true;
	}
	ASSERT(!("ATTEMPTED TO READ PAST END OF BUFFER"));
	Reset();
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::ReadUint8(uint8& data)
{
	ASSERT(_pcBuf); ASSERT(_length <= _max); ASSERT(_offset <= _length);

	if (_offset < _length)
	{
		data = *((PACKED uint8*)(_pcBuf + _offset));
		_offset++;
		return true;
	}
	ASSERT(!("ATTEMPTED TO READ PAST END OF BUFFER"));
	Reset();
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::ReadInt16(int16& data)
{
	ASSERT(_pcBuf); ASSERT(_length <= _max); ASSERT(_offset <= _length);

	if (_offset+sizeof(int16) <= _length)
	{
		int16 netdata = *((PACKED int16*)(_pcBuf + _offset));
		data = NTOHS(netdata);
		_offset += sizeof(int16);
		return true;
	}
	ASSERT(!("ATTEMPTED TO READ PAST END OF BUFFER"));
	Reset();
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::ReadUint16(uint16& data)
{
	ASSERT(_pcBuf); ASSERT(_length <= _max); ASSERT(_offset <= _length);

	if (_offset+sizeof(uint16) <= _length)
	{
		uint16 netdata = *((PACKED uint16*)(_pcBuf + _offset));
		data = NTOHS(netdata);
		_offset += sizeof(uint16);
		return true;
	}
	ASSERT(!("ATTEMPTED TO READ PAST END OF BUFFER"));
	Reset();
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::ReadInt32(int32& data)
{
	ASSERT(_pcBuf); ASSERT(_length <= _max); ASSERT(_offset <= _length);

	if (_offset+sizeof(int32) <= _length)
	{
		int32 netdata = *((PACKED int32*)(_pcBuf + _offset));
	//	MEMCPY((void*)&netdata, (void*)(_pcBuf + _offset), sizeof(int32));
		data = NTOHL(netdata);
		_offset += sizeof(int32);
		return true;
	}
	ASSERT(!("ATTEMPTED TO READ PAST END OF BUFFER"));
	Reset();
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::ReadUint32(uint32& data)
{
	ASSERT(_pcBuf); ASSERT(_length <= _max); ASSERT(_offset <= _length);

	if (_offset+sizeof(uint32) <= _length)
	{
		uint32 netdata = *((PACKED uint32*)(_pcBuf + _offset));
	//	MEMCPY((void*)&netdata, (void*)(_pcBuf + _offset), sizeof(uint32));
		data = NTOHL(netdata);
		_offset += sizeof(uint32);
		return true;
	}
	ASSERT(!("ATTEMPTED TO READ PAST END OF BUFFER"));
	Reset();
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::ReadInt64(int64& data)
{
	ASSERT(_pcBuf); ASSERT(_length <= _max); ASSERT(_offset <= _length);

	if (_offset+sizeof(int64) <= _length)
	{
		int32 d1, d2;
		ReadInt32(d1);
		ReadInt32(d2);

		//For little-endian, the 1st int32 is high, and the 2nd int32 is low.
#ifdef WIN32
		//MSVC++ can't handle int64<<32, so use pointers and write contents.
		//ptrs don't have to be packed b/c they point to an aligned var
		int32* pLo = (int32*)(&data);
		int32* pHi = pLo + 1; //little-endian means hi-order is higher
		*pHi = d1;
		*pLo = d2;
		DEBUGI64(( "ReadInt64(0x%08lx%08lx)",HTONL(d1),HTONL(d2) ));
		DEBUGI64(( "ReadInt64=0x%08lx%08lx",*pHi,*pLo ));
	//	DEBUGI64(( "ReadInt64=0x%016I64x", data ));
#else
		int64 hi = d1;
		int64 lo = d2;
		data = (hi << 32) | lo;
		ARMDEBUG(( "ReadInt64(0x%08lx%08lx)",HTONL(d1),HTONL(d2) ));
		ARMDEBUG(( "ReadInt64=0x%016llx", data ));
#endif

		return true;
	}
	ASSERT(!("ATTEMPTED TO READ PAST END OF BUFFER"));
	Reset();
	return false;
}
//-----------------------------------------------------------------------------
//	OUTPUT
//-----------------------------------------------------------------------------
bool IOStreamBuf::WriteBytes(const uint8* data, int len)
{
	DEBUGBUF(( "IOStreamBuf::WriteBytes(len=%d)", len ));

	/*
	#ifdef _DEBUG
			char chars[10], ch;
			char line[128];
			line[0] = '\0';
			for (int idx=0; idx<len; idx++)
			{
				ch = data[idx];
				SPRINTF(chars, "%02x", ch);
				STRCAT(line, chars);
			}
			DEBUGBUF(( "IOStreamBuf::WriteBytes(): %d bytes [%s]", len, line ));
	#endif
	*/

	if (!len) return true;		//this is ok, it says do nothing
	if (!data) return false;	//this is an error, since len!=0 at this point
	if (len<0) return false;
	ASSERT(_pcBuf); ASSERT(_length <= _max);

	if (_offset+len <= _max)
	{
		PACKED uint8* ptr = (PACKED uint8*)data;
		while (len--)
		{
			if (!WriteInt8(*ptr++))
				break;
		}
		if (len<0)
			return true;
	}
	ASSERT(!("ATTEMPTED TO OVERRUN END OF BUFFER"));
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::WriteChars(const char* data)
{
	DEBUGBUF(( "IOStreamBuf::WriteChars" ));

	int len = data ? STRLEN(data) : -1;
	if (!len) return true;		//this is ok, it says do nothing
	if (!data) return false;	//this is an error, since len!=0 at this point
	if (len<0) return false;
	ASSERT(_pcBuf); ASSERT(_length <= _max);

	if (_offset+len <= _max)
	{
		PACKED char* ptr = (PACKED char*)data;
		while (*ptr && WriteInt8(*ptr))
		{
			ptr++;
		}
		if (WriteInt8('\0')) //null terminator
			return true;
	}
	ASSERT(!("ATTEMPTED TO OVERRUN END OF BUFFER"));
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::WriteBool(bool data)
{
	DEBUGBUF(( "IOStreamBuf::WriteBool(0x%x)", data ));

	uint8 bByte = (uint8)data;
	return WriteUint8(bByte);

	/*
	ASSERT(_pcBuf); ASSERT(_length <= _max);

	if (_offset+sizeof(bool) <= _max)
	{
		if (sizeof(bool) == 2)
		{
			uint16 netdata = HTONS(data);
			MEMCPY((void*)(_pcBuf + _offset), (void*)&netdata, sizeof(bool));
		}
		else if (sizeof(bool) == 4)
		{
			uint32 netdata = HTONL(data);
			MEMCPY((void*)(_pcBuf + _offset), (void*)&netdata, sizeof(bool));
		}
		else
		{
			MEMCPY((void*)(_pcBuf + _offset), (void*)&data, sizeof(bool));
		}
		_offset += sizeof(bool);
		if (_offset > _length)
			_length = _offset;
		return true;
	}
	ASSERT(!("ATTEMPTED TO OVERRUN END OF BUFFER"));
	return false;
	*/
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::WriteInt8(int8 data)
{
	DEBUGBUF(( "IOStreamBuf::WriteInt8(0x%02x)", data ));

	ASSERT(_pcBuf); ASSERT(_length <= _max);

	if (_offset < _max)
	{
		*((PACKED int8*)(_pcBuf + _offset)) = data;
		_offset++;
		if (_offset > _length)
			_length = _offset;
		return true;
	}
	ASSERT(!("ATTEMPTED TO OVERRUN END OF BUFFER"));
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::WriteUint8(uint8 data)
{
	DEBUGBUF(( "IOStreamBuf::WriteUint8(0x%02x)", data ));

	ASSERT(_pcBuf); ASSERT(_length <= _max);

	if (_offset < _max)
	{
		*((PACKED int8*)(_pcBuf + _offset)) = data;
		_offset++;
		if (_offset > _length)
			_length = _offset;
		return true;
	}
	ASSERT(!("ATTEMPTED TO OVERRUN END OF BUFFER"));
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::WriteInt16(int16 data)
{
	DEBUGBUF(( "IOStreamBuf::WriteInt16(0x%04x)", data ));

	ASSERT(_pcBuf); ASSERT(_length <= _max);

	if (_offset+sizeof(int16) <= _max)
	{
		int16 netdata = HTONS(data);
		MEMCPY((void*)((PACKED char*)(_pcBuf + _offset)), (void*)&netdata, sizeof(int16));
		_offset += sizeof(int16);
		if (_offset > _length)
			_length = _offset;
		return true;
	}
	ASSERT(!("ATTEMPTED TO OVERRUN END OF BUFFER"));
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::WriteUint16(uint16 data)
{
	DEBUGBUF(( "IOStreamBuf::WriteUint16(0x%04x)", data ));

	ASSERT(_pcBuf); ASSERT(_length <= _max);

	if (_offset+sizeof(uint16) <= _max)
	{
		uint16 netdata = HTONS(data);
		MEMCPY((void*)((PACKED char*)(_pcBuf + _offset)), (void*)&netdata, sizeof(uint16));
		_offset += sizeof(uint16);
		if (_offset > _length)
			_length = _offset;
		return true;
	}
	ASSERT(!("ATTEMPTED TO OVERRUN END OF BUFFER"));
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::WriteInt32(int32 data)
{
	DEBUGBUF(( "IOStreamBuf::WriteInt32(0x%08x)", data ));

	ASSERT(_pcBuf); ASSERT(_length <= _max);

	if (_offset+sizeof(int32) <= _max)
	{
		int32 netdata = HTONL(data);
		MEMCPY((void*)((PACKED char*)(_pcBuf + _offset)), (void*)&netdata, sizeof(int32));
		_offset += sizeof(int32);
		if (_offset > _length)
			_length = _offset;
		return true;
	}
	ASSERT(!("ATTEMPTED TO OVERRUN END OF BUFFER"));
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::WriteUint32(uint32 data)
{
	DEBUGBUF(( "IOStreamBuf::WriteUint32(0x%08x)", data ));

	ASSERT(_pcBuf); ASSERT(_length <= _max);

	if (_offset+sizeof(uint32) <= _max)
	{
		uint32 netdata = HTONL(data);
		MEMCPY((void*)((PACKED char*)(_pcBuf + _offset)), (void*)&netdata, sizeof(uint32));
		_offset += sizeof(uint32);
		if (_offset > _length)
			_length = _offset;
		return true;
	}
	ASSERT(!("ATTEMPTED TO OVERRUN END OF BUFFER"));
	return false;
}
//-----------------------------------------------------------------------------
bool IOStreamBuf::WriteInt64(const int64& data)
{
//	DEBUGI64(( "WriteInt64(0x%016I64x)", data ));
	ARMDEBUG(( "WriteInt64(0x%016llx)", data ));

	ASSERT(_pcBuf); ASSERT(_length <= _max);

	if (_offset+sizeof(int64) <= _max)
	{
		//swap memory in this temp variable
		//ptrs don't have to be packed b/c they point to an aligned var
		int64 netdata = data;
		int32* pLo = (int32*)&netdata;
		int32* pHi = pLo + 1; //little-endian means hi-order is higher

		DEBUGI64(( "WriteInt64(0x%08lx%08lx)",*pHi,*pLo ));

		//need to swap hi/lo int32 contents
		int32 temp;
		temp = HTONL(*pHi);
		*pHi = HTONL(*pLo);
		*pLo = temp;

		MEMCPY((void*)((PACKED char*)(_pcBuf + _offset)), (void*)&netdata, sizeof(uint64));

		//DEBUGI64(( "WriteInt64=0x%016I64x", netdata ));
		DEBUGI64(( "WriteInt64=0x%08lx%08lx",*pHi,*pLo ));
		ARMDEBUG(( "WriteInt64=0x%016llx", netdata ));

		_offset += sizeof(uint64);
		if (_offset > _length)
			_length = _offset;
		return true;
	}
	ASSERT(!("ATTEMPTED TO OVERRUN END OF BUFFER"));
	return false;
}


//-----------------------------------------------------------------------------
bool IOStreamBuf::WriteUint32AtMark(uint32 mark, uint32 data)
{
	uint32 off = _offset;
	_offset = mark;
	bool ok = WriteUint32(data);
	_offset = off;
	return ok;
}

