/*=============================================================================

  Atlas Mobile Tournament Platform

  Copyright (c) 2003-2004 Atlas Mobile, Inc. All Rights Reserved.

  This software is the confidential and proprietary information of Atlas Mobile
  ("Confidential Information").  You shall not disclose such Confidential
  Information and shall use it only in accordance with the terms of the license
  agreement you entered into with Atlas Mobile.

  ATLAS MOBILE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
  NONINFRINGEMENT. ATLAS MOBILE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
  LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
  DERIVATIVES.
*/

#include "..\ForPrizes\Core\AMTPFramework.h"

#include "AEEHeap.h"

//-----------------------------------------------------------------------------
// DBGLOG_ENABLED flag:
//
// This flag determines whether certain debug functions will be included in
// the build.  These functions should only be invoked via their all upper case
// macro definitions so that all references will disappear in release when
// the associated code is left out of the build.
//-----------------------------------------------------------------------------


///////////////////////////////////////////////////////////////////////////////
// DbgBeep()
//
// Useful when trying to debug phone hardware and need some simple
// diagnostic feedback

#if DBGLOG_ENABLED
	void DbgBeep( AEESoundTone in_ToneId, uint16 in_DurationMilliSec )
	{
		ISound *pISound;
		
		if((ISHELL_CreateInstance( GETSHELL(), 
					AEECLSID_SOUND, (void **)&pISound)) == SUCCESS)
		{
			AEESoundToneData	aToneData;

			aToneData.eTone = in_ToneId;
			aToneData.wDuration = in_DurationMilliSec;

			ISOUND_PlayTone( pISound, aToneData );

			ISOUND_Release(pISound);
			pISound = NULL;
		}
	}// DbgBeep
#endif


///////////////////////////////////////////////////////////////////////////////
// DbgStk()
//
// Probes the operating memory of the stack

#if DBGSTK_ENABLED

	void DbgStk_Report( const char * in_pMsg )
	{
		AMTPApplet	*a_pApplet = (AMTPApplet*)GETAPPINSTANCE();

		if (a_pApplet && a_pApplet->_pClient){
			uint32	a_ProbeLo;
			uint32	a_ProbeHi;

			a_pApplet->_pClient->_DbgStk.Update();

			if ( a_pApplet->_pClient->_DbgStk.m_State == DbgStk::STATE_NeedReport ){
				a_ProbeLo = (uint32) a_pApplet->_pClient->_DbgStk.GetProbeLo();
				a_ProbeHi = (uint32) a_pApplet->_pClient->_DbgStk.GetProbeHi();

				a_pApplet->_pClient->_DbgLog.Append( in_pMsg );
				a_pApplet->_pClient->_DbgLog.Append( "-" );
				a_pApplet->_pClient->_DbgLog.Append( (int) a_ProbeLo, "0x%lx" );
				a_pApplet->_pClient->_DbgLog.Append( "-" );
				a_pApplet->_pClient->_DbgLog.Append( (int) a_ProbeHi, "0x%lx" );
				a_pApplet->_pClient->_DbgLog.Append( "-" );
				a_pApplet->_pClient->_DbgLog.Append( (int) (a_ProbeHi - a_ProbeLo), "0x%lx" );
				a_pApplet->_pClient->_DbgLog.Append( "-" );
				a_pApplet->_pClient->_DbgLog.Append( (int) &a_pApplet, "0x%lx" );
				a_pApplet->_pClient->_DbgLog.Append( "\n" );
				a_pApplet->_pClient->_DbgLog.FlushNow();

				a_pApplet->_pClient->_DbgStk.m_State = DbgStk::STATE_Idle;
			}
		}
	}

	void DbgStk_CheckExplicit( const void * in_pAddr )
	{
		AMTPApplet	*a_pApplet = (AMTPApplet*)GETAPPINSTANCE();

		if (a_pApplet && a_pApplet->_pClient){
			a_pApplet->_pClient->_DbgStk.Update();
			a_pApplet->_pClient->_DbgStk.RecordVisitedAddr( in_pAddr );
		}
	}

	void DbgStk_CheckLight( void )
	{
		AMTPApplet	*a_pApplet = (AMTPApplet*)GETAPPINSTANCE();

		if (a_pApplet && a_pApplet->_pClient){
			a_pApplet->_pClient->_DbgStk.Update();
		}
	}

	void DbgStk_CheckDeep( void )
	{
		AMTPApplet	*a_pApplet = (AMTPApplet*)GETAPPINSTANCE();

		if (a_pApplet && a_pApplet->_pClient){
			a_pApplet->_pClient->_DbgStk.Update();
			a_pApplet->_pClient->_DbgStk.PerformDeepTest();
		}
	}

	//-----------------------------------------------------------------------------
	//	DbgStk member functions
	//-----------------------------------------------------------------------------

	void DbgStk::InitProbes( const void *in_pVisitedAddr )
	{
		m_pProbeLo =	(void*) in_pVisitedAddr;
		m_pProbeHi =	(void*) in_pVisitedAddr;
		m_pProbeStung =	(void*) in_pVisitedAddr;
		m_State = STATE_NeedReport;
	}


	void DbgStk::Update( void )
	{
		switch( m_State ){
			case STATE_NeedInit:
				//InitProbes();
				//WriteSignature();
				//m_State = STATE_NeedReport;
				break;
			default:
				//ProbeStack();
				break;
		}
		RecordVisitedAddr( GetSampleGrowthPtr() );
	}


	void DbgStk::WriteSignature( void )
	{
	}


	void DbgStk::PerformDeepTest( void )
	{
	}


	void *DbgStk::GetSampleGrowthPtr( void )
	{
		uint32	 a_StackVar;
		uint32	*a_pStackVar = &a_StackVar; // Copy to ptr before return avoids a warning.

		return ( a_pStackVar );
	}


	void DbgStk::RecordVisitedAddr( const void *in_pVisitedAddr )
	{
		ASSERT( m_pProbeLo <= m_pProbeHi );

		if ( in_pVisitedAddr < m_pProbeLo ){
			m_pProbeLo = (void*) in_pVisitedAddr;
			m_State = STATE_NeedReport;
		}else if( in_pVisitedAddr > m_pProbeHi ){
			m_pProbeHi = (void*) in_pVisitedAddr;
			m_State = STATE_NeedReport;
		}
	}
#endif


///////////////////////////////////////////////////////////////////////////////
// DbgLog()
//
// Sends debug messages directly to a log file.  Useful when other means of
// logging are found to be unreliable on specific phone hardware.  See the
// following thread from Brew forum:  http://tinyurl.com/x946

#if DBGLOG_ENABLED

	/////////////////////////////////////////////////
	// API for C and C++
	/////////////////////////////////////////////////

	#if DBGLOG_VAARGS_ENABLED
		extern "C" void DbgLog_Append( const char * in_pFormatStr, ... )
		{
			va_list		a_ArgsVA;
			AMTPApplet	*a_pApplet = (AMTPApplet*)GETAPPINSTANCE();

			if (a_pApplet && a_pApplet->_pClient){
				va_start ( a_ArgsVA, in_pFormatStr );
				a_pApplet->_pClient->_DbgLog.Append_VA( in_pFormatStr, a_ArgsVA );
				va_end( a_ArgsVA );
			}
		}
		extern "C" void DbgLog_AppendLn( const char * in_pFormatStr, ... )
		{
			va_list		a_ArgsVA;
			AMTPApplet	*a_pApplet = (AMTPApplet*)GETAPPINSTANCE();

			if (a_pApplet && a_pApplet->_pClient){
				va_start ( a_ArgsVA, in_pFormatStr );
				a_pApplet->_pClient->_DbgLog.Append_VA( in_pFormatStr, a_ArgsVA );
				va_end( a_ArgsVA );
				a_pApplet->_pClient->_DbgLog.Append_SimpStr( "\n" );
			}
		}
	#else
		extern "C" void DbgLog_Append( const char * in_pFormatStr, ... )
		{
			AMTPApplet	*a_pApplet = (AMTPApplet*)GETAPPINSTANCE();

			if (a_pApplet && a_pApplet->_pClient){
				a_pApplet->_pClient->_DbgLog.Append_SimpStr( in_pFormatStr );
			}
		}
		extern "C" void DbgLog_AppendLn( const char * in_pFormatStr, ... )
		{
			AMTPApplet	*a_pApplet = (AMTPApplet*)GETAPPINSTANCE();

			if (a_pApplet && a_pApplet->_pClient){
				a_pApplet->_pClient->_DbgLog.Append_SimpStr( in_pFormatStr );
				a_pApplet->_pClient->_DbgLog.Append_SimpStr( "\n" );
			}
		}
	#endif


	extern "C" void DbgLog_Flush( void )
	{
		((AMTPApplet*)GETAPPINSTANCE())->_pClient->_DbgLog.FlushNow();
	}


	/////////////////////////////////////////////////
	// API for C++ only
	/////////////////////////////////////////////////

	void DbgLog_Append( const AECHAR * in_pMsg )
	{
		((AMTPApplet*)GETAPPINSTANCE())->_pClient->_DbgLog.Append( in_pMsg );
	}


	void DbgLog_Append( int in_iVal )
	{
		// ((AMTPApplet*)GETAPPINSTANCE())->_pClient->_DbgLog.Append( in_iVal );
		// ((AMTPApplet*)GETAPPINSTANCE())->_pClient->_DbgLog.Append( in_iVal, "%ld" );
		((AMTPApplet*)GETAPPINSTANCE())->_pClient->_DbgLog.Append( in_iVal, "0x%lx" );
	}


	void DbgLog_Append( bool in_bVal )
	{
		((AMTPApplet*)GETAPPINSTANCE())->_pClient->_DbgLog.Append( in_bVal );
	}


	void DbgLog_Enabler( bool in_bVal )
	{
		AMTPApplet	*a_pApplet = (AMTPApplet*)GETAPPINSTANCE();

		if (a_pApplet && a_pApplet->_pClient){
			if ( in_bVal ){
				a_pApplet->_pClient->_DbgLog.EnableFlags( DbgLog::FLAG_ACTIVE );
				a_pApplet->_pClient->_DbgLog.Append_SimpStr( "\nDBGLOG is active.\n" );
			}else{
				a_pApplet->_pClient->_DbgLog.Append_SimpStr( "\nDBGLOG is inactive.\n" );
				a_pApplet->_pClient->_DbgLog.DisableFlags( DbgLog::FLAG_ACTIVE );
			}
		}
	}


	void DbgLog_Mem( const char * in_pMsg )
	{
		AMTPApplet	*a_pApplet = (AMTPApplet*)GETAPPINSTANCE();

		if (a_pApplet && a_pApplet->_pClient){
			IHeap	*a_pIHeap;
			int		 a_InstCreateResult;
			uint32	 a_MemInUse;
			uint32	 a_BustSize;

			a_InstCreateResult = ISHELL_CreateInstance( GETSHELL(), AEECLSID_HEAP, (void **)& a_pIHeap);
			a_MemInUse = IHEAP_GetMemStats( a_pIHeap );

			for ( a_BustSize = 0x10;; a_BustSize *= 2 ){
				if ( ! IHEAP_CheckAvail( a_pIHeap, a_BustSize) ) break;
			}

			IHEAP_Release( a_pIHeap );

			a_pApplet->_pClient->_DbgLog.Append( in_pMsg );
			if ( a_InstCreateResult != SUCCESS ){
				a_pApplet->_pClient->_DbgLog.Append( "-?- " );
			}
			a_pApplet->_pClient->_DbgLog.Append( a_MemInUse, "0x%lx" );
			a_pApplet->_pClient->_DbgLog.Append( "-" );
			a_pApplet->_pClient->_DbgLog.Append( a_BustSize, "0x%lx" );
			a_pApplet->_pClient->_DbgLog.Append( "-" );
			a_pApplet->_pClient->_DbgLog.Append( (int) &a_BustSize, "0x%lx" );
			a_pApplet->_pClient->_DbgLog.Append( "\n" );
			a_pApplet->_pClient->_DbgLog.FlushNow();
		}
	}


	//-----------------------------------------------------------------------------
	//	DbgLog member functions
	//-----------------------------------------------------------------------------


	DbgLog::DbgLog()
		: m_pFileMgr( NULL )
		, m_pFileHdl( NULL )
		//
		, m_Flags( 0 )
		, m_iWritesPerAutoFlush( 1 )
		, m_iAutoFlushCountdown( 1 )
		, m_iWriteFailureCt( 0 )
	{
		m_pBufTail = m_pBuf;
		m_pBufWriteThreshhold = m_pBuf;
		SetFilePath( "dbg.log" );
		//
		EnableFlags( DbgLog::FLAG_ACTIVE );
		EnableWriteImmediate();
		EnableAutoFlush();
		DisableFlushDeep();
	}


	#if DBGLOG_VAARGS_ENABLED && 0
		void DbgLog::Append( const char * in_pFormatStr, ... )
		{
			va_list a_ArgsVA;

			va_start ( a_ArgsVA, in_pFormatStr );
			Append_VA( in_pFormatStr, a_ArgsVA );
			va_end( a_ArgsVA );
		}// DbgLog::Append
	#endif


	#if DBGLOG_VAARGS_ENABLED
		void DbgLog::Append_VA( const char * in_pFormatStr, va_list in_ArgsVA )
		{
			const char		*a_pNextChar = in_pFormatStr;
			const char		*a_StrArg;
			int				 a_IntArg;
			char			 a_CharArg;


			for(;;){
				ASSERT( a_pNextChar );

				if ( *a_pNextChar == 0 ){
					break;
				}else if( *a_pNextChar == '%' ){
					a_pNextChar++;
					if ( *a_pNextChar == 'l' ){
						a_pNextChar++;
						if ( *a_pNextChar == '0' ){
							a_pNextChar++;
							if ( (*a_pNextChar == '4') || (*a_pNextChar == '8') ){
								a_pNextChar++;
							}
						}
					}
					if ( *a_pNextChar == 0 ) break;

					switch ( *a_pNextChar ){
						case 's':
							a_StrArg = va_arg( in_ArgsVA, const char * );
							Append_SimpStr( a_StrArg );
							break;
						case 'd':
							a_IntArg = va_arg( in_ArgsVA, int );
							Append( a_IntArg, "%ld" );
							break;
						case 'x':
							a_IntArg = va_arg( in_ArgsVA, int );
							Append( a_IntArg, "%lx" );
							break;
						case 'u':
							a_IntArg = va_arg( in_ArgsVA, int );
							Append( a_IntArg, "%lu" );
							break;
						case 'c':
							a_CharArg = va_arg( in_ArgsVA, char );
							Append( a_CharArg );
							break;
						case '%':
							Append( *a_pNextChar );
							break;
						default:
							a_CharArg = va_arg( in_ArgsVA, char );
							Append( '?' );
							break;
					}
				}else{
					Append( *a_pNextChar );
				}
				a_pNextChar++;
			}
		}// DbgLog::Append_VA
	#endif


	void DbgLog::Append( char in_Char )
	{
		if ( CheckFlags( FLAG_ACTIVE ) ){
			if ( m_pBufTail >= m_pFilePath ){
				WriteToFileNow();
			}

			*m_pBufTail = in_Char;
			m_pBufTail++;
		}
	}


	void DbgLog::Append_SimpStr( const char *in_pMsg )
	{
		if ( CheckFlags( FLAG_ACTIVE ) ){
			uint32 aMsgLen = STRLEN( in_pMsg );
			uint32 aBufAvail = m_pFilePath - m_pBufTail;

			if ( aMsgLen > aBufAvail ){
				WriteToFileNow();
				aBufAvail = m_pFilePath - m_pBufTail;
			}
			
			if ( aMsgLen > aBufAvail ){
				m_iWriteFailureCt++;
				aMsgLen = aBufAvail;
			}
			
			STRNCPY( m_pBufTail, in_pMsg, aMsgLen );

			m_pBufTail += aMsgLen;
		}
	}


	void DbgLog::Append( const AECHAR *in_pMsg )
	{
		if ( CheckFlags( FLAG_ACTIVE ) ){
			uint32 aMsgLen = WSTRLEN( in_pMsg );
			uint32 aBufAvail = m_pFilePath - m_pBufTail;

			if ( aMsgLen > aBufAvail ){
				WriteToFileNow();
				aBufAvail = m_pFilePath - m_pBufTail;
			}
			
			if ( aMsgLen > aBufAvail ){
				m_iWriteFailureCt++;
				aMsgLen = aBufAvail;
			}
			
			WSTRTOSTR( in_pMsg, m_pBufTail, aMsgLen);

			m_pBufTail += aMsgLen;
		}
	}


	void DbgLog::Append( int in_iVal, const char * in_pFormatStr )
	{
		char pConvertedMsg[ 32 ];

		SPRINTF( pConvertedMsg, in_pFormatStr, in_iVal );
		Append_SimpStr( pConvertedMsg );
	}// DbgLog_Append int value


	void DbgLog::Append( bool in_bVal )
	{
		if ( in_bVal ){
			Append_SimpStr( "true" );
		}else{
			Append_SimpStr( "false" );
		}
	}// DbgLog::Append bool value


	void DbgLog::SetFilePath( const char *in_pFilePath )
	{
		int32	 aPathLen = STRLEN( in_pFilePath );
		char	*pPathStart = &m_pBuf[ BUFSIZE ] - (aPathLen + 1);

		ASSERT( aPathLen < (BUFSIZE/4) ); // Complain if path consumes a ridiculous portion of buffer.
		ASSERT( in_pFilePath );
		ASSERT( m_pBufTail < pPathStart );
		ASSERT( m_pBufWriteThreshhold < pPathStart );

		m_pFilePath = pPathStart;
		STRCPY(m_pFilePath, in_pFilePath);
	}


	void DbgLog::SetWriteThreshhold( int in_Count )
	{
		m_pBufWriteThreshhold = &m_pBuf[ in_Count ];

		ASSERT( m_pBufWriteThreshhold < m_pFilePath );
	}


	void DbgLog::SetWritesPerAutoFlush( int in_Count )
	{
		m_iWritesPerAutoFlush = in_Count;
	}


	void DbgLog::WriteToFileNow( void )
	{
		//bool bStepSuccess	= false;
		bool bWriteSuccess	= false;

		do { // once through

			// Is there anything to write?
			if (m_pBufTail > m_pBuf){

				// Try m_pFileMgr.
				if ( ! m_pFileMgr ){
					if( SUCCESS != (
						ISHELL_CreateInstance( GETSHELL(), AEECLSID_FILEMGR, (void **)&m_pFileMgr )
					)) break;
					if ( ! m_pFileMgr ) break;
				}

				// Try m_pFileHdl
				if ( ! m_pFileHdl ){
					OpenFileMode mode;

					if(IFILEMGR_Test(m_pFileMgr, m_pFilePath) == SUCCESS){
						mode = _OFM_APPEND;
					}else{
						mode = _OFM_CREATE;
					}

					m_pFileHdl = IFILEMGR_OpenFile(m_pFileMgr, m_pFilePath, mode);
					if ( ! m_pFileHdl ) break;
				}

				// Write to file.
				IFILE_Write( m_pFileHdl, m_pBuf, (uint32)(m_pBufTail - m_pBuf) );

				// Reset m_pBufTail.
				m_pBufTail = m_pBuf;
			}

			// Flush if appropriate.
			if ( CheckFlushNextWrite() ){
				ShallowFlush_PrimitiveOnly();
				if ( CheckFlushDeep() ){
					DeepFlush_PrimitiveOnly();
				}
				DisableFlushNextWrite();
			}

			// Countdown for next auto flush.
			if ( CheckAutoFlush() ){
				if ( m_iAutoFlushCountdown > 1 ){
					m_iAutoFlushCountdown--;
				}else{
					m_iAutoFlushCountdown = m_iWritesPerAutoFlush;
					EnableFlushNextWrite();
				}
			}

			// Write was successful.
			bWriteSuccess = true;

			//DbgBeep( AEE_TONE_HIGH_PITCH_A, 50 );
		} while( false ); // once through

		if ( ! bWriteSuccess ){
			if ( m_iWriteFailureCt < WRITE_FAILURE_CT_MAX ){
				DbgBeep( AEE_TONE_LOW_PITCH_A, 1000 );
				m_iWriteFailureCt++;
			}
		}
	}// WriteToFileNow


	void DbgLog::FlushNow( void )
	{
		ShallowFlushNow();
		if ( CheckFlushDeep() ){
			DeepFlush_PrimitiveOnly();
		}
	}


	void DbgLog::ShallowFlushNow( void )
	{
		WriteToFileNow();
		ShallowFlush_PrimitiveOnly();
	}


	void DbgLog::DeepFlushNow( void )
	{
		ShallowFlushNow();
		DeepFlush_PrimitiveOnly();
	}


	void DbgLog::ShallowFlush_PrimitiveOnly( void )
	{
		if ( m_pFileHdl ){
			IFILE_Release(m_pFileHdl);
			m_pFileHdl = NULL;
		}
	}


	void DbgLog::DeepFlush_PrimitiveOnly( void )
	{
		if ( m_pFileMgr ){
			IFILEMGR_Release(m_pFileMgr);
			m_pFileMgr = NULL;
		}
	}
#endif
