/*=============================================================================

  Atlas Mobile Tournament Platform

  Copyright (c) 2003-2004 Atlas Mobile, Inc. All Rights Reserved.

  This software is the confidential and proprietary information of Atlas Mobile
  ("Confidential Information").  You shall not disclose such Confidential
  Information and shall use it only in accordance with the terms of the license
  agreement you entered into with Atlas Mobile.

  ATLAS MOBILE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
  NONINFRINGEMENT. ATLAS MOBILE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
  LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
  DERIVATIVES.

=============================================================================*/

#ifndef DEBUG_H
#define DEBUG_H

#include "AEEStdLib.h"

//-----------------------------------------------------------------------------
// DBGLOG_ENABLED
//
// This flag determines whether certain debug functions will be included in
// the build.  These functions should only be invoked via their all upper case
// macro definitions so that all references will disappear in release when
// the associated code is left out of the build.
//
// NOTE (031202-Joe):  Might want to use following in defining DBGLOG_ENABLED:
//    ( defined(_DEBUG) || defined(_ARMDEBUG) || 1 )
// To use the above, need to determine where to define _ARMDEBUG.  Might have to add it to makefile.
//-----------------------------------------------------------------------------
#define DBGLOG_ENABLED			0
#define DBGLOG_VAARGS_ENABLED	0
#define DBGSTK_ENABLED			0
//#define _DEBUG ESC - NOW DEFINED BY COMPILER
//-----------------------------------------------------------------------------
//	TRACE
//-----------------------------------------------------------------------------
#if !defined(NOTRACE)
	#if DBGLOG_ENABLED && 0
		#define TRACE(expr)		DBGLOGLN(expr)
	#elif defined(_DEBUG)
		#define	TRACE(expr)		(DBGPRINTF expr)
	#else
		#define TRACE(expr)
	#endif
#else
	#define TRACE(expr)
#endif
//-----------------------------------------------------------------------------
#if DBGLOG_ENABLED && 0
	#define DEBUG(expr)			DBGLOGLN(expr)
#elif defined(_DEBUG)
	#define DEBUG(expr)			(DBGPRINTF expr)
#else
	#define DEBUG(expr)
#endif
//-----------------------------------------------------------------------------
#if defined(_ARMDEBUG)
	#define ARMDEBUG(expr)		(DBGPRINTF expr)
#else
	#define ARMDEBUG(expr)
#endif

//-----------------------------------------------------------------------------
//	Debug Conditional
//-----------------------------------------------------------------------------
#if defined(_DEBUG)
	#define DEBUGIF(expr) if(expr)
#else
	#define DEBUGIF(expr)
#endif

//-----------------------------------------------------------------------------
//	DbgStk
//-----------------------------------------------------------------------------
#if DBGSTK_ENABLED
	#ifdef __cplusplus
	extern "C"
	{
	#endif

		void DbgStk_Report( const char * in_pMsg );
		void DbgStk_CheckExplicit( const void * in_pAddr );
		void DbgStk_CheckLight( void );
		void DbgStk_CheckDeep( void );

	#ifdef __cplusplus
	}
	#endif

	/////////////////////////////////////////////////
	// Debug Macros
	/////////////////////////////////////////////////
	#define DBGSTK_REPORT(msg)				DbgStk_Report(msg)
	#define DBGSTK_CHECK_EXPLICIT(addr)		DbgStk_CheckExplicit(addr)
	#define DBGSTK_CHECK_LIGHT				DbgStk_CheckLight()
	#define DBGSTK_CHECK_DEEP				DbgStk_CheckDeep()
#else
	/////////////////////////////////////////////////
	// Stubbed Macros for non-debug build.
	/////////////////////////////////////////////////
	#define DBGSTK_REPORT(msg)
	#define DBGSTK_CHECK_EXPLICIT(addr)
	#define DBGSTK_CHECK_LIGHT
	#define DBGSTK_CHECK_DEEP
#endif


//-----------------------------------------------------------------------------
//	DbgBeep and DbgLog
//-----------------------------------------------------------------------------
#if DBGLOG_ENABLED

	/////////////////////////////////////////////////
	// API for C++ only
	/////////////////////////////////////////////////
	#ifdef __cplusplus
		#include "AEESound.h" // For tone id enums.

		void DbgBeep(
			AEESoundTone	in_ToneId			= AEE_TONE_MED_PITCH_A, // See AEESound.h for constants such as AEE_TONE_MED_PITCH_A.
			uint16			in_DurationMilliSec	= 200 );
		void DbgLog_Append( const AECHAR * in_pMsg );
		void DbgLog_Append( int in_iVal );
		void DbgLog_Append( bool in_bVal );
		void DbgLog_Enabler( bool in_bVal ); // The "bool" type is not part of ansi-C.
	#endif // __cplusplus

	/////////////////////////////////////////////////
	// API for C and C++
	/////////////////////////////////////////////////
	#ifdef __cplusplus
	extern "C"
	{
	#endif

		void DbgLog_Append( const char * in_pFormatStr, ... );
		void DbgLog_AppendLn( const char * in_pFormatStr, ... );
		void DbgLog_Flush( void );
		void DbgLog_Mem( const char * in_pMsg );

	#ifdef __cplusplus
	}
	#endif

	/////////////////////////////////////////////////
	// Debug Macros
	/////////////////////////////////////////////////
	#define DBGLOG(msg)					(DbgLog_Append msg)
	#define DBGLOGLN(msg)				(DbgLog_AppendLn msg)
	#define DBGLOGFLUSH					DbgLog_Flush()
	#define DBGLOGON					DbgLog_Enabler(true)
	#define DBGLOGOFF					DbgLog_Enabler(false)
	#define DBGLOGMEM(msg)				DbgLog_Mem(msg)
	#define DBGBEEP(toneid,duration)	DbgBeep(toneid,duration)

#else
	/////////////////////////////////////////////////
	// Stubbed Macros for non-debug build.
	/////////////////////////////////////////////////
	#define DBGLOG(msg)
	#define DBGLOGLN(msg)
	#define DBGLOGFLUSH
	#define DBGLOGON
	#define DBGLOGOFF
	#define DBGLOGMEM
	#define DBGBEEP(toneid,duration)	{}
#endif

//-----------------------------------------------------------------------------
//	ASSERT macro
//
//	This version of ASSERT will not exit the app w/an exception
//-----------------------------------------------------------------------------
#if defined(_WINDOWS) && defined(_DEBUG)

#define ASSERT(expr)  if(!(expr))DBGPRINTF("%s(%d) : ASSERT FAILED: (%s)", __FILE__, __LINE__, #expr);else;

#elif defined(_ARMDEBUG)

#define ASSERT(expr)  if(!(expr))DBGPRINTF("ASSERT_FAILED(%s)", #expr);else;

#else

#define ASSERT(expr)

#endif


//-----------------------------------------------------------------------------
//	pragma removes this warning in MSVC caused by empty ASSERTS in release build:
//		"warning C4390: ';' : empty controlled statement found; is this the intent?"
//-----------------------------------------------------------------------------
#if defined(_WINDOWS) && !defined(_DEBUG)
#pragma warning(disable: 4390)
#endif

#endif //DEBUG_H
