#============================================================================
#  Name:
#    brewcmx-gcc.mak
#
#  Description:
#    Makefile to build the $(TARGET) downloadable module.
#
#   The following targets are available in this makefile:
#
#     all           - make .elf and .mod image files (default)
#     clean         - delete object directory and image files
#
#   The above targets can be made with the following command:
#
#     nmake /f brewcmx-gcc.mak [target]
#	  make -f brewcmx-gcc.mak [target]
#
#  Assumptions:
#    1. cygwin is installed on your machine and is in the path.  
#	 2. GCCHOMEPATH points to location where GNU tools are installed.
#
#  Notes:
#    1. You might need to modify the GCCHOMEPATH, GCCBINPATH, ELF2MODTOOLPATH 
#		and LIBDIRS variable below.
#    2. While building the application using this make file, the following
#		warnings may be received. They can be safely ignored:
#		"Warning: _divsi3.o does not support interworking, whereas <Your App>.elf does not"
#		"Warning: _dvmd_tls.o does not support interworking, whereas <Your App>.elf does not"
#
#        Copyright  2000-2003 QUALCOMM Incorporated.
#               All Rights Reserved.
#            QUALCOMM Proprietary/GTDR
#
#----------------------------------------------------------------------------
#-----------------------------------------------------------------------
# Target File Name and Type Definitions
#   (you'll need to specify a target name; i.e. what you want the
#    program to be called)
#-----------------------------------------------------------------------
TARGET	=   $(APPNAME)
EXETYPE	=	elf
BINARY  =	bin
MODULE	=	mod
PROGRAM = Progra~1

#-----------------------------------------------------------
# Fix the path here to location of AEEAppGen.c and AEEModGen.c
#-----------------------------------------------------------
AEESRCPATH = "C:/Program Files/brew 3.1.5/sdk/src"

#-----------------------------------------------------------
# Fix the path here to location of BREW header files
#-----------------------------------------------------------
AEEINCPATH = "C:/Program Files/brew 3.1.5/sdk/inc"

#-----------------------------------------------------------
# Fix the path here to location of framework files
#-----------------------------------------------------------

#-----------------------------------------------------------
# Additional objects apart from $(TARGET).o 
#-----------------------------------------------------------
ADD_OBJS = aScroll.o aGame.o aKey.o aSprite.o aTextBox.o aUtils.o aScore.o aMain.o aSound.o bText.o aImage.o#aText.o mem.o aDebug.o aProfiler.o 

#-----------------------------------------------------------------------
# Set the object files that we care about
# (if your program has more than one source file, you'll need to add
# the rest of them to this list (changing the extension to .o))
#-----------------------------------------------------------------------
APP_OBJS =	AEEModGen.o AEEAppGen.o $(ADD_OBJS) #

#-----------------------------------------------------------------------
# Target compile time symbol definitions
#
# Tells the SDK source stuffs that we're building a dynamic app.
#-----------------------------------------------------------------------
DEFINES = 	-DDYNAMIC_APP

#-----------------------------------------------------------------------
# Software tool and environment definitions
#   (GCCHOMEPATH needs to be set to the location of your gcc cross-compiler
#    and binutils.  ELF2MODTOOLPATH needs to set to the location of of
#	BREWelf2mod.exe)
#-----------------------------------------------------------------------
GCCHOMEPATH  =  C:/$(PROGRAM)/GNUARM
GCCBINPATH = $(GCCHOMEPATH)/bin
ELF2MODTOOLPATH = $(GCCHOMEPATH)/bin

GCC	=	$(GCCBINPATH)/arm-elf-gcc.exe
G++	=	$(GCCBINPATH)/arm-elf-g++.exe
LD	=	$(GCCBINPATH)/arm-elf-ld.exe
ELF2MODTOOL	=	$(ELF2MODTOOLPATH)/BREWelf2mod.exe

#-----------------------------------------------------------------------
# Compiler optimization options
#   -O0 disables compiler optimizations.  Other options probably work as
#     well.  Set to taste.
#-----------------------------------------------------------------------
OPT	=	-Os

#-----------------------------------------------------------------------
# Compiler code generation options
#   Add $(TARG) to the CODE line if you're building a Thumb binary (at
#   the moment, this doesn't work).
#
# Note: to compile for nokia devices you must delete "-fshort-enums" in
#       the CODE Flag
#-----------------------------------------------------------------------
END	=	-mlittle-endian
CODE++ =	$(END) -fno-builtin -fno-exceptions -ffunction-sections -fno-unwind-tables -fno-rtti -fshort-enums
CODECC =	$(END) -fno-builtin -ffunction-sections -fshort-enums
#CODE =	$(END) -fno-builtin

#-----------------------------------------------------------------------
# Include file search path options
#   (change this to point to where the BREW SDK headers are located)
#-----------------------------------------------------------------------
INC	=	-I$(AEEINCPATH) #-I$(GAMEINCPATH) -I$(GAMECONFIGINCPATH) -I$(FRAMEWORKINCPATH) -I$(DEBUGINCPATH) -I$(PROFILERINCPATH)#-I$(GCCHOMEPATH)\lib\gcc\arm-elf\4.0.2\include #-I$(GCCHOMEPATH)\arm-elf\include 

#-----------------------------------------------------------------------
# Library search path options. It points the location of libgcc.a
#-----------------------------------------------------------------------
LIBDIRS = -L$(GCCHOMEPATH)/lib/gcc/arm-elf/4.0.2/thumb -L$(GCCHOMEPATH)/arm-elf/lib/thumb#-L . -L$(GCCHOMEPATH)/arm-elf/lib
#-----------------------------------------------------------------------
# Nothing below here (except for the dependencies at the bottom of the
#  file) should need to be changed for a reasonably normal compilation.
#-----------------------------------------------------------------------

#-----------------------------------------------------------------------
# Processor architecture options
# Sets the designated target processor for this applet.
# Currently, all BREW phones use the ARM 7t chip
#-----------------------------------------------------------------------
CPU	=	-mcpu=arm7tdmi

#-----------------------------------------------------------------------
# ARM Procedure Call Standard (APCS) options
# -fPIC             sets posititon independent code.  Other option: -fpic
# -mthumb-interwork enables switching between ARM and Thumb code
# -mapcs-frame      runs on systems with the frame ptr. specified in the
# 			APCS
#-----------------------------------------------------------------------
ROPI	= 	
INTRWK	=	-mthumb-interwork
TARG =	-mthumb
FRAME = -mapcs-frame
#APCS	=	-mapcs-frame $(ROPI) $(INTRWK)
APCSC  = $(FRAME) $(TARG) $(INTRWK) -mcallee-super-interworking
APCSC++  = $(FRAME) $(TARG) $(INTRWK)

#-----------------------------------------------------------------------
# Compiler output options
# -c sets object file output only
#-----------------------------------------------------------------------
OUT	=	-c

#-----------------------------------------------------------------------
# Compiler/assembler debug Options
#   -g is the standard flag to leave debugging information in the
#   object files.
#-----------------------------------------------------------------------
#DBG	=	-g
#DBG	= -g -ffixed-ip

#-----------------------------------------------------------------------
# Compiler flag definitions
#-----------------------------------------------------------------------
CFLAGS0 =	$(OUT) $(DEFINES) $(CPU) $(APCSC) $(CODECC) $(DBG) 
CFLAGS1 =	$(OUT) $(DEFINES) $(CPU) $(APCSC++) $(CODE++) $(DBG) 
CFLAGS =	$(CFLAGS0) $(INC) $(OPT)
C++FLAGS =	$(CFLAGS1) $(INC) $(OPT) 


#-----------------------------------------------------------------------
# Linker Options
# -o sets the output filename
#-----------------------------------------------------------------------
#LINK_CMD  = -Ttext 0 --emit-relocs -entry AEEMod_Load -o
LINK_CMD = --script armelf.brew --verbose --emit-relocs --no-warn-mismatch -o 
LIBS      = -lgcc -lc#-lm -lsupc++  

#-----------------------------------------------------------------------
# Linker flag definitions
#-----------------------------------------------------------------------
LDFLAGS =	$(LIBDIRS)

#-----------------------------------------------------------------------
# Default target
#-----------------------------------------------------------------------
default: $(TARGET).$(MODULE)

#-----------------------------------------------------------------------
# All target
#-----------------------------------------------------------------------
all: $(TARGET).$(MODULE)

#-----------------------------------------------------------------------
# Targets for making the actual binary
#-----------------------------------------------------------------------
$(TARGET).$(MODULE) :   $(TARGET).$(EXETYPE)
	$(ELF2MODTOOL) $(TARGET).$(EXETYPE) $(TARGET).$(MODULE)

$(TARGET).$(EXETYPE) :  $(APP_OBJS)
	$(LD) $(LINK_CMD) $(TARGET).$(EXETYPE) $(LDFLAGS) $(APP_OBJS) $(LIBS) $(LINK_ORDER)

#-----------------------------------------------------------------------
# Cleanup 
#-----------------------------------------------------------------------
clean:
	rm -f $(APP_OBJS) $(TARGET).$(EXETYPE) $(TARGET).$(MODULE)		

#-----------------------------------------------------------------------
# Object File Dependencies
#   You may well want to add more dependencies here.
#-----------------------------------------------------------------------
$(TARGET).o:	$(TARGET).cpp

AEEAppGen.o:
	$(GCC) $(CFLAGS) -o AEEAppGen.o $(AEESRCPATH)/AEEAppGen.c
AEEModGen.o:
	$(GCC) $(CFLAGS) -o AEEModGen.o $(AEESRCPATH)/AEEModGen.c
GCCResolver.o: 
	$(GCC) $(CFLAGS) -o GCCResolver.o $(AEESRCPATH)/misc/GCCResolver.c
aScroll.o:
	$(G++) $(C++FLAGS) -o aScroll.o ./aScroll.cpp
aGame.o:
	$(G++) $(C++FLAGS) -o aGame.o ./aGame.cpp
aKey.o:
	$(G++) $(C++FLAGS) -o aKey.o ./aKey.cpp
aSprite.o:
	$(G++) $(C++FLAGS) -o aSprite.o ./aSprite.cpp
aText.o:
	$(G++) $(C++FLAGS) -o aText.o ./aText.cpp
aTextBox.o:
	$(G++) $(C++FLAGS) -o aTextBox.o ./aTextBox.cpp
aUtils.o:
	$(G++) $(C++FLAGS) -o aUtils.o ./aUtils.cpp
aDebug.o:
	$(G++) $(C++FLAGS) -o aDebug.o ./aDebug.cpp
aProfiler.o:
	$(G++) $(C++FLAGS) -o aProfiler.o ./aProfiler.cpp
aScore.o:
	$(G++) $(C++FLAGS) -o aScore.o ./aScore.cpp
aMain.o: 
	$(G++) $(C++FLAGS) -o aMain.o ./aMain.cpp
mem.o: 
	$(G++) $(C++FLAGS) -o mem.o ./mem.cpp
aSound.o: 
	$(G++) $(C++FLAGS) -o aSound.o ./aSound.cpp
bText.o:
	$(G++) $(C++FLAGS) -o bText.o ./bText.cpp
aImage.o:
	$(G++) $(C++FLAGS) -o aImage.o ./aImage.cpp





