#ifndef _TB_EVENTS_H_
#define _TB_EVENTS_H_

#include "..\Core\TBBaseEvents.h"
#include "TBCompliance.h"
#include "AEEAppGen.h"

/**
 * <p>TBEvents is the main Client-Facing class which serves both to initialize the ForPrize 
 * adapter, and as the main event bus which drives all ForPrize Client/Module interaction.</p>
 * 
 * <p>TBEvents provides two mechanisms:</p><pre> 
 * A centralized repository for interface invocation/implementation management
 * 
 * A "Poor Man's Event Handler", TBEvents provides a mechanism for stacking
 * multiple interface implementations on top of one another.
 * </pre>
 * 
 * Interface implementation should be cached on initialization using:
 * <pre>FPEvent.registerEventHandler(SupportedInterface)</pre>
 * 
 * And Invocation controllers references can be attained with:
 * <pre>FPEvent.getSupportedInterfaceHandler()</pre>
 * 
 * <p>The set of "SupportedInterface"s is currently hard-coded into this class for
 * speed and size reasons.  The capacity for the client to over-ride built-in
 * functionality should depend on the scoping of the register and get Methods.</p>
 * 
 * <p>Currently there are two main types of interfaces supported at the moment (the types 
 * are purely conceptual):</p>
 * 
 * <p><b>Controls</b> provide a mechanism for the client to make requests
 * to the FP Module.  These requests are not garanteed to be granted (the boolean return
 * flag will inform the client whether or not they are).  In general, only requests 
 * Salient to the current state of the FP Module will be garanteed to work.</p>
 * 
 * <p><b>EventListeners</b> provide a mechanism for the client to recieve FP Module specific
 * events, and choose whether or not to act on those events.</p>
 * 
 * <p>Cached Implementations are stored into a LIFO stack.  When the Interface's invocation
 * controller method is called, the the invocation controller iterates down from
 * the top of the stack calling the implementing interface references until one of
 * them returns true, or the stack empties (in which case false is returned).  This
 * allows for partial over-ride of built in system behavior.</p>
 * 
 * <p>While theoretically there are benefits in design flexibility to stack-caching
 * both the Controls and Event listeners, for the sake of code size, this particular
 * implementation only caches EventListeners.  Only a single instance of Controls is
 * stored.</p>
 * 
 * @author E Scott Cragg.  Twistbox Client Core Engineering.
 * 
 * @author Tiago Teixeira.  Twistbox Client Core Engineering.
 */

class TBEvents:
	public TBBaseEvents
{
	AEEApplet* myMain;
public:
	TBEvents(AEEApplet* _main);
	~TBEvents();
	static void* compliance;
public:
	static void fpInit(AEEApplet* _main,byte localizationCode);

	static void Release();

	void menuTick(){
		//TODO Auto-generated method stub
	};
};


#endif _TB_EVENTS_H_
