#ifndef _ITB_MENU_CONTROLS_H_
#define _ITB_MENU_CONTROLS_H_


#include "..\..\src\BrewString.h"


/**
 * <p>MenuControls provides a mechanism for the client to report user menu interactions during For Prize Module
 * driven UI back to the module.</p>
 * 
 * <p>The For Prize Module Menu element supports two types on menu feedback:</p>
 * 
 * <p><b>Commands</b> are directives to the for Prize Module to do processing and enter a new state.
 * <p><b>Input</b> are mechanisms for user generated Variable data to be fed back to the ForPrize Module
 * to be saved for later Operation when a 'command' is recieved.
 * 
 * @author E Scott Cragg. Twistbox Client Core Engineering.
 *
 * and Tiago Teixeira. Twistbox Client Core Engineering.
 *
 */
class ITBMenuControls {
	
public:
	/**
	 * Update the internal For Prize Registers associated with an input-type menu entry.  See <i>MenuEventListener</i>
	 * for more information.
	 * 
	 * @param menuId The Menu ID of the input field that was modified
	 * @param menuEntryIndex The Menu Index of the field that was modified
	 * @param menuEntryType The Type of Menu Entry Modified
	 * @param entryData The new Entry Data to save (if switch entry)
	 * @param entryText The new Entry Text to save (if textInput entry)
	 * @return true if the For Prize Module understood the request and saved the data, false otherwise.
	 */
	virtual bool menuInput(int menuId, int menuEntryIndex, int menuEntryType, int entryData, String entryText) = 0;
	
	/**
	 * <p>Specifies that a specific command on the menu has been requested by the user. See <i>MenuEventListener</i>
	 * for more information.</p>
	 * 
	 * @param menuCommandData The entryData of the command that was specified by the user.
	 * @return true if the For Prize Module understood the request and is acting on the command, false otherwise.
	 */
	virtual bool menuCommand(int menuCommandData) = 0;

	/**
	 * <p> A Mechnism for providing periodic access to the LOAD thread of the client application, this mechanism is
	 * optional (calling it once enables it).  Once enabled, _ALL_ setMenu() event series will be returned via the
	 * thread that calls menuTick() - This prevents certain thread syncronization issues in applications which 
	 * were not designed top-down to be thread-safe.  Otherwise, in the default implementation, setMenu events
	 * can happen on arbitrary threads and at arbitrary time, and the application should properly support this</p>
	 * 
	 * <p> menuTick() should simply be called ever game tick/cycle, if you choose to use it (warning, calling menuTick()
	 * Once, and then failing to call it again, will cause the Module driven UI to hang) </p>
	 */
	virtual void menuTick() = 0;
	
};

#endif //_ITB_MENU_CONTROLS_H_
