#ifndef _TB_BASE_EVENTS_H_
#define _TB_BASE_EVENTS_H_


#include "..\..\src\BrewString.h"
#include "..\..\src\BrewVector.h"
#include "..\..\src\aImage.h"


#include "ITBStringFactory.h"
#include "ITBClientControls.h"
#include "ITBClientEventListener.h"
#include "ITBMenuControls.h"
#include "ITBMenuEventListener.h"
#include "ITBSeedGenerator.h"

/*
 * This is the Base Twistbox Multiplayer Event handler.
 * 
 * The events are simply a system of cachable interfaces
 * 
 * @author E Scott Cragg.  Twistbox Client Core Engineering.
 *
 * @author Tiago Teixeira. Twistbox Client Core Engineering.
 */

class TBBaseEvents:
	public ITBStringFactory,
	public ITBClientControls,
	public ITBClientEventListener,
	public ITBMenuControls,
	public ITBMenuEventListener
{
	/**
	 *	Singleton Reference 
	 */
public:
	static TBBaseEvents* bus;

	/* ****************************************************************************** */
	/* Cachable Interface: ITBStringFactory			 						 		  */
	/* ****************************************************************************** */
private:
	static VoidVector* fpStringFactories;

public:
	static void registerEventHandler(ITBStringFactory* newHandler){
		if(!fpStringFactories)
			fpStringFactories = new VoidVector();
		bus->fpStringFactories->append((void *)newHandler);
	}

	static ITBStringFactory* getStringFactory() {
		return (ITBStringFactory*)bus;
	}

	String getString(int stringIndex);

	/* ****************************************************************************** */
	/* Cachable Interface: ITBClientControls		 						 		  */
	/* ****************************************************************************** */
private:
	static VoidVector* fpClientControls;

public:
	static void registerEventHandler(ITBClientControls* newHandler) {
		if(!fpClientControls)
			fpClientControls = new VoidVector();
		bus->fpClientControls->append((void*)newHandler);
	}
	
	static ITBClientControls* getClientControls() {
		return (ITBClientControls*)bus;
	}

	bool notifyModule(int code, int data);

	/* ****************************************************************************** */
	/* Cachable Interface: ITBClientEventListener. 							 		  */
	/* ****************************************************************************** */
private:
	static VoidVector* tbClientEventListener;

public:
	static void registerEventHandler(ITBClientEventListener* newHandler){
		if(!tbClientEventListener)
			tbClientEventListener = new VoidVector();
		bus->tbClientEventListener->append(newHandler);
	}

	static ITBClientEventListener* getClientEventListener() {
		return (ITBClientEventListener*)bus;
	}

	bool notifyClient(int code, Bbyte data);

	/* ****************************************************************************** */
	/* Cachable Interface: ITBMenuControls.		 							 		  */
	/* ****************************************************************************** */
private:
	static VoidVector* tbMenuControls;

public:
	static void registerEventHandler(ITBMenuControls* newHandler) {
		if(!tbMenuControls)
			tbMenuControls = new VoidVector();
		bus->tbMenuControls->append((void*)newHandler);
	}

	static ITBMenuControls* getMenuControls() {
		return (ITBMenuControls*)bus;
	}

	bool menuCommand(int menuCommandData);

	bool menuInput(int menuId, int menuEntryIndex, int menuEntryType, int entryData, String entryText) ;

	/* ****************************************************************************** */
	/* Cachable Interface: ITBMenuEventListener. 								 		  */
	/* ****************************************************************************** */
private:
	static VoidVector* tbMenuEventListener;

public:
	static void registerEventHandler(ITBMenuEventListener* newHandler) {
		if(!tbMenuEventListener)
			tbMenuEventListener = new VoidVector();
		bus->tbMenuEventListener->append((void*)newHandler);
	}
	
	static ITBMenuEventListener* getMenuEventListener() {
		return (ITBMenuEventListener*)bus;
	}
	
	bool setMenu(int menuId, int menuIndexSelected, int menuSize, String menuName);

	bool setMenuEntry(int menuId, int menuIndex, int entryType, String entryName, int entryData, String entryText, ConvertedBitmapInfo* entryImage);

	bool endSetMenu(int menuId);

	/******************************************************************************* */
	/* Static Interface: IFPNetworkEventListener. 						 		  */
	/* ****************************************************************************** */
private:
	static ITBSeedGenerator* seedGenerator;
	
public:
	static void registerEventHandler(ITBSeedGenerator* seedGenerator) {
		bus->seedGenerator = seedGenerator;
	}
	
	static ITBSeedGenerator* getSeedGenerator() {
		return bus->seedGenerator;
	}
	
	Bbyte getSeed();

	/****************************************************************************/
	/* Release                                                                  */
	/****************************************************************************/

	static void Release();


};

#endif
