#ifndef _CARRIER_ADAPTER_H_
#define _CARRIER_ADAPTER_H_

#include "ForPrizesAdapterConstantDefinitions.h"
#include "FPCStringsTestbed.h"
#include "ForPrizesAdapter.h"
#include "ForPrizeTestbed.h"
//#include "..\..\Std\HttpConnection.h"
#include "..\..\Std\InputStream.h"
#include "..\..\Std\IOSTreamBuf.h"
#include "..\..\Std\TBMessages.h"
#include "..\..\Std\Web.h"
#include "..\..\Core\ITBMenuControls.h"
#include "..\..\Core\ITBStringFactory.h"
//#include "..\..\Std\AMTPUtil.h"
#include "md5.h"

#include "AMTPErrors.h"
#include "AMTPGameResult.h"

#define KSIZE	 40			// size of key data
#define HASH_SIZE	16		// hash size

#define MAX_VALUE 1024

#define EVT_AMTP_RESERVED					EVT_USER + 2000

#define EVT_AMTP_EXIT						EVT_AMTP_RESERVED
#define EVT_AMTP_ERROR						EVT_AMTP_RESERVED + 1			// (wParam:errorID as listed in AMTPErrors.h)
#define EVT_AMTP_ERROR_REPORT				EVT_AMTP_RESERVED + 2

#define EVT_AMTP_WEB_RESPONSE				EVT_AMTP_RESERVED + 100
#define EVT_AMTP_WEB_PROGRESS				EVT_AMTP_RESERVED + 101
#define EVT_AMTP_REGISTER_RESPONSE			EVT_AMTP_RESERVED + 102
#define EVT_AMTP_GET_TOURNAMENTS_RESPONSE	EVT_AMTP_RESERVED + 103
#define EVT_AMTP_ENTER_TOURNAMENT_RESPONSE	EVT_AMTP_RESERVED + 104
#define EVT_AMTP_REPORT_SCORE_RESPONSE		EVT_AMTP_RESERVED + 105
#define EVT_AMTP_REPORT_FORFEIT_RESPONSE	EVT_AMTP_RESERVED + 106
#define EVT_AMTP_GET_SCORES_RESPONSE		EVT_AMTP_RESERVED + 107
#define EVT_AMTP_SET_OPTIONS_RESPONSE		EVT_AMTP_RESERVED + 108
#define EVT_AMTP_GET_PROMOTION_RESPONSE		EVT_AMTP_RESERVED + 109
#define EVT_AMTP_WAIT						EVT_AMTP_RESERVED + 110	// AW: added to wait state

#ifndef GETCLASSID
#define GETCLASSID()	((AEEApplet*)GETAPPINSTANCE())->clsID
#endif

#define DBGLOG(msg)
class CarrierAdapter:
	public ITBMenuControls
{
public:
	
enum { NUM_ERROR_BUF_CHARS		= 256 };
enum { ERROR_BUFFER_SIZE			= NUM_ERROR_BUF_CHARS * sizeof(AECHAR) };
	enum
	{
		CONNECT_TIMEOUT_DEFAULT			= 60000,		// Socket establishment timeout (milliseconds)
		IDLE_TIMEOUT_DEFAULT				= 15,				// Socket idle timeout (seconds)
		LINGER_TIMEOUT_DEFAULT 			= 15,				// PPP linger timeout (seconds)
		REQUEST_TIMEOUT_DEFAULT			= 120000,   // Request timeout (milliseconds)
		REQUEST_TIMEOUT_MINIMUM			= 60000,    // Request timeout minimum (milliseconds)
		REQUEST_RETRY_COUNT					= 4
	};


	enum { NUM_DISPLAY_BUF_CHARS	= 1028 };
	enum { DISPLAY_BUFFER_SIZE		= NUM_DISPLAY_BUF_CHARS * sizeof(AECHAR) };
	enum { ALL_TOURNAMENTS				= 0 };
	enum { NUM_TOURNAMENT_TYPES		= 2 };			// 0=all, 1=limited, 2=progressive
	enum { NUM_HISTORIES					= 10 };
	enum
	{
		MSGID_REGISTER = 1,
		MSGID_GET_TOURNAMENTS,
		MSGID_ENTER_TOURNAMENT,
		MSGID_REPORT_SCORE,
		MSGID_REPORT_FORFEIT,
		MSGID_GET_SCORES,
		MSGID_GET_OPTIONS,
		MSGID_SET_OPTIONS
	};

	enum
	{
		SCORES_TYPE_MY_CURRENT_RANK		= 1,
		SCORES_TYPE_MY_RECENT_MATCHES	= 2,
		SCORES_TYPE_MY_TOURNAMENTS		= 3,
		SCORES_TYPE_GLOBAL_RANKS		= 4
	};

	/**
	 * The ForPrizesAdapter host
	 */
	class ForPrizesAdapter* _adapter;
	class ForPrizeTestbed* owner;
	ITBStringFactory* strings;
	
	/* I/O stuff */
	//OutputStream _sendStream;
	IOStreamBuf _sendBuf;
	IOStreamBuf _recvBuf;
	//InputStream _receiveStream;
	class TBMessages* _pMsg; 
	
	/**
	 * Number of bytes in response
	 */
	int _nBytesInResponse;
	
	/**
	 * Number of bytes read
	 */
	int _nBytesRead;
	
	/**
	 * Time http request started
	 */
	long _requestStartTime;
	
	/**
	 * Type of last response received from server
	 */
	int _responseType;
	/**
	 * Type of last request to server
	 */
	int _requestType;
	
	/**
	 * Buffer to put info in when received from server
	 */
	byte _receiveBuffer[PARAM_RECEIVE_BUFFER_SIZE];
	
	
	/**
	 * Message ID (auto-increments by one for each unique message sent)
	 */
	int _msgID;
	
	/**
	 * Flag set from outside to signal a user abort
	 */
	bool _aborted;
	
	/**
	 * Current message transmission retry count
	 */
	int _retryCount;
	
	/**
	 * Request timeout in seconds
	 */
	int _requestTimeout;
	
	/**
	 * Current HTTP Connection
	 */
	//HttpConnection* _httpConnection;
	
	/**
	 * State of registration on server
	 * (PRM_REGISTER_PHONE_NEEDED, PRM_REGISTER_PIN_NEEDED,
	 * PRM_REGISTER_PIN_VALIDATED, PRM_REGISTER_DONE)
	 */
	byte _registerState;
	
	/**
	 * Whether client is in process of entering a tournament
	 */
	bool _isEnteringTournament;
	
	/**
	 * Whether client is in process of reporting a score
	 */
	bool _isReportingScore;
	
	/**
	 * Whether client is in process of getting scores
	 */
	bool _isGettingScore;
	
	/**
	 * Whether client has received next set of tournaments from server
	 */
	//bool _hasNextTournaments;
	
	/**
	 * Type of score to get
	 * (PRM_SCORES_TYPE_MY_CURRENT_RANK, PRM_SCORES_TYPE_MY_RECENT_MATCHES,
	 * PRM_SCORES_TYPE_GLOBAL_RANKS, PRM_SCORES_TYPE_MY_TOURNAMENTS)
	 */
	int _scoreType;
	
	bool _forceRegister;
	
	bool shouldSavePrefs;
	String TEXT_CONST_tne;
	String TEXT_CONST_te9;
	String TEXT_CONST_te8;

	/***********************************************************************************************/
public:
	CarrierAdapter();
	CarrierAdapter(ITBStringFactory* _strings);

	/**
	 * Set the ForPrizesAdapter instance to be used by this adapter
	 *
	 * @param adapter the ForPrizesAdapter instance
	 */
	void setForPrizesAdapter(ForPrizesAdapter* adapter) {
		this->_adapter = adapter;
	};

	/**
	 * Called on exit to release facilities
	 */
	void Release();

	/**
	 * Gets the current H2H & progressive tournaments and their associated data (news, winner, prize info & images)
	 * from the AMTP server.
	 * Should be called when user wants to play a tournament game, but before any information is displayed.
	 *
	 * @return PARAM_ERR_NONE if the request was successfully generated and sent to the
	 *         server and the response was successfully received and processed.
	 *         If a failure occurred, an error code will be returned. Error codes
	 *         can either generated locally by the client or network layer or
	 *         remotely by the AMTP server.
	 * @throws Exception if a catastrophic error occurred sending the request or
	 *                   processing the response.
	 */
	bool getTournaments();
	bool SendGetTournamentsRequest(int32 tournamentType);
	/**
	 * Used to get scores from AMTP at any time after registration.
	 *
	 * @param tournamentType Type of tournament. Value can be one of <ul>
	 *                       <li>PRM_TOURNAMENT_TYPE_H2H</li>
	 *                       <li>PRM_TOURNAMENT_TYPE_PROGRESSIVE</li></ul>
	 * @param scoreType      Type of score. Value can be one of <ul>
	 *                       <li>PRM_SCORES_TYPE_MY_CURRENT_RANK</li>
	 *                       <li>PRM_SCORES_TYPE_MY_RECENT_MATCHES</li>
	 *                       <li>PRM_SCORES_TYPE_MY_TOURNAMENTS</li>
	 *                       <li>PRM_SCORES_TYPE_GLOBAL_RANKS</li></ul>
	 * @return PARAM_ERR_NONE if the request was successfully generated and sent to the
	 *         server and the response was successfully received and processed.
	 *         If a failure occurred, an error code will be returned. Error codes
	 *         can be either generated locally by the client or network layer or
	 *         remotely by the AMTP server.
	 * @throws Exception if a catastrophic error occurred sending the request or
	 *                   processing the response.
	 */
	bool getScores(byte tournamentType, int scoreType);
	bool getScores();
	bool SendGetScoresRequest(int32 tournamentType, int32 scoreType);

	/**
	 * Should be called when starting a tournament game in order to get matched up
	 * and to let the server know that user is entered in this tournamnet.
	 *
	 * @param tournamentType int The current tournament type which we are entering.
	 *                       Either PRM_TOURNAMENT_TYPE_H2H or PRM_TOURNAMENT_TYPE_PROGRESSIVE
	 * @return int Returns PARAM_ERR_NONE if successful
	 */
	bool enterTournament(byte tournamentType);

	/**
	 * Register the user with the AMTP server.
	 *
	 * @param pin
	 * @param phoneType
	 * @param timeLeft
	 * @param optIn
	 *
	 * @return Error code.  PARAM_ERR_NONE if the request was successfully generated and sent to the
	 *         server and the response was successfully received and processed.
	 *         If a failure occurred, an error code will be returned. Error codes
	 *         are either generated locally by the client or network layer or
	 *         remotely by the AMTP server.
	 * @throws Exception if a catastrophic error occurred sending the request or
	 *                   processing the response.
	 */
	int registerPlayer(String pin, String phoneType, long timeLeft, bool optIn);

	/**
	 * send GetPromoRequest
	 * @param tournID
	 * @param promoID
	 *
	 * @throws Exception
	 */
	int sendGetPromoRequest( int tournType, int promoID);

	/**
	 * Convenience function to report the last game's score that was
	 * stored in the client after the gameStarted() & gameEnded()
	 * sequence of calls. Called by reportGameScore().
	 *
	 * @throws Exception
	 */
	bool reportScore();
	bool SendReportScoreRequest(const AMTPTournament& tour, const AMTPGameResult &result);
	void ReportGameScore(uint32 score,
			const void* pGameResults,
			unsigned lenGameResults);

	/**
	 * Return whether or not the user must enter their phone number during
	 * the registration process.  The default implementation returns false,
	 * but some (e.g., T-Mobile USA) will return true.
	 *
	 * @return boolean
	 */
	bool isPhoneNeededForRegistration();

	/**
	 * Kill the active transaction.
	 */
	void abortTransaction() {
		_aborted = true;
	};

	// Abstract, carrier-specific methods
	
	//---------------------------------------------------------------------
	
	
	/**
	 * Returns carrier ID
	 */
	virtual byte getCarrierID() = 0;

	/**
	 * Set up the active HttpRequest header.
	 *
	 * @throws IOException
	 */
	virtual void prepareCarrierHttpRequest() = 0;

	/**
	 * Prepare a request for transmission to the server.
	 *
	 * @param requestType The type of request to prepre
	 * @throws Exception If an error occurs preparing the request.
	 */
	virtual void prepareRequest( int requestType ) = 0;
	
	String getCarrierIDString();
	/**
	 * Given a phone number, return a correctly-formatted MIN
	 *
	 * @param phoneNumber
	 * @return a MIN
	 */
	String toMIN(String phoneNumber);

	//---------------------------------------------------------------------
	
	
	/**
	 * Sends subscription request using SMS shortcode
	 * Called by resubscribe().
	 *
	 * @return int Returns PARAM_ERR_NONE if successful
	 */
	virtual int sendSubscriptionRequest( String _subscriptionRequestURL ) = 0;
	
	//---------------------------------------------------------------------
	
	
	/**
	 * Handles response from server
	 *
	 * @param responseType int identifies the matching request type
	 * @param errorNum     int error number
	 * @param errorStr     String error text
	 */
	static void HandleResponse(CarrierAdapter* pClient );
	static void HandleProgress(CarrierAdapter* pClient, int percentTimeout, int bytesRead);

	// **************************************************************************
	
	virtual int carrierRegistration() = 0;
	virtual bool setMenu(int newMenuState) = 0;

	void setOwner(class ForPrizeTestbed* _owner){
		owner = _owner;
	}

	char* CreateServerURL(char *rootURL, char *mobileID, int productID, short gameType, char *gameVersion);
	String getRequestURL();
	void GetResponse();

	Web *									_pWeb;

	bool _isResponsePending;
	int _requestRetries;
	int _requestTimeoutMillis;
	void GetPromoItems();

	int						_nErrorCount;
	//uint32					_nErrorNum;
	AECHAR					_pErrorBuf[NUM_ERROR_BUF_CHARS];

	void GetRequestTimeout();
	AECHAR								_outOfSeedsErrorBuf[NUM_ERROR_BUF_CHARS];
	AECHAR * GetOutOfSeedsErrorStr() ;

	AECHAR								_pDisplayMessageBuf[NUM_DISPLAY_BUF_CHARS];
	void CheckRedirect(TBMessages* pMsg);
	
	void SetOutOfSeeds(bool outOfSeeds);
	AECHAR * GetResponseStr(int type);

// ********** PROMO DATA
	bool									_hasPromo;
	//AECHAR*									_promoStr;
	//ConvertedBitmapInfo						_promoCBI;
	//int32									_promoNext;


	char *								_pczRedirectRootURL;

	int32 SetSendBufferLength();
	AEEHANDLER  gameHandleEvent;

	//void CancelWebRequests();
	bool _isRunning;
	void GetWinnerItems() ;
	void GetNewsItems();

	AMTPTournament				_nextTournament[NUM_TOURNAMENT_TYPES];
	AMTPTournament				_enteredTournament;
	AMTPGameResult				_lastGame;

	int32									_scoreTournamentType;
	
	bool SendEnterTournamentRequest(const AMTPTournament& tour);

	bool _hasScoreResults;
	bool SendReportForfeitRequest(const AMTPTournament& tour);
	
	String* fullUrl;
};

#endif _CARRIER_ADAPTER_H_