/*=============================================================================

  Atlas Mobile Tournament Platform

  Copyright (c) 2003-2004 Atlas Mobile, Inc. All Rights Reserved.

  This software is the confidential and proprietary information of Atlas Mobile
  ("Confidential Information").  You shall not disclose such Confidential
  Information and shall use it only in accordance with the terms of the license
  agreement you entered into with Atlas Mobile.

  ATLAS MOBILE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
  NONINFRINGEMENT. ATLAS MOBILE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
  LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
  DERIVATIVES.

=============================================================================*/

#include "CfgFile.h"

#define FILEREADER_DEBUG 0

//-----------------------------------------------------------------------------
CfgFile::CfgFile()
	: _pFileReader(NULL)
{
}

//-----------------------------------------------------------------------------
bool CfgFile::Start(const char* pczFilename, bool isWritable)
{
	if (_pFileReader)
	{
		Stop();
	}
	ASSERT(!_pFileReader);

	OpenFileMode ofm;
	if (isWritable)
		ofm = _OFM_READWRITE;
	else
		ofm = _OFM_READ;

	if (_pFileReader) delete _pFileReader;
	NEW(); _pFileReader = new FileReader();
	if (!_pFileReader ||
		!_pFileReader->Start() ||
		!_pFileReader->Open(pczFilename, ofm))
	{
		Stop();
		return false;
	}
	return true;
}
//-----------------------------------------------------------------------------
bool CfgFile::Restart(const char* pczFilename, bool isWritable)
{
	if (_pFileReader)
	{
		OpenFileMode ofm;
		if (isWritable)
			ofm = _OFM_READ;
		else
			ofm = _OFM_READWRITE;

		return _pFileReader->Open(pczFilename, ofm);
	}
	return Start(pczFilename, isWritable);
}
//-----------------------------------------------------------------------------
void CfgFile::Stop()
{
	if (_pFileReader)
	{
		delete _pFileReader;
		_pFileReader = NULL;
	}
}
//-----------------------------------------------------------------------------
void CfgFile::Rewind()
{
	if (_pFileReader) _pFileReader->Rewind();
}

//-----------------------------------------------------------------------------
bool CfgFile::FindToken(const char* pczName)
{
	if (!_pFileReader)
		return false;

	char *pczToken;
	do
	{
		pczToken = _pFileReader->ReadWord("=");
		if (!pczToken)
			break;

		if (STRCMP(pczToken, pczName))	//not found
		{
			pczToken = _pFileReader->ReadLine();//go to next line
		}
		else
		{
			return true;
		}
	}
	while (pczToken);

	return false;
}

//-----------------------------------------------------------------------------
char* CfgFile::GetString(const char* pczName, char* pcBuf, int lenBuf)
{
	TRACE(( "CfgFile.GetString(): \"%s\"", pczName ));

	if (!_pFileReader) return NULL;

	if (FindToken(pczName))
	{
		DEBUG(( "CfgFile::GetString(): found" ));

		char *pczToken = _pFileReader->ReadLine();
		if (!pczToken) return NULL;

		int len = lenBuf;

		if (!pcBuf)
		{
			if (!len) len = STRLEN(pczToken) + 1;

			pcBuf = (char*)MEMALLOC(len);
		}

		ASSERT(pcBuf);
		if (!pcBuf)
		{
			DEBUG(( "CfgFile::GetString() ERROR allocating string" ));
		}
		else
		{
			STRNCPY(pcBuf, pczToken, len-1);
			pcBuf[len-1] = '\0';
		}
		return pcBuf;
	}
	return NULL;
}

//-----------------------------------------------------------------------------
bool CfgFile::GetBool(const char* pczName, bool& value)
{
	char pcValue[2];

	if (!GetString(pczName, pcValue, 2))
		return false;

	if (*pcValue=='Y' || *pcValue=='y' ||
		*pcValue=='T' || *pcValue=='t' ||
		*pcValue=='1')
	{
		value = true;
	}
	else {
		value = false;
	}

	return true;
}

//-----------------------------------------------------------------------------
bool CfgFile::SetBool(const char* pczName, bool value)
{
	DEBUG(( "CfgFile::SetBool(): \"%s\"", pczName ));
	if (!FindToken(pczName))
	{
		DEBUG(( "CfgFile::SetBool(): ERROR unable to find token" ));
		return false;
	}
	DEBUG(( "CfgFile::SetBool(): found" ));

	char ch;
	if (value)
		ch = 'Y';
	else
		ch = 'N';

	bool ok = _pFileReader->WriteChar(ch);
	_pFileReader->Rewind();//for safety, rewind after writing
	return ok;
}

//-----------------------------------------------------------------------------
bool CfgFile::SetByte(const char* pczName, byte value)
{
	DEBUG(( "CfgFile::SetByte(): \"%s\"", pczName ));
	if (!FindToken(pczName))
	{
		DEBUG(( "CfgFile::SetByte(): ERROR unable to find token" ));
		return false;
	}
	DEBUG(( "CfgFile::SetByte(): found" ));

	char ch;
	if (value == LANGUAGE_ENGLISH)
		ch = 'E';
	else if (value == LANGUAGE_SPANISH)
		ch = 'S';
	else
		ch = 'P';

	bool ok = _pFileReader->WriteChar(ch);
	_pFileReader->Rewind();//for safety, rewind after writing
	return ok;
}