/*=============================================================================

  Atlas Mobile Tournament Platform

  Copyright (c) 2003-2004 Atlas Mobile, Inc. All Rights Reserved.

  This software is the confidential and proprietary information of Atlas Mobile
  ("Confidential Information").  You shall not disclose such Confidential
  Information and shall use it only in accordance with the terms of the license
  agreement you entered into with Atlas Mobile.

  ATLAS MOBILE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
  NONINFRINGEMENT. ATLAS MOBILE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
  LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
  DERIVATIVES.

=============================================================================*/

#ifndef IOSTREAMBUF_H
#define IOSTREAMBUF_H

#include "AEE.h"
#include "AEEAppGen.h"
#include "AEEGraphics.h"
#include "AEEStdLib.h"

#include "DbgLog.h"
#include "Debug.h"
#include "Memory.h"
#include "PlatformBREW.h"
#include "AEEComDef.h"

//#define IOSTREAMBUF_DYNAMICALLOC


class IOStreamBuf
{
public:
	IOStreamBuf();
	~IOStreamBuf() { Stop(); }

	bool Start(uint32 bufsize);
	void Stop();

	bool Seek(uint32 offset);
	void SeekEnd() { _offset=_length; }
	void Reset();
	bool IsEmpty() { return _offset >= _length; }

	uint32 Mark() { return _offset; };
	uint32 MarkLength(uint32 mark) { return _offset>mark ? _offset-mark : 0; };

	bool WriteUint32AtMark(uint32 mark, uint32 data);

	uint32 GetLength() { return _length; }
	const uint8* GetBuf() { return _pcBuf; }

	void EnsureCapacity(uint32 Capacity);

	//input
	bool ReadBytes(uint8* data, int len);
	bool ReadChars(char* data);
	bool ReadBool(bool& data);
	bool ReadInt8(int8& data);
	bool ReadUint8(uint8& data);
	bool ReadInt16(int16& data);
	bool ReadUint16(uint16& data);
	bool ReadInt32(int32& data);
	bool ReadUint32(uint32& data);
	bool ReadInt64(int64& data);

	//output
	bool WriteBytes(const uint8* data, int len);
	bool WriteChars(const char* data);
	bool WriteBool(bool data);
	bool WriteInt8(int8 data);
	bool WriteUint8(uint8 data);
	bool WriteInt16(int16 data);
	bool WriteUint16(uint16 data);
	bool WriteInt32(int32 data);
	bool WriteUint32(uint32 data);
	bool WriteInt64(const int64& data);

protected:
	uint8* _pcBuf;
	uint32 _offset, _length, _max;
};

//-----------------------------------------------------------------------------
//	input operator>>
//-----------------------------------------------------------------------------
inline IOStreamBuf& operator>>(IOStreamBuf& iosbuf, char* data)
{
	iosbuf.ReadChars(data); return iosbuf;
}
inline IOStreamBuf& operator>>(IOStreamBuf& iosbuf, bool& data)
{
	iosbuf.ReadBool(data); return iosbuf;
}
inline IOStreamBuf& operator>>(IOStreamBuf& iosbuf, int8& data)
{
	iosbuf.ReadInt8(data); return iosbuf;
}
inline IOStreamBuf& operator>>(IOStreamBuf& iosbuf, uint8& data)
{
	iosbuf.ReadUint8(data); return iosbuf;
}
inline IOStreamBuf& operator>>(IOStreamBuf& iosbuf, int16& data)
{
	iosbuf.ReadInt16(data); return iosbuf;
}
inline IOStreamBuf& operator>>(IOStreamBuf& iosbuf, uint16& data)
{
	iosbuf.ReadUint16(data); return iosbuf;
}
inline IOStreamBuf& operator>>(IOStreamBuf& iosbuf, int32& data)
{
	iosbuf.ReadInt32(data); return iosbuf;
}
inline IOStreamBuf& operator>>(IOStreamBuf& iosbuf, uint32& data)
{
	iosbuf.ReadUint32(data); return iosbuf;
}
inline IOStreamBuf& operator>>(IOStreamBuf& iosbuf, int64& data)
{
	iosbuf.ReadInt64(data); return iosbuf;
}

//-----------------------------------------------------------------------------
//	output operator<<
//-----------------------------------------------------------------------------
inline IOStreamBuf& operator<<(IOStreamBuf& iosbuf, char* data)
{
	iosbuf.WriteChars(data); return iosbuf;
}
inline IOStreamBuf& operator<<(IOStreamBuf& iosbuf, bool data)
{
	iosbuf.WriteBool(data); return iosbuf;
}
inline IOStreamBuf& operator<<(IOStreamBuf& iosbuf, int8 data)
{
	iosbuf.WriteInt8(data); return iosbuf;
}
inline IOStreamBuf& operator<<(IOStreamBuf& iosbuf, uint8 data)
{
	iosbuf.WriteUint8(data); return iosbuf;
}
inline IOStreamBuf& operator<<(IOStreamBuf& iosbuf, int16 data)
{
	iosbuf.WriteInt16(data); return iosbuf;
}
inline IOStreamBuf& operator<<(IOStreamBuf& iosbuf, uint16 data)
{
	iosbuf.WriteUint16(data); return iosbuf;
}
inline IOStreamBuf& operator<<(IOStreamBuf& iosbuf, int32 data)
{
//	TRACE("operator<<(IOStreamBuf&, int32)");
	iosbuf.WriteInt32(data); return iosbuf;
}
inline IOStreamBuf& operator<<(IOStreamBuf& iosbuf, uint32 data)
{
//	TRACE("operator<<(IOStreamBuf&, uint32)");
	iosbuf.WriteUint32(data); return iosbuf;
}
inline IOStreamBuf& operator<<(IOStreamBuf& iosbuf, int64& data)
{
	iosbuf.WriteInt64(data); return iosbuf;
}

#endif //IOSTREAMBUF_H

