//=============================================================================
//	PlatformBREW
//
//	Abstraction Layer for BREW API
//=============================================================================
#ifndef PLATFORMBREW_H
#define PLATFORMBREW_H

#include "Memory.h"
#include "AEEAppGen.h"
#include "AEEStdLib.h"

typedef enum _PLATFORM_ID {
	// NULL platform id used for unsupported devices:
	PLATFORM_ID_NULL		= 0,
	// Generic platform id for uncustomized devices:
	PLATFORM_ID_GENERIC		= 1000,

	// Brew 1.0
	PLATFORM_ID_Z800		= 4000,
	PLATFORM_ID_CDM7900		= 4001,
	PLATFORM_ID_QCP3035		= 5000,

	// Brew 1.1
	PLATFORM_ID_VX10		= 1001,
	PLATFORM_ID_VX4400		= 1009,
	PLATFORM_ID_A530		= 2013,
	PLATFORM_ID_A605		= 2023,
	PLATFORM_ID_A610		= 2025,
	PLATFORM_ID_A690		= 2027,
	PLATFORM_ID_QCP3225		= 5005,
	PLATFORM_ID_T720		= 8003,
	PLATFORM_ID_C343		= 8009,
	PLATFORM_ID_T732		= 8018,
	PLATFORM_ID_T606		= 12000,
	PLATFORM_ID_CDM9500		= 13000,
	PLATFORM_ID_CDM9500R	= 13003,
	PLATFORM_ID_3589I		= 17000,
	PLATFORM_ID_3589I_EM	= 17017,	// A little endian variation for emulator.

	// Brew 1.2
	PLATFORM_ID_6585		= 17001,

	// Brew 2.0
	PLATFORM_ID_VX6000		= 1014,
	PLATFORM_ID_VX4600		= 1024,
	PLATFORM_ID_VX4500		= 1025,
	PLATFORM_ID_VX7000		= 1036,
	PLATFORM_ID_VX5450		= 1037,
	PLATFORM_ID_A650		= 2034,
	PLATFORM_ID_A670		= 2044,
	PLATFORM_ID_A790		= 2048,
	PLATFORM_ID_SE47		= 5013,
	PLATFORM_ID_KX414		= 5014,
	PLATFORM_ID_SE47_B		= 5024,
	PLATFORM_ID_E310		= 8005,
	PLATFORM_ID_V810		= 8010,
	PLATFORM_ID_V710		= 8034,
	PLATFORM_ID_CDM9900		= 13005,
	PLATFORM_ID_CDM8600		= 16006,
	PLATFORM_ID_CDM8400		= 16010,
	PLATFORM_ID_CDM8900		= 16014,
	PLATFORM_ID_CDM8410		= 16025

} PLATFORM_ID;


typedef struct _PlatformIdNamePair
{
	PLATFORM_ID		ePlatformId;
	char*			name;
} 
PlatformIdNamePair;


typedef struct _BREW11Device
{
	int x, y, bits, RAM;
	uint32 vMajor, vMinor, vSub;
	PLATFORM_ID ePlatformId;
} 
BREW11Device;


#ifndef GETSHELL
#define GETCLASSID()	((AEEApplet*)GETAPPINSTANCE())->clsID
#define GETSHELL()		((AEEApplet*)GETAPPINSTANCE())->m_pIShell
#define GETDISPLAY()	((AEEApplet*)GETAPPINSTANCE())->m_pIDisplay
#endif

#ifdef __cplusplus
extern "C"
{
#endif

char * 		Platform_GetMIN();
uint32		Platform_GetPhoneID();
char *		Platform_GetPhoneName(char *nameBuf);
boolean 	Platform_GetScreenSize(int16* piWidth, int16* piHeight);

#ifdef __cplusplus
}
#endif

#endif // PLATFORMBREW_H

