/*=============================================================================

  Atlas Mobile Tournament Platform

  Copyright (c) 2003-2004 Atlas Mobile, Inc. All Rights Reserved.

  This software is the confidential and proprietary information of Atlas Mobile
  ("Confidential Information").  You shall not disclose such Confidential
  Information and shall use it only in accordance with the terms of the license
  agreement you entered into with Atlas Mobile.

  ATLAS MOBILE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
  NONINFRINGEMENT. ATLAS MOBILE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
  LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
  DERIVATIVES.

=============================================================================*/

#ifndef AMTPREQUESTS_H
#define AMTPREQUESTS_H

#include "TBAtoms.h"
#include "../ForPrizes/Core/AMTPTournament.h"

class AMTPTournament;

typedef struct _ResponseItem
{
	bool isSet;
	void* data;
	union
	{
		int16 i16;
		int32 i32;
		int64 i64;
	};

} ResponseItem;

typedef struct _PackageHeader
{
	uint32	length;
	uint32	atomType;
	byte	retryCount;
	byte	padding;
	uint16	numSubAtoms;
} 
PackageHeader;

class TBMessages
{
public:
	enum {
		IDX_NAMED_MSGID = 0,
		IDX_NAMED_MIN,
		IDX_NAMED_USERNAME,
		IDX_NAMED_PHONETYPE,
		IDX_NAMED_PREFS,
		IDX_NAMED_OPTIN,
		IDX_NAMED_GAMEID,
		IDX_NAMED_PRODID,
		IDX_NAMED_GAMETYPE,
		IDX_NAMED_GAMEVERS,
		IDX_NAMED_TOURNTYPE,
		IDX_NAMED_TOURNID,
		IDX_NAMED_GAMEPLAY,
		IDX_NAMED_SCORE,
		IDX_NAMED_SCORETYPE,
		IDX_NAMED_STARTTIME,
		IDX_NAMED_ENDTIME,
		IDX_NAMED_ELAPSED,
		IDX_NAMED_PLAYDATA,
		IDX_NAMED_ASSET_CLASS,
		IDX_NAMED_PRIZE_STR,
		IDX_NAMED_PRIZE_IMG,
		IDX_NAMED_NEWS_STR,
		IDX_NAMED_NEWS_IMG,
		IDX_NAMED_WINNER_STR,
		IDX_NAMED_WINNER_IMG,
		IDX_NAMED_REQUEST_TIMEOUT,
		IDX_NAMED_PROMOTION_ID,
		IDX_NAMED_TBDATA,
		IDX_NAMED_SEED,
		IDX_NAMED_NULL,
		NUM_NAMES
		};
	enum
	{
		IDX_RESP_MSGID = 0,
		IDX_RESP_ERRNUM,
		IDX_RESP_ERRMSG,
		IDX_RESP_CERTIFICATE,
		IDX_RESP_REDIRECT,
		IDX_RESP_SUGGESTED_NAME,
		IDX_RESP_SERVER_TIME,
		IDX_RESP_MATCHSUM,
		IDX_RESP_RANKSUM,
		IDX_RESP_SUM,
		IDX_RESP_TOURNID,
		IDX_RESP_TOURNDESC,
		IDX_RESP_TOURNLIST,
		IDX_RESP_HISTORY,
		IDX_RESP_SEED,
		IDX_RESP_PARAMSTRING,
		IDX_RESP_NEWS_STR,
		IDX_RESP_NEWS_IMG,
		IDX_RESP_WINNER_STR,
		IDX_RESP_WINNER_IMG,
		IDX_RESP_REQUEST_TIMEOUT,
		IDX_RESP_PROMO_STR,
		IDX_RESP_PROMO_IMG,
		IDX_RESP_PROMO_NEXT,
		IDX_RESP_NULL,
		NUM_RESPONSEITEMS
	};

protected:
	enum {
		RESP_TYPE_NONE = 0,
		RESP_TYPE_STRING,
		RESP_TYPE_BINARY,
		RESP_TYPE_INT16,
		RESP_TYPE_INT32,
		RESP_TYPE_INT64,
		RESP_TYPE_COMPOUND,
		RESP_TYPE_HISTORY,
		RESP_TYPE_PACKET,
		RESP_TYPE_ATOM
		};
	enum
	{
		IDX_COMPOUND_TTYP = 0,
		IDX_COMPOUND_TOUR,
		IDX_COMPOUND_DESCR,
		IDX_COMPOUND_PRODID,
		IDX_COMPOUND_GAMEID,
		IDX_COMPOUND_GAMEVER,
		IDX_COMPOUND_GAMESTART,
		IDX_COMPOUND_GAMEEND,
		IDX_COMPOUND_SEED,
		IDX_COMPOUND_PARAM,
		IDX_COMPOUND_PRIZE_STR,
		IDX_COMPOUND_PRIZE_IMG,
		IDX_COMPOUND_TOURNAMENT_STATUS_STR,
		IDX_COMPOUND_NULL,
		NUM_COMPOUNDITEMS,			//the num of fields in a 'trna'
		NUM_COMPOUNDS = 10			//the num of tournaments in a "[tours" list
	};
	enum
	{
		IDX_HISTORY_HEADER = 0,
		IDX_HISTORY_SUMMARY,
		IDX_HISTORY_NULL,
		NUM_DETAILITEMS,			// the num of fields in a 'tdet'
		NUM_HISTORIES = 10			// the num of histories in a "[tlist" list
	};

	enum { LEN_DUMMY = 0 };
	enum { LEN_NAMEBUF = 64 };
	enum { LEN_STRINGBUF = 8192 };
	enum { LEN_MAXATOMLENGTH = (24 * 1024) };

public:
	TBMessages();
	~TBMessages();

	bool Start();
	void Stop();

	void FreeMemory();

	uint32 WriteAtomHeader(IOStreamBuf& buf, int32 len, int32 type);
	uint32 WriteNamedAtomHeader(IOStreamBuf& buf, int idxName);

	void WriteAtom(IOStreamBuf& buf, AtomText& text);
	void WriteAtom(IOStreamBuf& buf, bool data);
	void WriteAtom(IOStreamBuf& buf, int16 data);
	void WriteAtom(IOStreamBuf& buf, int32 data);
	void WriteAtom(IOStreamBuf& buf, int64& data);
	void WriteAtom(IOStreamBuf& buf, const void* data, int32 len);
	void WriteNamedAtom(IOStreamBuf& buf, int idxName, AtomText& text);
	void WriteNamedAtom(IOStreamBuf& buf, int idxName, bool data);
	void WriteNamedAtom(IOStreamBuf& buf, int idxName, int16 data);
	void WriteNamedAtom(IOStreamBuf& buf, int idxName, int32 data);
	void WriteNamedAtom(IOStreamBuf& buf, int idxName, int64& data);
	void WriteNamedAtom(IOStreamBuf& buf, int idxName, const void* data, int32 len);

	void WriteMessageID(IOStreamBuf& buf);
	void WriteMIN(IOStreamBuf& buf, const char* pcMIN);

	void WritePrefsAtom(IOStreamBuf& buf, bool isOptIn);
	//void WriteSeedAtom(IOStreamBuf& buf, int64 seed, const char* pParam);
	//void WriteSeedAtom(IOStreamBuf& buf, int64 seed, const byte* pParam, int32 lenParam);

	bool ReadAtomHeader(IOStreamBuf& buf, int32& len, int32& type);
	int16 ReadAtomText(IOStreamBuf& buf, const int32 lenBuf, char* pcBuf);

	int32 GetResponseSize(IOStreamBuf& buf);

	bool ParseResponse(IOStreamBuf& buf);
	bool ReadAtom(IOStreamBuf& buf);
	bool ReadNamedAtomData(IOStreamBuf& buf);
	bool ReadNamedAtom(IOStreamBuf& buf);
	bool ReadNamedBody(IOStreamBuf& buf);
	bool ReadListBody(IOStreamBuf& buf, int& idx);
	bool ReadMapBody(IOStreamBuf& buf, int& idx);

	void ReadCertificate(IOStreamBuf& buf);

	bool	GetInt32(int idxItem, int32& i);
	void *	GetResponseData(int idxString, int& lenBuf);
	void *	GetResponseData(int idxString, int& lenBuf, bool& wasSet);
	bool	GetErrorMessage(uint32 &nError, char* pErrorMessage, int lenBuf);
	bool	GetMsgID(uint32 &idMsg);
	int32	GetResponseType();
	char *	GetRedirectURL();
	int64	GetServerTime();
	bool	GetTournament(int typeTournament, AMTPTournament& tour);
	void	ResetHistoryList();
	char *	GetHistoryHeader(int idx);
	char *	GetHistorySummary(int idx);
	void	FreeResponseData(int idxString);

protected:

	uint32	_nMessageID;
	char	_pczMessageID[32];

	char*	_pNameBuf;
	char*	_pStringBuf;

	char*	_pCertificateBuf;
	int		_lenCertificateBuf;
	int		_lenCertificate;

	int32	_responseType;

	const int _lenMaxAtom;

	int _numNames;
	int _numResponses;
	int _numCompounds;
	int _numDetails;

	const char* _pcNames[NUM_NAMES];
	const char* _pcResponseNames[NUM_RESPONSEITEMS];
	const char* _pcCompoundNames[NUM_COMPOUNDITEMS];
	const char* _pcHistoryNames[NUM_DETAILITEMS];

	int _pResponseTypes[NUM_RESPONSEITEMS];
	int _pCompoundTypes[NUM_COMPOUNDITEMS];
	int _pHistoryTypes[NUM_DETAILITEMS];

	//The following stores the response data
	ResponseItem _responseItem[NUM_RESPONSEITEMS];
	ResponseItem _compoundItems[NUM_COMPOUNDS][NUM_COMPOUNDITEMS];

	ResponseItem _historyItems[NUM_HISTORIES][NUM_DETAILITEMS];

	bool _isInCompound;
	int _idxCompoundAtom;
	bool _isInHistory;
	int _idxHistoryAtom;

	void WriteMessageID(IOStreamBuf& buf, int nMsgID);

#if defined( _DEBUG ) || DBGLOG_ENABLED
protected:
	void DebugPrintType(uint32 type);
	void DebugIndent(int inc);
	int _dbgIn;
	char* _in;
#endif
#ifdef _ARMDEBUG
protected:
	void DebugText(const char* pChars);
#endif
};

#endif //AMTPREQUESTS_H
