/*=============================================================================

  Atlas Mobile Tournament Platform

  Copyright (c) 2003-2004 Atlas Mobile, Inc. All Rights Reserved.

  This software is the confidential and proprietary information of Atlas Mobile
  ("Confidential Information").  You shall not disclose such Confidential
  Information and shall use it only in accordance with the terms of the license
  agreement you entered into with Atlas Mobile.

  ATLAS MOBILE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
  NONINFRINGEMENT. ATLAS MOBILE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
  LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
  DERIVATIVES.

=============================================================================*/

#ifndef WEB_H
#define WEB_H

#include "AEE.h"
#include "AEEAppGen.h"
#include "AEEGraphics.h"
#include "AEEStdLib.h"

#include "DbgLog.h"
#include "Debug.h"
#include "Memory.h"
#include "PlatformBREW.h"
#include "../ForPrizes/Core/AMTPErrors.h"

#include "AEEWeb.h"
#include "AEENet.h"

#include "IOStreamBuf.h"

// Defined in BREW 2.x header but not 1.x
#ifndef WEBCODE_MULTIPLECHOICES
#define WEBCODE_MULTIPLECHOICES		300
#endif
#ifndef WEBCODE_MOVEDPERMANENTLY
#define WEBCODE_MOVEDPERMANENTLY	301
#endif
#ifndef WEBCODE_FOUND
#define WEBCODE_FOUND				302
#endif
#ifndef WEBCODE_SEEOTHER
#define WEBCODE_SEEOTHER			303
#endif
#ifndef WEBCODE_NOTMODIFIED
#define WEBCODE_NOTMODIFIED			304
#endif
#ifndef WEBCODE_USEPROXY
#define WEBCODE_USEPROXY			305
#endif
#ifndef WEBCODE_UNUSED
#define WEBCODE_UNUSED				306
#endif
#ifndef WEBCODE_TEMPORARYREDIRECT
#define WEBCODE_TEMPORARYREDIRECT	307
#endif

//-----------------------------------------------------------------------------
class Web
{
public:
	typedef void (*ProgressFnPtr)(void* pUser, int percentTimeout, int bytesRead);

	enum { READ_BUFFER_LEN = 256 };
	enum { ERROR_MSG_LEN = 128 };

	enum
	{
		TIMEOUT_TIMER_INTERVAL 	= 1000		// Timeout timer interval (millseconds)
	};
	enum
	{
		STATUS_STOPPED = 0,

		STATUS_FINDHOST = 1000,
		STATUS_CONNECTING,
		STATUS_SENDING,
		STATUS_RECEIVING,
		STATUS_WAITING,
		STATUS_READING,
		STATUS_DONE,
		STATUS_ERROR,

		STATUS_REDIRECTING,
		STATUS_STARTING,
		STATUS_CANCELLED,
		STATUS_CACHEHIT,
		STATUS_UNKNOWN
	};

public:	//static class methods
	static void HandleResponse(Web* pWeb);
	static void HandleHeader(Web* pWeb, const char *pcName, GetLine *pglMimeType);
	static void HandleStatus(Web* pWeb, WebStatus webs, void *pVal);
	static void HandleRequestTimeoutTimer(Web* pWeb);

public:
	Web(int connectTimeoutMillis, int idleTimeoutSecs, int lingerTimeoutSec);
	~Web();

	bool Start(IShell* pShell, IOStreamBuf* pStreamBuf, PFNNOTIFY pNotifyFn, void* pNotifyObj);
	void Stop();

	bool SendRequest(const char* pcURL, const uint8* pcData, int32 lenData, uint32 requestTimeoutMillis);
	void DeleteRequest();

	void SetProgressHandler(ProgressFnPtr pProgressFn, void* pProgressObj);

	int GetStatus() { return _status; };
	bool GetErrorMessage(uint32 &nError, char* pErrorMessage, int bufLen);
	char *GetRedirectRootURL() { return _redirectRootURL; };

protected:

	bool CreateIWeb();
	void SetConnectionLingerTimeout(int lingerTimeoutSeconds);
	void Notify();
	void UpdateProgress();
	void StartRequestTimeoutTimer(int requestTimeoutMillis);
	void CancelRequestTimeoutTimer();
	void SetError(WebRespInfo* pInfo);
	void SetTimeoutError();

protected:

	bool 			_isRunning;
	bool			_isT720;

	int				_connectTimeoutMillis;
	int				_idleTimeoutSec;
	int				_lingerTimeoutSecs;
	int				_requestTimeoutMillis;

	int 			_status;
	bool 			_responsePending;
	char*			_redirectRootURL;

	IWeb*			_pIWeb;
	IWebResp*		_pIWebResp;
	IPeek*			_pIPeek;

	int 			_readBufLen;
	char* 			_pReadBuf;
	IOStreamBuf* 	_pRecvBuf;

	AEECallback		_cbHandleResponse;

	PFNNOTIFY 		_pNotifyFn;
	void* 			_pNotifyObj;

	ProgressFnPtr 	_pProgressFn;
	void* 			_pProgressObj;

	int 			_bytesRead;
	int 			_nTimerCountdown;

	int 			_nErrorNum;
	char 			_pErrorMsg[ERROR_MSG_LEN];
};

#endif //WEB_H
