/*=============================================================================

  Atlas Mobile Tournament Platform

  Copyright (c) 2003-2004 Atlas Mobile, Inc. All Rights Reserved.

  This software is the confidential and proprietary information of Atlas Mobile
  ("Confidential Information").  You shall not disclose such Confidential
  Information and shall use it only in accordance with the terms of the license
  agreement you entered into with Atlas Mobile.

  ATLAS MOBILE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
  NONINFRINGEMENT. ATLAS MOBILE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
  LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
  DERIVATIVES.

=============================================================================*/

#ifndef TBUTIL_H
#define TBUTIL_H
#include "AEE.h"
#include "AEEAppGen.h"
#include "AEEGraphics.h"
#include "AEE.h"
#include "AEEAppGen.h"
#include "AEEGraphics.h"
#include "AEEStdLib.h"
#include "PlatformBREW.h"
#include "..\..\Module\Std\Debug.h"
//#include "AMTPFramework.h"

typedef struct 
{
	void *			bitmap;
	AEEImageInfo	bitmapInfo;
	boolean			memAlloc;
}
ConvertedBitmapInfo;


#ifdef __cplusplus
extern "C"
{
#endif


/**
 * Get the name of the main AMTP resource file.
 */
char * TBUTIL_GetResFile();


/**
 * Get the name of the ancilliary AMTP game-specific resource file.
 */
char * TBUTIL_GetGameResFile();


/**
 * Converts a version string to a version number in the format required by
 * the AMTP server.
 *
 * Params:
 *   versionStr - A string of the form major.minor.build. All three
 *                components must exist in the string (e.g. "1.0.4").
 *
 * Returns:
 *    An int32 containing the version number in the format required by the
 *    AMTP server. The major version is stored in the first 2 bytes, the
 *    minor version is stored in the next two bytes, and the build number
 *    is stored in the last 4 bytes.
 */
int32 TBUTIL_CreateVersionNumber(char *versionStr);


/**
 * Converts an 8-bit character to a 16-bit character.
 *
 * NOTE: This function could stand some optimization.
 *
 * Params:
 *   in_Char - Input 8-byte character.
 *
 * Returns:
 *    Converted wide 16-byte character.
 */
AECHAR TBUTIL_CharToWchar( const char in_Char );


/**
 * Converts an 8-bit character string to a 16-bit character string.
 *
 * Patches a bug that appears in the Brew string handling for Nokia 3589i.
 * The runtime code for this phone treats a newline at the begining of a
 * line (beginning of string or right after another newline) as a string
 * terminator.  The workaround applied here is to insert an extra space
 * character before every newline character.  This seems a safe for all
 * phones wherever STRTOWSTR would be used but we could test whether this
 * is a 3589i and divert to STRTOWSTR otherwise.  This function may also
 * differ from STRTOWSTR in that it will fill and terminate the output
 * buffer up to in_BufSize and still return out_pBuf without complaint
 * if the buffer was not big enough.
 */
AECHAR *TBUTIL_StrToWstr( const char *in_pSz, AECHAR *out_pBuf, int in_BufSize );


/**
 * Load a resource string into a single-byte character buffer. The returned buffer
 * must be  freed by the caller using MEMFREE.
 *
 * Params:
 *   rsrcFile	- Name of the resource file containing resource
 *   rsrcID		- Resource ID of string to load
 */
char * TBUTIL_LoadResStr(char *rsrcFile, int rsrcID);

/**
 * Load a resource string into a double-byte AECHAR. The returned buffer must be 
 * freed by the caller using MEMFREE.
 *
 * Params:
 *   rsrcFile	- Name of the resource file containing resource
 *   rsrcID		- Resource ID of string to load
 */
AECHAR * TBUTIL_LoadResWStr(char *rsrcFile, int rsrcID);

/**
 * Load a resource string into a single byte character buffer with optional new-line
 * character insertion. The returned buffer must be freed by the caller using
 * MEMFREE.
 *
 * Params:
 *   rsrcFile		 - Name of the resource file containing resource
 *   rsrcID          - Resource ID of string to load
 *   insertNewLines  - Indicated whether new lines should be inserted into the string
 *   replacementChar - Character that will be replaced with new lines. If insertNewLines
 *                     is false, this parameter is ignored.
 */
char * TBUTIL_LoadResStrNL(char *rsrcFile, int rsrcID, bool insertNewLines, char replacementChar);

/**
 * Load a resource string into a double-byte AECHAR buffer with optional new-line
 * character insertion. The returned buffer must be freed by the caller using
 * MEMFREE.
 *
 * Params:
 *   rsrcFile		 - Name of the resource file containing resource
 *   rsrcID          - Resource ID of string to load
 *   insertNewLines  - Indicated whether new lines should be inserted into the string
 *   replacementChar - Character that will be replaced with new lines. If insertNewLines
 *                     is false, this parameter is ignored.
 */
AECHAR * TBUTIL_LoadResWStrNL(char *rsrcFile, int rsrcID, bool insertNewLines, char replacementChar);

/**
 * Replaces all instances of the specified character with new-line characters.
 * Useful for adding new line characters to string resources.
 */
void TBUTIL_InsertNewLines(AECHAR *buffer, char replaceChar);

/**
 * Convert raw device-independent image data into a device-dependent bitmap.
 */
bool TBUTIL_CreateBitmap(ConvertedBitmapInfo *cbi, void *rawData, int rawDataSize);

/**
 * Release a device-dependent bitmap created with AMTPUTIL_CreateBitmap.
 */
void TBUTIL_ReleaseBitmap(ConvertedBitmapInfo *cbi);


#ifdef __cplusplus
} //extern "C"
#endif

#endif // AMTPUTIL_H
