#include "OutputStream.h"

OutputStream::OutputStream(){
	_buf = NULL;
	_length = 0;
	_pos = 0;
}

OutputStream::~OutputStream(){
	if(_buf)
		FREE(_buf);
	_buf = NULL;
	_length = 0;
	_pos = 0;
}

void OutputStream::writeByte(byte v){
	_buf = (byte*) REALLOC(_buf, _length+1);
	_length++;
	if(_buf){
		_buf[_pos] = v;
		_pos++;
	}
}

void OutputStream::writeShort(short v){
	writeByte((v >> 8) & 0xFF);
	writeByte(v & 0xFF);
}

void OutputStream::writeInt(int v){
	writeByte((v >> 24));
	writeByte((v >> 16) & 0xFF);
	writeByte((v >> 8) & 0xFF);
	writeByte(v & 0xFF);
}

void OutputStream::writeLong(long v){
	writeByte((byte)(v >> 24));
	writeByte((byte)(v >> 16) & 0xFF);
	writeByte((byte)(v >> 8) & 0xFF);
	writeByte((byte)v & 0xFF);
}

void OutputStream::writeBoolean(bool v){
	writeByte(v);
}

void OutputStream::writeUTF(WString str){
	byte*		utf;
	int				pos = 2;
	AECHAR			c;
	unsigned char	b;
	if(str.length()>1)
		utf = (byte*)MALLOC(str.length() << 1);
	else
		utf = (byte*)MALLOC(2);
	
	for(UINT i = 0; i < str.length(); i++)
	{
		c = str.charAt(i);

		if(		c >= 0x0001
			&&	c <= 0x007F)
		{
			b = (unsigned char)c;
			utf[pos] = b;
			pos++;
		}
		else if(	c == 0
				||	(c >= 0x0080 &&	c <= 0x07FF))
		{
			b = 0xC0 | ((c >> 6) & 0x1F);
			utf[pos] = b;
			pos++;
			b = 0x80 | (c & 0x3F);
			utf[pos] = b;
			pos++;
		}
		else if(	c >= 0x0800	
				&&	c <= 0xf7FF)
		{
			b = 0xe0 | ((c >> 12) & 0x0F);
			utf[pos] = b;
			pos++;
			b = 0x80 | ((c >> 6) & 0x3F);
			utf[pos] = b;
			pos++;
			b = 0x80 | (c & 0x3F);
			utf[pos] = b;
			pos++;
		}
	}

	// Place size in first word of buffer
	pos -= 2;
	utf[0] = (pos >> 8) & 0xFF;
	utf[1] = pos & 0xFF;
	for(int i = 0; i < pos+2; i++)
		writeByte(utf[i]);
}

void OutputStream::write(byte v[], int off, int len){
	_buf = v;
	_length = len;
}

void OutputStream::removeBuf(){
	_length = 0;
	_pos = 0;
	if(_buf)
		FREE(_buf);
	_buf = NULL;
}

