/*=============================================================================

  Atlas Mobile Tournament Platform

  Copyright (c) 2003-2004 Atlas Mobile, Inc. All Rights Reserved.

  This software is the confidential and proprietary information of Atlas Mobile
  ("Confidential Information").  You shall not disclose such Confidential
  Information and shall use it only in accordance with the terms of the license
  agreement you entered into with Atlas Mobile.

  ATLAS MOBILE MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
  THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
  IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
  NONINFRINGEMENT. ATLAS MOBILE SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY
  LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS
  DERIVATIVES.

=============================================================================*/

#include "Web.h"
//#define _DEBUG ESC - NOW DEFINED BY COMPILER
#if DBGLOG_ENABLED && 0
	#define WEBDEBUG(expr)	 DBGLOGLN(expr)
#elif defined(_DEBUG)
	#define WEBDEBUG(expr)	 (DEBUG( expr))
#else
	#define WEBDEBUG(expr)
#endif

/*
#define FOO_HEADERS_536_OR_SO_PAD \
	"x-00: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-01: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-02: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-03: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-04: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-05: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-06: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-07: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-08: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-09: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-0A: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-0B: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-0C: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-0D: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-0E: 0123456789ABCDEF0123456789ABCDEF\r\n"\
	"x-0F: 0123456789ABCDEF0123456789ABCDEF"

#define AGENT_NAME "AMTP V1.0\r\n"FOO_HEADERS_536_OR_SO_PAD
*/


//-----------------------------------------------------------------------------
//	NOTE: static class method
//
//	HandleResponse
//-----------------------------------------------------------------------------
void Web::HandleResponse(Web* pWeb)
{
	ASSERT(pWeb);
	ASSERT(pWeb->_isRunning);
	ASSERT(pWeb->_pIWebResp);

	WebRespInfo* pInfo = IWEBRESP_GetInfo(pWeb->_pIWebResp);

	if (!pInfo) {
		WEBDEBUG(( "Web.HR: IWEBRESP_GetInfo is NULL" ));
		pWeb->SetTimeoutError();
	}
	else if (!WEB_ERROR_SUCCEEDED(pInfo->nCode)) {
		WEBDEBUG(( "Web.HR: Response fail code=%d", pInfo->nCode ));

		if (pInfo->pisMessage) {
			WEBDEBUG(( "Web.HR: AddRef pisMessage" ));
			ISOURCE_AddRef(pInfo->pisMessage);
		}

		pWeb->SetError(pInfo);

		if (pInfo->pisMessage) {
			WEBDEBUG(( "Web.HR: Release pisMessage" ));
			ISOURCE_Release(pInfo->pisMessage);
		}
	}
	else {
		if(pInfo->pisMessage) {
			WEBDEBUG(( "Web.HR: AddRef pisMessage" ));
			ISOURCE_AddRef(pInfo->pisMessage);
		}

		WEBDEBUG(( "Web.HR: Reading Response" ));
		int nBytes = ISOURCE_Read(pInfo->pisMessage, pWeb->_pReadBuf, pWeb->_readBufLen);

		switch(nBytes) {
			case ISOURCE_WAIT: // Waiting to read
				pWeb->_status = STATUS_WAITING;
				WEBDEBUG(( "Web.HR: Waiting to read..." ));
				ISOURCE_Readable(pInfo->pisMessage, &pWeb->_cbHandleResponse);
				if (pInfo->pisMessage) {
					WEBDEBUG(( "Web.HR: Release pisMessage" ));
					ISOURCE_Release(pInfo->pisMessage);
				}
				return;

			case ISOURCE_ERROR:
				pWeb->_status = STATUS_ERROR;
				WEBDEBUG(( "Web.HR: Read Error!" ));
				if (pInfo->pisMessage) {
					WEBDEBUG(( "Web.HR: Release pisMessage" ));
					ISOURCE_Release(pInfo->pisMessage);
				}
				break;

			case ISOURCE_END:
				pWeb->_status = STATUS_DONE;
				WEBDEBUG(( "Web.HR: Reached end of response" ));
				if (pInfo->pisMessage) {
					WEBDEBUG(( "Web.HR: Release pisMessage" ));
					ISOURCE_Release(pInfo->pisMessage);
				}
				break;

			default:
				pWeb->_status = STATUS_READING;
				if (nBytes > 0) {
					pWeb->_pRecvBuf->WriteBytes((uint8*)pWeb->_pReadBuf, nBytes);
					pWeb->_bytesRead += nBytes;
					DEBUG(("READ %d", pWeb->_bytesRead));
					pWeb->UpdateProgress();
				}
/*
				#ifdef _DEBUG
					char chars[10], ch;
					char line[128];
					line[0] = '\0';
					for (int idx=0; idx<nBytes; idx++) {
						ch = pWeb->_pBuf[idx];
						SPRINTF(chars, "%02x", ch);
						chars[2] = '\0';//force termination b/c -1 is messing up SPRINTF
						STRCAT(line, chars);
					}
					WEBDEBUG(( "Web.HR: %d bytes [%s] END_BUFFER", nBytes, line ));
				#endif
*/
				WEBDEBUG(( "Web.HR: Read %d bytes", nBytes ));

				ISOURCE_Readable(pInfo->pisMessage, &pWeb->_cbHandleResponse);
				if (pInfo->pisMessage) {
					WEBDEBUG(( "Web.HR: Release pisMessage" ));
					ISOURCE_Release(pInfo->pisMessage);
				}
				return;
		}
	}

	pWeb->DeleteRequest();

	if ((pInfo && WEB_ERROR_MAP(pInfo->nCode) == WEB_ERROR_CONNECT) ||
		(STRCMP(pWeb->_pErrorMsg, "WEB_ERROR_CONNECT") == 0))
	{
		WEBDEBUG(( "Web.HR: WEB_ERROR_CONNECT occurred" ));

		if (pWeb->_pIWeb) {
			WEBDEBUG(( "Web.HR: IWEB_Release" ));
			while (IWEB_Release(pWeb->_pIWeb) >= 1);
			pWeb->_pIWeb = NULL;
		}
	}

	pWeb->Notify();
}
//-----------------------------------------------------------------------------
//	NOTE: static class method
//
//	HandleHeader
//-----------------------------------------------------------------------------
void Web::HandleHeader(Web* pWeb, const char *pcName, GetLine *pglVal)
{
	WEBDEBUG(( "Web.HH: %s (%s)", pcName, pglVal->psz ));

	if (STRICMP(pcName,"Location") == 0) {
		if (pWeb->_redirectRootURL) {
			MEMFREE(pWeb->_redirectRootURL);
			pWeb->_redirectRootURL = NULL;
		}
		pWeb->_redirectRootURL = STRDUP(pglVal->psz);
		WEBDEBUG(( "Web::HH: Redrect URL root: %s", pWeb->_redirectRootURL ));
	}
}
//-----------------------------------------------------------------------------
//	NOTE: static class method
//
//	HandleStatus
//-----------------------------------------------------------------------------
void Web::HandleStatus(Web* pWeb, WebStatus webs, void *pVal)
{
	int nStatus;
	const char *pszStatus = NULL;
	(void)pVal;

	switch (webs)
	{
		case WEBS_GETHOSTBYNAME:
			nStatus = STATUS_FINDHOST;
			pszStatus = "finding host...";
			break;
		case WEBS_CONNECT:
			nStatus = STATUS_CONNECTING;
			pszStatus = "connecting...";
			break;
		case WEBS_SENDREQUEST:
			nStatus = STATUS_SENDING;
			pszStatus = "sending...";
			break;
		case WEBS_READRESPONSE:
			nStatus = STATUS_RECEIVING;
			pszStatus = "receiving...";
			break;
		case WEBS_GOTREDIRECT:
			nStatus = STATUS_REDIRECTING;
			pszStatus = "redirecting...";
			break;
		case WEBS_CACHEHIT:
			nStatus = STATUS_CACHEHIT;
			pszStatus = "retrieving from cache...";
			break;
		case WEBS_STARTING:
			nStatus = STATUS_STARTING;
			pszStatus = "starting...";
			break;
		case WEBS_CANCELLED:
			nStatus = STATUS_CANCELLED;
			pszStatus = "cancelled";
			break;
		default:
			nStatus = STATUS_UNKNOWN;
			pszStatus = "unknown";
			break;
	}

	if (NULL != pszStatus) {
		WEBDEBUG(( "Web.HS: %s", pszStatus ));
	}

	//don't overwrite an error status
	if (pWeb->_status != STATUS_ERROR) {
		pWeb->_status = nStatus;
		if (NULL != pszStatus) {
			STRCPY(pWeb->_pErrorMsg, pszStatus);
		}
	}
}
//-----------------------------------------------------------------------------
void Web::HandleRequestTimeoutTimer(Web* pWeb)
{
	if (pWeb->_responsePending) {

		if (pWeb->_nTimerCountdown > 0) {
			pWeb->_nTimerCountdown--;
		}

		if (pWeb->_nTimerCountdown) {
			pWeb->UpdateProgress();
			ISHELL_SetTimer(GETSHELL(), TIMEOUT_TIMER_INTERVAL, (PFNNOTIFY)Web::HandleRequestTimeoutTimer, pWeb);
		}
		else {
			WEBDEBUG(( "Web.HTT: Request timed out" ));

			if (pWeb->_status != STATUS_ERROR) {
				pWeb->SetTimeoutError();
			}

			pWeb->DeleteRequest();

			// Release IWeb interface to help re-establishment
			if (pWeb->_pIWeb) {
				WEBDEBUG(( "Web.HTT: IWEB_Release" ));
				while (IWEB_Release(pWeb->_pIWeb) >= 1);
				pWeb->_pIWeb = NULL;
			}

			pWeb->Notify();
		}
	}
}

//-----------------------------------------------------------------------------
//	ctor, dtor
//-----------------------------------------------------------------------------
Web::Web(int connectTimeoutMillis, int idleTimeoutSecs, int lingerTimeoutSec)
	: _pIWeb(NULL)
	, _pIWebResp(NULL)
	, _pIPeek(NULL)

	, _isT720(false)

	, _responsePending(false)
	, _readBufLen(0)
	, _pReadBuf(NULL)
	, _pRecvBuf(NULL)
	, _pNotifyFn(NULL)
	, _pNotifyObj(NULL)
	, _redirectRootURL(NULL)
	, _isRunning(false)
	, _nErrorNum(0)
	, _status(0)
	, _nTimerCountdown(0)
	, _bytesRead(0)
{
	WEBDEBUG(( "Web()" ));

	_connectTimeoutMillis	= connectTimeoutMillis;
	_idleTimeoutSec			= idleTimeoutSecs;
	_lingerTimeoutSecs		= lingerTimeoutSec;
}

//-----------------------------------------------------------------------------

Web::~Web()
{
	WEBDEBUG(( "~Web()" ));

	if (_isRunning) {
		Stop();
	}
}

//-----------------------------------------------------------------------------
bool Web::Start(IShell* pShell, IOStreamBuf* pRecvBuf, PFNNOTIFY pNotifyFn, void* pNotifyObj)
{
	WEBDEBUG(( "Web.Start" ));

	if (_isRunning) {
		WEBDEBUG(( "Web.Start: WARNING: Web already started" ));
	}

	// Make the read buffer
	if (!_pReadBuf) {
		_readBufLen = READ_BUFFER_LEN;
		_pReadBuf = (char*)MEMALLOC(_readBufLen);
		if (_pReadBuf) {
			//add null terminator and protect it from being overwritten
			_pReadBuf[_readBufLen-1] = 0;
			_readBufLen--;
		}
		else {
			WEBDEBUG(( "Web.Start: ERROR creating read buffer" ));
			return false;
		}
	}

	_isRunning = true;
	_nErrorNum = 0;
	_status = 0;

	_pRecvBuf = pRecvBuf;
	_pNotifyFn = pNotifyFn;
	_pNotifyObj = pNotifyObj;

	return true;
}
//-----------------------------------------------------------------------------
void Web::Stop()
{
	WEBDEBUG(( "Web.Stop" ));

	if (!_isRunning) {
		WEBDEBUG(( "Web.Stop: WARNING: Web already stopped" ));
		return;
	}

	_isRunning = false;

	DeleteRequest();

	if (_pReadBuf) {
		MEMFREE(_pReadBuf);
		_pReadBuf = NULL;
		_readBufLen = 0;
	}

	if (_redirectRootURL) {
		MEMFREE(_redirectRootURL);
		_redirectRootURL = NULL;
	}

	if (_pIWeb) {
		WEBDEBUG(( "Web.Stop: IWEB_Release" ));
		while (IWEB_Release(_pIWeb) >= 1);
		_pIWeb = NULL;
	}

	_pRecvBuf = NULL;
	_pNotifyFn = NULL;
	_pNotifyObj = NULL;
}
//-----------------------------------------------------------------------------
bool Web::CreateIWeb()
{
	int iRet;

	if (_pIWeb) {
		return true;
	}

	TRACE(( "ISHELL_CreateInstance(AEECLSID_WEB)" ));
	iRet = ISHELL_CreateInstance(GETSHELL(), AEECLSID_WEB, (void**)(&_pIWeb));
	if (iRet != SUCCESS) {
		WEBDEBUG(( "Web.CIW: ERROR creating IWeb instance" ));
		return false;
	}
	ASSERT(_pIWeb);

	WEBDEBUG(( "Web.CIW: Created IWeb" ));

	WebOpt webopt[3] = 	{
							{ WEBOPT_CONNECTTIMEOUT,  (void *)_connectTimeoutMillis },
							{ WEBOPT_IDLECONNTIMEOUT, (void *)_idleTimeoutSec },
							{ WEBOPT_END, (void *)0 }
						};

	IWEB_AddOpt(_pIWeb, webopt);

	return true;
}

//-----------------------------------------------------------------------------
void Web::StartRequestTimeoutTimer(int requestTimeoutMillis)
{
	_requestTimeoutMillis = requestTimeoutMillis;

	CancelRequestTimeoutTimer();

	_nTimerCountdown = _requestTimeoutMillis / TIMEOUT_TIMER_INTERVAL;

	ISHELL_SetTimer(GETSHELL(), TIMEOUT_TIMER_INTERVAL, (PFNNOTIFY)Web::HandleRequestTimeoutTimer, this);
}
//-----------------------------------------------------------------------------
void Web::CancelRequestTimeoutTimer()
{
	_nTimerCountdown = 0;
	ISHELL_CancelTimer(GETSHELL(), (PFNNOTIFY)HandleRequestTimeoutTimer, (void *)this);
}

//-----------------------------------------------------------------------------
bool Web::SendRequest(const char* pcURL, const uint8* pcData, int32 dataLen, uint32 requestTimeoutMillis)
{
	DEBUG(("%s", pcURL));
	WEBDEBUG(( "Web.SendRequest" ));
	DBGSTK_CHECK_LIGHT;

	ASSERT(pcURL);
	ASSERT(pcData);
	ASSERT(_isRunning);
	if (!pcURL) return false;
	if (!pcData) return false;
	if (!_isRunning) return false;

	WEBDEBUG(( "Web.SR('%s', %d)", pcURL, dataLen ));

	if (_responsePending) {
		WEBDEBUG(( "Web.SR: REJECTED: Response is pending" ));
//		STRCPY(_pErrorMsg, "Response already pending!");
		return false;
	}

	if (!CreateIWeb()) {
//		STRCPY(_pErrorMsg, "Unable to create IWeb instance!);
		return false;
	}

	ISourceUtil* pISrcUtil;
	if (ISHELL_CreateInstance(GETSHELL(), AEECLSID_SOURCEUTIL, (void**)&pISrcUtil) == SUCCESS) {
		int iRet = ISOURCEUTIL_PeekFromMemory(pISrcUtil, pcData, dataLen, 0, 0, &_pIPeek);
		ISOURCEUTIL_Release(pISrcUtil);
		if (iRet != SUCCESS) {
			WEBDEBUG(( "Web.SR: ERROR ISOURCEUTIL_PeekFromMemory failure" ));
//			STRCPY(_pErrorMsg, "Unable to peek in ISource");
			return false;
		}
	}
	else {
//		STRCPY(_pErrorMsg, "Unable to create ISource instance!);
		return false;
	}

	_pErrorMsg[0] = '\0';
	_nErrorNum = 0;
	_status = 0;
	_bytesRead = 0;

	// Setup receive buffer
	_pRecvBuf->Reset();

	// Launch the request timeout timer before issuing the request
	StartRequestTimeoutTimer(requestTimeoutMillis);

	// Setup the response callback
	MEMSET(&_cbHandleResponse, 0, sizeof(AEECallback));
	CALLBACK_Init(&_cbHandleResponse, (PFNNOTIFY)Web::HandleResponse, this);

	_responsePending = true;

    // Issue the request
	IWEB_GetResponse(_pIWeb, (_pIWeb, &_pIWebResp, &_cbHandleResponse, pcURL,
						WEBOPT_HANDLERDATA, this,
						WEBOPT_HEADERHANDLER, (PFNNOTIFY)Web::HandleHeader,
						WEBOPT_STATUSHANDLER, (PFNNOTIFY)Web::HandleStatus,
						WEBOPT_HEADER, "X-Method: POST\r\n",
						WEBOPT_METHOD, "POST",
						WEBOPT_BODY, _pIPeek,
						WEBOPT_CONTENTLENGTH, dataLen,
						WEBOPT_END)
					);

	return true;
}

//-----------------------------------------------------------------------------
void Web::DeleteRequest()
{
	WEBDEBUG(( "Web.DeleteRequest()" ));

	_responsePending = false;

	CancelRequestTimeoutTimer();

	WEBDEBUG(( "Web.DR: Cancel response callback" ));
	CALLBACK_Cancel(&_cbHandleResponse);

	if (_pIWebResp) {
		WEBDEBUG(( "Web.DR: IWEBRESP_Release()" ));
		IWEBRESP_Release(_pIWebResp);
		_pIWebResp = NULL;
	}

	if (_pIPeek) {
		WEBDEBUG(( "Web.DR: IPEEK_Release()" ));
		IPEEK_Release(_pIPeek);
		_pIPeek = NULL;
	}

	SetConnectionLingerTimeout(_lingerTimeoutSecs);
}

//-----------------------------------------------------------------------------
void Web::SetConnectionLingerTimeout(int lingerTimeoutSecs)
{
	// Create an INETMGR interface to set PPP linger time.
	//.....................................................
	INetMgr *pINetMgr;
	int iRet = ISHELL_CreateInstance(GETSHELL(), AEECLSID_NET, (void**)(&pINetMgr));
	if (iRet == SUCCESS) {
		INETMGR_SetLinger(pINetMgr, lingerTimeoutSecs);
		INETMGR_Release(pINetMgr);
	}
}

//-----------------------------------------------------------------------------
//	Notify the client that the response has arrived or error has occurred
//-----------------------------------------------------------------------------
void Web::Notify()
{
	if (_pNotifyFn) {
		_pRecvBuf->Seek(0);
		_pNotifyFn(_pNotifyObj);
	}
}

//-----------------------------------------------------------------------------
//	Notify the client of progress
//-----------------------------------------------------------------------------
void Web::UpdateProgress()
{
	if (_pProgressFn) {
		int fullcount = _requestTimeoutMillis / TIMEOUT_TIMER_INTERVAL;
		int percentTime = 100 * (fullcount - _nTimerCountdown) / fullcount;
		_pProgressFn(_pProgressObj, percentTime, _bytesRead);
	}
}
//-----------------------------------------------------------------------------
void Web::SetProgressHandler(ProgressFnPtr pProgressFn, void* pProgressObj)
{
	_pProgressFn = pProgressFn;
	_pProgressObj = pProgressObj;
}
//-----------------------------------------------------------------------------
void Web::SetTimeoutError()
{
	_status = STATUS_ERROR;
	_nErrorNum = ERR_TIMEOUT;
	STRCPY(_pErrorMsg, "Connection Timeout");
}
//-----------------------------------------------------------------------------
void Web::SetError(WebRespInfo* pInfo)
{
	//this is only called if there's an error
	_status = STATUS_ERROR;

	if (pInfo->nCode > 0) {
		WEBDEBUG(( "Web.SE: HTTP ERROR = %d", pInfo->nCode ));

		_nErrorNum = pInfo->nCode;
		if (pInfo->lContentLength && pInfo->pisMessage) {
			if (ISOURCE_Read(pInfo->pisMessage, _pErrorMsg, ERROR_MSG_LEN) > 0) {
				WEBDEBUG(( "Web.SE: msg = %s", _pErrorMsg ));
			}
		}
		return;
	}
	else {
		_nErrorNum = WEB_ERROR_MAP(pInfo->nCode);
	}

	char *pErr = (char *)"AEE ERROR";
	switch (_nErrorNum)
	{
		case WEB_ERROR_PROTOCOL:	pErr = (char *)"WEB_ERROR_PROTOCOL"; break;
		case WEB_ERROR_BADURL:		pErr = (char *)"WEB_ERROR_BADURL"; break;
		case WEB_ERROR_BADHOSTNAME:	pErr = (char *)"WEB_ERROR_BADHOSTNAME"; break;
		case WEB_ERROR_BADPORT:		pErr = (char *)"WEB_ERROR_BADPORT"; break;
		case WEB_ERROR_UNSUPSCHEME:	pErr = (char *)"WEB_ERROR_UNSUPSCHEME"; break;
		case WEB_ERROR_DNSCONFIG:	pErr = (char *)"WEB_ERROR_DNSCONFIG"; break;
		case WEB_ERROR_DNSTIMEOUT:	pErr = (char *)"WEB_ERROR_DNSTIMEOUT"; break;
		case WEB_ERROR_ADDRUNKNOWN:	pErr = (char *)"WEB_ERROR_ADDRUNKNOWN"; break;
		case WEB_ERROR_CONNECT:		pErr = (char *)"WEB_ERROR_CONNECT"; break;
		case WEB_ERROR_SEND:		pErr = (char *)"WEB_ERROR_SEND"; break;
		case WEB_ERROR_RECV:		pErr = (char *)"WEB_ERROR_RECV"; break;
		case WEB_ERROR_BADRESPONSE:	pErr = (char *)"WEB_ERROR_BADRESPONSE"; break;
		case WEB_ERROR_BODYLENGTH:	pErr = (char *)"WEB_ERROR_BODYLENGTH"; break;
		case WEB_ERROR_PROXYSPEC:	pErr = (char *)"WEB_ERROR_PROXYSPEC"; break;
		default:
			SetTimeoutError();
			return;
	}

	WEBDEBUG(( "Web.SE: %s = 0x%x = 0d%d", pErr, _nErrorNum, _nErrorNum ));
	STRCPY(_pErrorMsg, pErr);
}
//-----------------------------------------------------------------------------
//	Caller must provide a pre-allocated message buffer.
//	Returns true if there's an error.
//	Fills the message buffer with status regardless of an error or not.
//	Calling this routine will clear the error status.
//-----------------------------------------------------------------------------
bool Web::GetErrorMessage(uint32 &nError, char* pErrorMessage, int bufLen)
{
	nError = _nErrorNum;

	if (pErrorMessage && bufLen > 0) {
		int len = (bufLen < ERROR_MSG_LEN) ? bufLen : ERROR_MSG_LEN;

		STRNCPY(pErrorMessage, _pErrorMsg, len);
		pErrorMessage[len-1] = '\0';

		if (_status == STATUS_ERROR) {
			_status = STATUS_UNKNOWN;
			return true;
		}
	}
	return false;
}
