//=============================================================================
//	PlatformBREW
//
//	Abstraction Layer for BREW API
//=============================================================================

#include "PlatformBREW.h"
#include "AEEAppGen.h"
#include "AEETAPI.h"
#include "AEEStdLib.h"
#include "Debug.h"

#ifdef __cplusplus
extern "C"
{
#endif


//-----------------------------------------------------------------------------
//	Platform_GetMIN()
//-----------------------------------------------------------------------------

char * Platform_GetMIN()
{
	char* pMIN = NULL;
	ITAPI* pITAPI;
	TAPIStatus tapiStatus;

	//This requires TAPI privilege in MIF
	if (ISHELL_CreateInstance(GETSHELL(), AEECLSID_TAPI, (void**)&pITAPI) == SUCCESS)
	{
		if (ITAPI_GetStatus(pITAPI, &tapiStatus) == SUCCESS)
		{
			TRACE(( "Platform_GetMIN(): MIN = '%s'", (tapiStatus.szMobileID == NULL ? "<UNKNOWN>" : tapiStatus.szMobileID) ));
			if (tapiStatus.szMobileID[0])
			{
				pMIN = (char*)MEMALLOC(STRLEN(tapiStatus.szMobileID) + 1);
				if (pMIN)
				{
					STRCPY(pMIN, tapiStatus.szMobileID);
				}
			}
		}
		ITAPI_Release(pITAPI);
	}
	return pMIN;
}

//-----------------------------------------------------------------------------
//	Platform_GetScreenSize()
//-----------------------------------------------------------------------------

boolean Platform_GetScreenSize(int16* piWidth, int16* piHeight)
{
	AEEDeviceInfo deviceInfo;
	uint32 vBREW = GETAEEVERSION(NULL, 0, 0) >> 24;

	AEEApplet* pApplet = (AEEApplet*)GETAPPINSTANCE();
	ASSERT(pApplet);

	IShell* pShell = (pApplet ? pApplet->m_pIShell : NULL);
	if (!pShell)
	{
		*piWidth = *piHeight = 0;
		return FALSE;
	}

	deviceInfo.wStructSize = sizeof(AEEDeviceInfo);
	ISHELL_GetDeviceInfo(pShell, &deviceInfo);

	*piWidth = deviceInfo.cxScreen;
	*piHeight = deviceInfo.cyScreen;

//	DEBUG(("Platform_GetScreenSize() original: width=%d, height=%d",*piWidth, *piHeight ));

	if (vBREW >= 2)
	{
		switch (deviceInfo.dwPlatformID)
		{
			case PLATFORM_ID_KX414:
				*piHeight -= 12;
				break;
			default:
				break;
		}
	}

//	TRACE(( "Platform_GetScreenSize(): width=%d, height=%d", *piWidth, *piHeight ));
//	DEBUG(("Platform_GetScreenSize(): width=%d, height=%d", *piWidth, *piHeight ));
	return TRUE;
}

//-----------------------------------------------------------------------------
//	Platform_GetPhoneID()
//-----------------------------------------------------------------------------

uint32 Platform_GetPhoneID()
{
	AEEDeviceInfo deviceInfo;
	uint32 vBREW = GETAEEVERSION(NULL, 0, 0);
	uint32 vBREWMajor = vBREW >> 24;
	uint32 vBREWMinor = (vBREW & 0x00ff0000) >> 16;
	uint32 vBREWSub   = (vBREW & 0x0000ff00) >> 8;
	int idx;

	#ifdef AEE_SIMULATOR
		vBREWMajor = 1;
		vBREWMinor = 1;
	#endif

	ARMDEBUG(( "BREW=%ld.%ld.%ld",vBREWMajor,vBREWMinor,vBREWSub ));

	AEEApplet* pApplet = (AEEApplet*)GETAPPINSTANCE();
	IShell* pShell = (pApplet ? pApplet->m_pIShell : NULL);

	if (!pShell){
		//DEBUG(( "DEVICE=?" ));
		return PLATFORM_ID_NULL;
	}

	deviceInfo.wStructSize = sizeof(AEEDeviceInfo);
	ISHELL_GetDeviceInfo(pShell, &deviceInfo);

	// ID is easy on Brew 2.x and above.
	//..................................
	if ( vBREWMajor >= 2) {
		//DEBUG(( "DEVICE=%d", deviceInfo.dwPlatformID ));
		return deviceInfo.dwPlatformID;
	}

	// Must use fallback method for devices prior to Brew 2.x.
	//........................................................
//	DEBUG(( "SCREEN=%dx%dx%d", deviceInfo.cxScreen, deviceInfo.cyScreen, deviceInfo.nColorDepth ));

	// Table of known platform ID, screen sizes, and other
	// identifying attributes
	//....................................................

	BREW11Device PlatformHW[] =
	{
		// Brew 1.1
		{ 120, 115, 16, 0, 1, 1, 4, PLATFORM_ID_VX4400   },		// Confirmed.
		{ 128, 146,  8, 0, 1, 1, 4, PLATFORM_ID_A530     },		// Confirmed.
		{ 128, 146, 16, 0, 1, 1, 4, PLATFORM_ID_A610     },		// Confirmed.
		{ 128, 146, 16, 0, 1, 1, 5, PLATFORM_ID_A690     },		// Confirmed.
		{ 120, 130, 12, 0, 1, 1, 0, PLATFORM_ID_T720     },		// Confirmed.
		{  96,  54, 12, 0, 1, 1, 5, PLATFORM_ID_C343     },		// Unconfirmed.
		{ 144, 158, 16, 0, 1, 1, 4, PLATFORM_ID_CDM9500  },		// Confirmed.
		{ 144, 158, 16, 0, 1, 1, 4, PLATFORM_ID_CDM9500R },		// Confirmed.
		{  96,  65, 12, 0, 1, 2, 4, PLATFORM_ID_3589I    },     // Unconfirmed.
		{ 128, 128, 12, 0, 1, 2, 0, PLATFORM_ID_6585     },     // Unconfirmed.

		// The emulator skins give the wrong info about some hardware.
		{  96,  65,  8, 0, 1, 1, 0, PLATFORM_ID_3589I_EM },
		{  96,  54,  8, 0, 1, 1, 0, PLATFORM_ID_C343     },
		{ 104,  68,  8, 0, 1, 1, 0, PLATFORM_ID_KX414    },
		{ 121, 131,  8, 0, 1, 1, 0, PLATFORM_ID_T720     },
		{ 121, 116,  8, 0, 1, 1, 0, PLATFORM_ID_VX4400   },
		{ 144, 158,  8, 0, 1, 1, 0, PLATFORM_ID_CDM9500  },

		// NULL platform id used for unknown or unrecognized device:
		{   0,   0,  0, 0, 0, 0, 0, PLATFORM_ID_NULL     },
	};

	idx = 0;
	while ( PlatformHW[idx].ePlatformId != PLATFORM_ID_NULL ) {
		if ((deviceInfo.cxScreen == PlatformHW[idx].x) &&
			(deviceInfo.cyScreen == PlatformHW[idx].y) &&
			(deviceInfo.nColorDepth == PlatformHW[idx].bits))
		{
			if (!PlatformHW[idx].vSub || (PlatformHW[idx].vSub == vBREWSub)) {
				//DEBUG(( "DEVICE=%d", PlatformHW[idx].ePlatformId ));
				return PlatformHW[idx].ePlatformId;
			}
		}
		idx++;
	}

	ARMDEBUG(( "DEVICE=UNKNOWN" ));
	return PLATFORM_ID_NULL;
}

//-----------------------------------------------------------------------------
//	Platform_GetPhoneName()
//-----------------------------------------------------------------------------

char * Platform_GetPhoneName(char *nameBuf)
{
	SPRINTF(nameBuf, "%d", Platform_GetPhoneID());
	return nameBuf;
}

/*
char* Platform_GetPhoneName()
{
	PLATFORM_ID	ePlatformID = Platform_GetPhoneID();

	char*			pPhoneName = "Unknown";

	// Map platformID to a name.
	//...................................

	switch ( ePlatformID ){
		// Brew 1.0
		case PLATFORM_ID_Z800:		pPhoneName = "Z800";			break;
		case PLATFORM_ID_CDM7900:	pPhoneName = "CDM7900";			break;
		case PLATFORM_ID_QCP3035:	pPhoneName = "QCP3035";			break;
		// Brew 1.1
		case PLATFORM_ID_VX10:		pPhoneName = "VX10";			break;
		case PLATFORM_ID_VX4400:	pPhoneName = "VX4400";			break;
		case PLATFORM_ID_A530:		pPhoneName = "A530";			break;
		case PLATFORM_ID_A605:		pPhoneName = "A605";			break;
		case PLATFORM_ID_A610:		pPhoneName = "A610";			break;
		case PLATFORM_ID_A690:		pPhoneName = "A690";			break;
		case PLATFORM_ID_QCP3225:	pPhoneName = "QCP3225";			break;
		case PLATFORM_ID_T720:		pPhoneName = "T720";			break;
		case PLATFORM_ID_C343:		pPhoneName = "C343";			break;
		case PLATFORM_ID_T606:		pPhoneName = "T606";			break;
		case PLATFORM_ID_CDM9500:	pPhoneName = "CDM-9500";		break;
		case PLATFORM_ID_CDM9500R:	pPhoneName = "CDM-9500R";		break;
		case PLATFORM_ID_3589I:		pPhoneName = "3589I";			break;
		case PLATFORM_ID_3589I_EM:	pPhoneName = "3589I_EM";		break;
		// Brew 2.0
		case PLATFORM_ID_VX6000:	pPhoneName = "VX6000";			break;
		case PLATFORM_ID_VX4600:	pPhoneName = "VX4600";			break;
		case PLATFORM_ID_VX4500:	pPhoneName = "VX4500";			break;
		case PLATFORM_ID_KX414:		pPhoneName = "KX414";			break;
		case PLATFORM_ID_SE47:		pPhoneName = "SE47";			break;
		case PLATFORM_ID_V810:		pPhoneName = "V810";			break;
		case PLATFORM_ID_CDM9900:	pPhoneName = "CDM-9900";		break;
		case PLATFORM_ID_CDM8600:	pPhoneName = "CDM-8600";		break;
		case PLATFORM_ID_CDM8400:	pPhoneName = "CDM-8400";		break;
		// Default
		default:					ARMDEBUG(( "DEVICE=UNKNOWN" ));	break;
	}

	return pPhoneName;
}
*/


#ifdef __cplusplus
}
#endif
