

/*
 * Copyright (C) 2003 Radu Braniste (rbraniste@epicad.com)
 * All Rights Reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Author nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef __BrewString
#define __BrewString
#include "platformSpecific.h"
//extern void* mem_changeStr(void*  ptr, const char* file);
//extern void* mem_changeStr(void*  ptr, const AECHAR* file);






template <typename T>
bool isspace(T c){ return (c == ' '); }

 template <typename T  >
struct BrewStringPolicy 
 {
	static T* createBuffer(const AECHAR* c)
	{
		if (!c)
			return 0;
		int len = WSTRLEN(c)+1;
		T* a = new T[len];
		if (a)
			WSTRTOSTR(c, a, len);
		return a;
	}

	static int T_STRLEN(const T* s)
	{
		return STRLEN(s);
	}

	static void T_MEMCPY(T* dest, const T* src, size_t sz)
	{
		MEMCPY(dest, src, sz);
	}	

	static int T_STRCMP(const T* s, const T* s1)
	{
		return STRCMP(s, s1);
	}

	static int T_STRNCMP(const T* s, const T* s1, size_t sz)
	{
		return STRNCMP(s, s1, sz);
	}

	static T* T_STRSTR(const T* hayStack, const T* needle)
	{
		return STRSTR(hayStack, needle);
	}

	static T* T_STRCHR(const T* s, T c)
	{
		return STRCHR(s, c);
	}

	static T* T_STRLOWER( T* s)
	{
		return STRLOWER(s);
	}

	static T* T_STRUPPER( T* s)
	{
		return STRUPPER(s);
	}

 };

 template <>
struct BrewStringPolicy <AECHAR>
 {
	static AECHAR* createBuffer(const char* c)
	{
		if (!c)
			return 0;
		int len = STRLEN(c)+1;
		AECHAR* a = new AECHAR[len];
		if (a)
			STRTOWSTR(c, a, (len)*sizeof(AECHAR));
		return a;
	}

	static int T_STRLEN(const AECHAR* s)
	{
		return WSTRLEN(s);
	}

	static void T_MEMCPY(AECHAR* dest, const AECHAR* src, size_t sz)
	{
		MEMCPY(dest, src, sizeof(AECHAR) * sz);
	}

	static int T_STRCMP(const AECHAR* s, const AECHAR* s1)
	{
		return WSTRCMP(s, s1);
	}

	static int T_STRNCMP(const AECHAR* s, const AECHAR* s1, size_t sz)
	{
		return WSTRNCMP(s, s1, sz);
	}

	static AECHAR* T_STRSTR(const AECHAR* hayStack, const AECHAR* needle)
	{
		if ( ( WSTRLEN(hayStack) < WSTRLEN(needle) ) || (WSTRLEN(needle)==0) )
			return false;
		AECHAR* a = const_cast<AECHAR*>(hayStack);
		while ( (a=WSTRCHR(a, needle[0])) != 0)
		{
			if (WSTRLEN(a) < WSTRLEN(needle))
				return 0;
			if (MEMCMP(a, needle, WSTRLEN(needle)*sizeof(AECHAR))==0)
				return a;
			else
				a++;

		}
		return 0;
	}

	static AECHAR* T_STRCHR(const AECHAR* s, AECHAR c)
	{
		return WSTRCHR(s, c);
	}

	static AECHAR* T_STRLOWER( AECHAR* s)
	{
		WSTRLOWER(s);
		return s;
	}

	static AECHAR* T_STRUPPER( AECHAR* s)
	{
		WSTRUPPER(s);
		return s;
	}
 };


template <typename T ,  class P  >
class BrewString  
{
public:
	template <class U>	
	BrewString( const U& rhs , UINT sz = 0): pData_(0)
	{
		Wrapper p(rhs);
		CopyCtorImpl(p(), sz);
	}

	BrewString( const BrewString<T,P> &rhs ): pData_(0)
	{
		CopyCtorImpl(rhs, 0);
	}

	template < class U, class V>
	BrewString (const U& lhs, UINT lnl, const V& rhs, UINT lnr): pData_(0)
	{
		Wrapper r(rhs);
		Wrapper l(lhs);
		if (!ensureCapacity(lnr + lnl ))
			return;
		P::T_MEMCPY( pData_->buffer, getBuf(l()) , lnl );
		P::T_MEMCPY( pData_->buffer+lnl, getBuf(r()) , lnr );
		pData_->buffer[pData_->length=pData_->capacity] = 0;
	}

	BrewString( ) : pData_(0)
	{
	}

   ~BrewString()  
	{ 
		_FREE(pData_);
	}

   void destruct()
   {
		_FREE(pData_);
   }

   void renew(const BrewString &rhs)
   {
		_FREE(pData_);
		CopyCtorImpl(rhs, 0);
   }

	const BrewString<T,P> &
	operator=( const BrewString &rhs )
	{
		if ( this == &rhs )
			return *this;
		return opEqImpl(rhs);
	}

	boolean	operator==( const BrewString &rhs )
	{
		return this->equals(rhs);
	}

	template <typename U>
	const BrewString<T,P> &
	operator=( const U* rhs )
	{
		Wrapper p(rhs);
		return opEqImpl(p());
	}

	template <class U>	
	const BrewString<T,P> &
	operator+=( const U& rhs )
	{
		Wrapper p(rhs);
		UINT ln = getLen(p());
		UINT newLen = increaseBuffer(ln);
		if (!newLen)
			return *this;
		cat(getBuf(p()), ln);
		pData_->length = newLen;
		return *this;
	}



	template <class U>
	bool equals( const U& rhs ) const
	{
		Wrapper p(rhs);
		return ( bufferImpl() && lengthImpl() == getLen(p()) && P::T_STRCMP( pData_->buffer, getBuf(p()) ) == 0 );
	}


	template <class U>
	bool startsWith( const U &rhs, UINT offset = 0 ) const
	{
		Wrapper p(rhs);
		UINT ln = getLen(p());
		if ( lengthImpl()< ln + offset  || ln == 0)
			return false;
		return P::T_STRNCMP( &pData_->buffer[offset], getBuf(p()), ln ) == 0;
	}

	template <class U>
	bool endsWith( const U& rhs ) const
	{
		Wrapper p(rhs);
		UINT ln = getLen(p());
		if ( lengthImpl() < ln || ln == 0)
			return false;
		return P::T_STRCMP( &pData_->buffer[ pData_->length - ln], getBuf(p()) ) == 0;
	}

	template <class U>
	int indexOf( const U &rhs, UINT fromIndex = 0 ) const
	{
		Wrapper p(rhs);
		if (lengthImpl()< getLen(p()) + fromIndex)
			return -1;
		const T *temp = P::T_STRSTR( &pData_->buffer[ fromIndex ], getBuf(p()) );
		return ( temp ) ? temp - pData_->buffer : -1;
	}


	template <class U>	
	int	lastIndexOf( const U &rhs ) const
	{
		Wrapper p(rhs);
		return lastIndexOf( rhs, lengthImpl() - getLen(p()) );
	}

	template <class U>
	int lastIndexOf( const U &rhs, UINT fromIndex ) const
	{
		Wrapper p(rhs);
		UINT ln = getLen(p());
		if ( ln == 0 || fromIndex + ln>= lengthImpl() )
			return -1;
		T temp = getBuf(p())[ 0 ];
		for ( int i = fromIndex; i >= 0; i-- )
		{
			if ( pData_->buffer[ i ] == temp && (!P::T_STRNCMP(getBuf(p()), &pData_->buffer[i], ln)) )
				return i;
		}
		return -1;
	}

	template <class U, class V>
	const BrewString<T,P>&
	replace( const U& match, const V& replacement )
	{
		Wrapper m(match);
		Wrapper r(replacement);
		UINT ln = getLen(m());
		if (ln == 0 || ln > lengthImpl())
			return *this;
		BrewString newString;
		int loc , oldLoc = 0;
		bool isReplaceable = false;
		while ( (loc = indexOf( m(), oldLoc )) != -1 )
		{
			if (!isReplaceable ) 
			{	
				isReplaceable = true;
				if (!newString.ensureCapacity(pData_->length))
					break;
			}
			newString += substring( oldLoc, loc  );
			newString += r();
			oldLoc = loc + ln;
		} 			   
		if (isReplaceable) 
		{
			newString += substring( oldLoc );
			return opEqImpl(newString);
		}
		return *this;
	}

	const BrewString<T,P>&
	replace( T findChar, T replaceChar )
	{
		if (pData_ && pData_->buffer)
		{
			T* temp = pData_->buffer;
			while( temp  = P::T_STRCHR( temp, findChar ) )
				*temp = replaceChar;
		}
		return *this;
	}
   
	const BrewString<T,P>
	substring( UINT left ) const
	{
		return substring( left, lengthImpl() );
	}

	const BrewString<T,P>
	substring( UINT left, UINT right ) const
	{
		if ( ( left >= right ) || ( right > lengthImpl()) )
		{
			return BrewString();	
		}  
		return BrewString ( pData_->buffer + left, right - left );
	}

	const BrewString<T,P>& partialString( UINT right ) 
	{
		if (  right >= lengthImpl() )
		{
			return *this;	
		}  
		pData_->length = right;
		pData_->buffer[pData_->length] = 0;
		return *this;
	}

	const BrewString<T,P>& partialString( UINT left, UINT right ) 
	{
		if ( ( left > right ) || ( right > lengthImpl() ) )
		{
			return *this;	
		}  
		pData_->length = right - left;
		T* strt = pData_->buffer;
		T* sstr = pData_->buffer + left;
		while( (UINT)(strt - pData_->buffer) < pData_->length )
		{
			*(strt++) = *(sstr++);
		}
		pData_->buffer[pData_->length] = 0;
		return *this;
	}

	const BrewString<T,P>& trim() 
	{
		if (lengthImpl() <1 )
			return *this;
		UINT i = 0;
		for ( UINT ln = pData_->length; i < ln; i++ )
		{
			if ( !isspace(pData_->buffer[i]) )
				break;
		}
		UINT j = pData_->length - 1;
		for ( ; j > i; j-- )
		{
			if ( !isspace(pData_->buffer[j]) )
				break;
		}
		return partialString( i, j + 1);
	}

	const BrewString<T,P>& toLowerCase( ) const	
	{
		if (pData_ && pData_->buffer)
			P::T_STRLOWER(pData_->buffer);
		return *this;
	}

	const BrewString<T,P>& toUpperCase( ) const
	{
		if (pData_ && pData_->buffer)
			P::T_STRUPPER(pData_->buffer);
		return *this;
	}

	const T*		toCharArray() const { return bufferImpl();}

	const UINT		length( ) const { return lengthImpl(); }

	const UINT		capacity( ) const { return capacityImpl(); }

	const T	charAt( int index ) const
	{
		return (!isIndexOutOfBounds( index )) ? pData_->buffer[ index ] : 0;
	}

	const UINT	setCharAt(const T p_Char, int index ) const
	{
		if(!isIndexOutOfBounds( index ))
		{
			pData_->buffer[ index ] = p_Char;
			return index;
		}
		return 0;
	}

	bool ensureCapacity(UINT sz)
	{
		if (!pData_)
		{
			if (!increaseBuffer(sz))
				return false;
			pData_->buffer[pData_->length] = 0;
			return true;
		}
		int l = sz - pData_->length;
		UINT newLen = l>0 ? l : pData_->length;
		if (!increaseBuffer(newLen))
			return false;
		return true;
	}

	void setLength(UINT sz)
	{
		if (sz < lengthImpl()) 
		{
			pData_->length = sz;
			//pData_->capacity = sz;
			pData_->buffer[sz] = 0;
			return;
		}
		if (sz > lengthImpl()) 
		{
			int l = increaseBuffer(sz - lengthImpl());
			if (l)
			{
				pData_->length = l;
				pData_->buffer[pData_->length] = 0;
			}
		}
	}

private: 
	bool setBuffer(UINT sz)
	{
		pData_ = (DATA*)_MALLOC(sizeof(DATA) + sz*sizeof(T));
		
	 	if (pData_)
		{
			pData_->capacity = sz;
			return true;
		}
		return false;
	}

	UINT increaseBuffer(UINT sz)
	{
		if (!pData_)
		{
			if (!setBuffer( sz ))
				return 0;
			pData_->length = 0;
			return sz;
		}
		UINT newLen = pData_->length + sz;
		if (newLen > pData_->capacity)
		{
			DATA *temp = pData_;
			if (!setBuffer( newLen ))
			{
				_FREE (temp);
				return 0;
			}
			pData_->length = newLen - sz;
			cpy( temp->buffer);
			//mem_changeStr(pData_, temp->buffer);
			_FREE (temp);
			if (!pData_->buffer) return 0;
		}
		return newLen;
	}
	bool isIndexOutOfBounds( UINT index ) const 
	{
		return (( index >= lengthImpl() ) || ( index <0 ) );
	}

	UINT getLen(const BrewString& s) const
	{ return s.pData_?s.pData_->length:0;}

	UINT getLen(const T* ch) const
	{ return ch?P::T_STRLEN(ch):0;}
		
	const T* getBuf(const BrewString& s) const
	{ return s.pData_->buffer;}

	const T* getBuf(const T* ch) const
	{ return ch;}

	template <class U>
	const BrewString<T,P> &
	opEqImpl( const U &rhs )
	{
		UINT ln = getLen(rhs);
		if (!pData_ || ln > capacityImpl() )
		{
			_FREE( pData_);
			if (!setBuffer( ln ))
				return *this;
		}
		pData_->length = ln;
		cpy(getBuf(rhs));
		//mem_changeStr(pData_, getBuf(rhs));
		return *this;
	}

	template <class U>
	void CopyCtorImpl( const U &rhs, UINT sz)
	{
		UINT ln = getLen(rhs);
		sz = (sz == 0) || (sz > ln)? ln : sz;
		if (!setBuffer( sz ))
			return;
		pData_->length = sz;
		cpy( getBuf(rhs) );
		//mem_changeStr(pData_, getBuf(rhs));
	}

	void cpy(const T* rhs)
	{
		if (pData_ && pData_->buffer && rhs)
		{
			P::T_MEMCPY( pData_->buffer, rhs , pData_->length );
			pData_->buffer[pData_->length] = 0;
		}
	}

	void cat(const T* rhs,  UINT sz )
	{
		if (pData_ && pData_->buffer && rhs)
		{
			P::T_MEMCPY( pData_->buffer+ pData_->length, rhs , sz );
			pData_->buffer[sz + pData_->length] = 0;
		}
	}

	T* bufferImpl() const
	{
		return pData_?pData_->buffer:0;
	}

	UINT lengthImpl() const 
	{
		return pData_?pData_->length:0;
	}

	UINT capacityImpl() const
	{
		return pData_?pData_->capacity:0;
	}

	friend BrewString<T,P> operator+ ( const BrewString<T,P>& lhs,  const T* rhs )
	{
		return BrewString<T,P>::opPlusImpl(lhs, rhs);
	}


	friend BrewString<T,P> operator+ ( const BrewString<T,P>& lhs,  const BrewString<T,P>& rhs )
	{
		return BrewString<T,P>::opPlusImpl(lhs, rhs);
	}

	friend BrewString<T,P> operator+ ( const T* lhs,  const  BrewString<T,P>& rhs )
	{
		return BrewString<T,P>::opPlusImpl(lhs, rhs);
	}

	template < class U, class V>
	static BrewString<T,P> opPlusImpl(const U& lhs, const V& rhs)
	{
		BrewString<T,P> t;
		UINT lnl = t.getLen(lhs);
		UINT lnr = t.getLen(rhs);
		return BrewString<T,P>(lhs, lnl,  rhs, lnr);
	}
	
	int getSize()
	{
		return pData_->length*sizeof(T);
	}


private:
	struct DATA
	{
		UINT length;
		UINT capacity;
		T buffer[1];
	};
	DATA* pData_;



	class Wrapper 
	{
	public:
		template <typename U>
		Wrapper(U* t) : t_(getArray(t))
		{}

		template <typename U, typename R>
		Wrapper(const BrewString<U,R>& t) : t_(getArray(t))
		{}

		~Wrapper()
		{
			if (owns_)
			{
				delete[] t_;
			}
		}

		const T* operator()() const 
		{
			return t_;
		}

	private:
		template <typename U>
		T* getArray(const U* c)
		{
			owns_ = true;
			return P::createBuffer(c);
		}

	
		T* getArray(const T* u)
		{
			owns_ = false;
			return const_cast<T*>(u);
		}

		template <typename U, typename R>
		T* getArray(const BrewString<U,R>& u)
		{
			owns_ = false;
			return getArray(u.toCharArray());
		}

		T* getArray(const BrewString<T,P>& u)
		{
			owns_ = false;
			return const_cast<T*>(u.toCharArray());
		}
	private:
		T* t_;
		bool owns_;
	};

};





typedef BrewString<char, BrewStringPolicy<char> > String;
typedef BrewString<AECHAR, BrewStringPolicy<AECHAR> > WString;

#endif
