#ifndef _FP_SEED_GENERATOR_H_
#define _FP_SEED_GENERATOR_H_

#include "AEEAppGen.h"
#include "BrewVector.h"
#include "..\Module\Core\ITBSeedGenerator.h"

/**
 * The Seed Generator is a special class within your application to use to test the
 * generation of seeds for FP Tournament games.  Obviously the simplest solution would
 * be to engineer your game so it can take a simple random value as a seed.
 * 
 * If this is not possible, however (say, if the game designer would like special tournament
 * levels or other specialized binary data), The code used to build the seed should be developed
 * in here.
 * 
 * Thirdparty clients are responsible for providing ALL seeds for ALL tournaments played.  
 * For Fairness, tournament games (or levels) should never also be available in single player
 * mode, and each tournament will need to have a new seed.
 * 
 * If your game's seed mechanism simply relies on random data, Then you can simply put a call
 * to Random() in getSeed() and not worry about it (although you should make sure that the same
 * seed values always produce the same games.
 * 
 * If your game depends on custom seeds, then it is mandatory that the 'out of seed' condition 
 * be tested.  To test 'out of seed', simply have getSeed() return null;
 * 
 * @author E Scott Cragg. Twistbox Client Core Engineering
 *
 * @author Tiago Teixeira. Twistbox Client Core Engineering
 */

class FPSeedGenerator: public ITBSeedGenerator
{
	BbyteVector seeds;
	int seedIndex;

	AEEApplet* myMain;

	/**
	 * This implementation provides two seeds, which are simply opposite patterns of alternating
	 * 1s and 0s for the trivial game provided.
	 */
public:
	FPSeedGenerator(AEEApplet* main);

	Bbyte getSeed();

};

#endif
