#ifndef __ASPRITE_H__
#define __ASPRITE_H__
#pragma once
//*************************************************************************************
// aSprite Class V1.0 BREW (C) Kolja Hegelich [Charismatix] 2006
// converted from 
// aSprite Class V1.2 MIDP1.0/2.0 (C) Ralf Niebecker & Andrea Middelhove GbR 2004/2005
//*************************************************************************************
//This class handles Sprites as defined in the MIDP2 specification but with some
//changes: sprite transformations are not supported (because of midp1 support) but
//you can cut your sprites out of any image starting at any position in the image you
//like, allowing to use one image for all your sprites.
//There is a full support for the bScroll scroll class implemented which allows to
//assign a sprite to the scrolling map which translates the x/y coordinates
//automatically to the active map of the scroll class. Collision detection with the
//scroll class is also supported.
//
//aSprite(Image img, int widthX, int widthY):
//Create a new sprite from a given image with widthX as x size and widthY as y size.
//This creates as many frames as possible out of the image by going from left
//to right and from up to down. Only full frames will be cutted out of the image
//rests are ignored.
//
//aSprite(Image img, int startX, int startY, int widthX, int widthY, int num):
//Create a new sprite and assign a frameset by cutting it out of the image.
//Have a look at the setImage(...) method for the parameters definition.
//
//aSprite():
//Create a new sprite without assigning a frameset to it. the image can be assigned
//later by using the setImage(...) method.
//
//void setImage(Image img, int startX, int startY, int widthX, int widthY, int num)
//Assign a new frameset to the sprite by cutting it out of the image. The parameter
//widthX and widthY define the size of the sprite, startX and startY the starting
//position of the first frame within the image and num the number of frames to cut
//out. Only full frames will be cutted out of the image rests are ignored.
//setImage() always creates a standard frameSquence by starting at frame 0 counting
//up to the last frame.
//
//void setImage(Image img, int widthX, int widthY):
//Cut as many frames out of the image as possible (from left to right, up to down).
//Only full frames will be cutted out of the image rests are ignored.
//setImage() always creates a standard frameSquence by starting at frame 0 counting
//up to the last frame.
//
//int getRawFrameCount()
//Returns the number of frames of the sprite.
//
//void setFrameSequence(byte[] sequence):
//void setFrameSequence(int[] sequence):
//Assign a new frameSequence to this sprite using an array of integers. frameSequnces
//are internally stored as byte arrays (saving some heap mem).
//
//int getFrameSequenceLength():
//Return the number of entries of the current frameSequence.
//
//void setFrame(int sequenceIndex)
//Set the current frame to the given index, the current frame gets painted when
//calling paint().
//
//void nextFrame():
//Selects the next frame in the frameSequence.
//
//void prevFrame():
//Selects the previous frame in the frameSequence.
//
//int getFrame():
//gets the current index in the frameSequence.
//
//void setPosition(int x,int y):
//Set the x/y coordinate of the sprite. If the sprite has been assigned to a scroll class
//the position is relative to the origin of the current map.
//
//int getX():
//Get the current x position.
//
//int getY():
//Get the current y position.
//
//void defineReferencePixel(int x, int y):
//Defines the reference pixel for this Sprite. The pixel is defined by its location relative
//to the upper-left corner of the sprite's frame, and it may lay outside of the frame's bounds.
//By default a sprite's reference pixel is located at (0,0) - the upper left pixel. 
//
//int getRefPixelX()
//Gets the horizontal position of this sprite's reference pixel.
//
//int getRefPixelY()
//Gets the vertical position of this sprite's reference pixel.
//
//void defineCollisionRectangle(int x, int y, int widthx, int widthy)
//Defines the Sprite's bounding rectangle that is used for collision detection.
//
//void assignTo(bScroll mybScroll)
//Assign this sprite to the scroll class coordinate system.
//Detache a sprite from a scroll class by setting mybScroll to null.
//
//int collidesWith(bScrollCollision myCollision, int minColValue, int maxColValue)
//Returns the number of tiles from the scroll class the sprite collides.
//The tiles itself are handed back using the myCollision object.
//The collision check will only be made with tiles whose collision values are within
//minColValue and maxColValue.
//This method can only be used if the sprite has been assigned to a scroll class.
//
//boolean collidesWith(aSprite s)
//Returns true if the bounding box of this sprites collides with the bounding box
//of the given sprite.
//
//boolean isOnScreen():
//Returns true if the sprite is within the render area of the scroll class the sprite
//has been assigned to.
//
//void paint(Graphics g)
//Render the current frame of the sprite to the Graphics object.
//
//void cleanup();
//Release all resources allocated by this sprite object.
//
// check if the reference pixel has been implemented correctly!?
#include "AEEAppGen.h"
#include "AEEStdLib.h"        // AEE Stb Lib Services
#include "aUtils.h"

class aSprite
{
public:
	aSprite();
	AEEApplet* myMain;
	//store sprites x/y position in spriteset for quick reference
	Bshort imagePosX;
	Bshort imagePosY;

	IImage*	spriteImage;



	short	maxFrames;












	//for our animation	
	Bbyte frameSequenceDef;
	Bbyte* frameSequence;
	short currentSequence;
	short currentSequenceLength;

	//store sprite x/y sizes
	short spriteWidthX;
	short spriteWidthY;

	//actual sprite position has to be public for col. detection
	short spritePosX;
	short spritePosY;

	//reference offsets for sprite based on actual sprite position
	short spriteOffsetX;
	short spriteOffsetY;

	//collision rect has to be public for collision detection
	short spriteColX;
	short spriteColY;
	short spriteColWidthX;
	short spriteColWidthY;





	//add a spriteset and split it into sprites
	//returns the index of the last sprite generated or -1 indicating an error
	void  create(IImage* img, int widthX, int widthY);
	void  create( Image* img, int widthX, int widthY);



	void  createSS(IImage* img, int startX, int startY, int widthX, int widthY, int num);
	void  createSS( Image* img, int startX, int startY, int widthX, int widthY, int num);
	void  createNONE();
	//keep it compatible with old version
	void  setImage(IImage* img, int widthX, int widthY);
	void  setImage(Image* img, int widthX, int widthY);
	void  setImageSS(IImage* img, int startX, int startY, int widthX, int widthY, int num);
	void setImageSS(Image* img, int startX, int startY, int widthX, int widthY, int num);



	void  cleanup();
	int  getFrameSequenceLength();
	Bbyte  getFrameSequence();
	int  getRawFrameCount();
	void  nextFrame();
	void  prevFrame();
	void  setFrameSequence(Bint sequence, int size);
	void setFrameSequenceInt(Bint sequence, int size);

	//void  setFrameSequenceByte(byte* sequence, int size);
	void  setFrame(int sequenceIndex);
	int  getFrame();
	void  setPosition(int x, int y);
	int  getX();
	int  getY();
	void  defineReferencePixel(int x, int y);
	int  getRefPixelX();
	int  getRefPixelY();
	void  defineCollisionRectangle(int x, int y, int widthx, int widthy);











	boolean  collidesWith(aSprite s);



	void  paint();
};

#endif
