/*
 *  Memory Handler (c)2000/2002 Ralf Niebecker aka Niewi
 */

//#include <stdlib.h>
//#include <time.h>
#include "mem.h"
//#include "sharedlib.h"
extern char* _itoa(int num, char* str, int radix);

struct MemEntry
{
    void*	ptr;
    size_t	len;
    const char*	file;
    int		line;
};
const int MEM_ENTRY_SIZE=sizeof(struct MemEntry);

struct Mem
{
	int	min_size;
	mem_errfct_t errfct;

	struct	MemStats stats;
	struct	MemEntry *table;
};
static struct Mem mem;
static int initialised=0;

/* allocate slots in this steps */


static struct MemEntry* mem_get_slot(const char* file,int line)
{
	int i;

	/* search free slot in table */
	if (mem.table != NULL)
		for (i=0;i < mem.stats.slots;i++)
			if (mem.table[i].ptr == NULL)
				return &mem.table[i];

	if (mem.table == NULL)
	{
		i=0;
		mem.table=(struct MemEntry*)MALLOC(MEM_ENTRY_SIZE*100);
		MEMSET(mem.table,0,MEM_ENTRY_SIZE*100);
		mem.stats.slots=100;
	}
	else
	{
		i=mem.stats.slots;
		mem.table=(struct MemEntry*)REALLOC(mem.table,
					(mem.stats.slots+100)*MEM_ENTRY_SIZE);

		MEMSET(&mem.table[mem.stats.slots],0,MEM_ENTRY_SIZE*100);
		mem.stats.slots+=100;
	}

	if (mem.table == NULL)
	{
		if (mem.errfct != NULL)
			mem.errfct(MEM_ENTRY_SIZE*100,NULL,file,line);
		
		//exit(EXIT_FAILURE);
		return NULL;
	}
	return &mem.table[i];
}

struct MemStats* mem_init(int min_size,mem_errfct_t errfct)
{
	mem.min_size=min_size;
	mem.errfct=errfct;

	/* reset stats */
	MEMSET(&mem.stats,0,MEM_STATS_SIZE);

	initialised++;

	return &mem.stats;
}

void* mem_alloc(size_t len,const char* file,int line)
{
	static int nCounter;
	struct MemEntry* e;
	void *ptr;

	nCounter++;
	if(nCounter == 5666 || nCounter == 5358)
		int a =0;
	if(len == 7612)
		int b = 0;

	e=mem_get_slot(file,line);

	if (mem.min_size != 0)
		len=((len/mem.min_size)+1)*mem.min_size;
		
	if ((ptr=MALLOC(len)) == NULL)
	{
		if (mem.errfct != NULL)
			mem.errfct(len,NULL,file,line);

		//exit(EXIT_FAILURE);
		return NULL;
	}

	e->ptr=ptr;
	e->len=nCounter;//len;
	e->file=file;
	e->line=line;

	mem.stats.bytes+=len;
	mem.stats.bytes_all+=len;
	
	mem.stats.slots_inuse++;
	mem.stats.slots_all++;

	return ptr;
}

void* mem_realloc(void* ptr,size_t len,const char* file,int line)
{
	void *nptr;
	int	 i,l;
	
	l=len;
	
	if (mem.table == NULL || ptr == NULL)
		return mem_alloc(len,file,line);
		
	if (mem.min_size != 0)
		len=((len/mem.min_size)+1)*mem.min_size;
	
	for (i=0;i < mem.stats.slots;i++)
	{
		if (mem.table[i].ptr == ptr) 
		{
			if (mem.table[i].len != len)
			{
				mem.stats.bytes+=len-mem.table[i].len;
				mem.stats.bytes_all+=len-mem.table[i].len;

				if ((nptr=REALLOC(ptr,len)) == NULL)
				{
					if (mem.errfct != NULL)
						mem.errfct(len,NULL,file,line);

					//exit(EXIT_FAILURE);
					return NULL;
				}
				
				mem.table[i].ptr  = nptr;
				mem.table[i].len  = len;
				mem.table[i].file = file;
				mem.table[i].line = line;

				return nptr;
			}
			return ptr;
		}
	}

	return mem_alloc(l,file,line);
}

char* mem_strdup(const char* text,const char* file,int line)
{
	char *ptr;
	int  len;

	if (text == NULL)
		return NULL;

	len=STRLEN(text)+1;

	if ((ptr=(char*)mem_alloc(len,file,line)) == NULL)
	{
		if (mem.errfct != NULL)
			mem.errfct(len,NULL,file,line);

		//exit(EXIT_FAILURE);
		return NULL;
	}

	MEMCPY(ptr,text,len);
	return ptr;
}

void mem_free(void* ptr,const char* info,const char* file,int line)
{
	int i=0;

	if (mem.table == NULL || ptr == NULL)
		return;

	for (i=0;i < mem.stats.slots;i++)
	{
		if (mem.table[i].ptr == ptr)
		{
			mem.table[i].ptr=NULL;
			mem.stats.bytes-=mem.table[i].len;
			mem.stats.slots_inuse--;
			break;
		}
	}
	if (i == mem.stats.slots)
	{
		if (mem.errfct != NULL)
			mem.errfct(0,info,file,line);

		//the main code has to decide if we should shutdown
		//because of this failure!
		//to be sure we don't free the given pointer
		FREEIF(ptr);
		return;
	}
	FREE(ptr);
}

void free_all()
{
	int i;

	for (i=0;i < mem.stats.slots;i++)
	{
		if (mem.table[i].ptr != NULL)
		{
			FREE(mem.table[i].ptr);
			mem.table[i].ptr=NULL;
		}
	}
	mem.stats.bytes=0;
	mem.stats.slots_inuse=0;
}

void print_table(IFile* fd)
{
	char _cBuf[10];
	if (fd == NULL)
		return;

	for (int i=0;i<mem.stats.slots;i++)
		if (mem.table[i].ptr != NULL)
		{
			String _szBuf = "Mem: Allocated ";
			_szBuf += _itoa(mem.table[i].len, _cBuf, 10);
			_szBuf += " Bytes in Modul ";
			_szBuf += mem.table[i].file;
			_szBuf += " Line ";
			_szBuf += _itoa(mem.table[i].line, _cBuf, 10);
			_szBuf += "\n";
			IFILE_Write(fd, _szBuf.toCharArray(), sizeof(char)*_szBuf.length());
		}
}

void mem_dump(const char* file)
{
	/*
	int i,n;
	FILE* fp;

	if (file == NULL)
		return;

	if ((fp=FOPEN(file,"at")) == NULL)
		return;

	fprintf(fp,	"Dumping Memory Allocations\n"
				"--------------------------\n");

	for (n=i=0;i < mem.stats.slots;i++)
	{
		if (mem.table[i].ptr != NULL)
		{
			n++;
			fprintf(fp,"Module:%s Line:%d Size:%d\n",mem.table[i].file,mem.table[i].line,mem.table[i].len);
			if (mem.table[i].len < 256)
				util_hexdump(mem.table[i].ptr,mem.table[i].len,fp);
			else
				fprintf(fp,"Too big for a Dump!\n\n");
		}
	}

	if (n == 0)
		fprintf(fp,"List is empty.\n\n");

	FCLOSE(fp);
	*/
}

void mem_cleanup(const char* fname)
{
	int i=0;
	//FILE* fd;
	
	if (mem.table == NULL)
		return;
/*
	if (fname != NULL)
	{
		for (;i < mem.stats.slots;i++)
			if (mem.table[i].ptr != NULL)
				break;
	
		if (i != mem.stats.slots)
		{
			if ((fd=fopen(fname,"at")) != NULL)
			{
				time_t  t;
				struct  tm* date;
				char buf[128];

				time(&t);
				date=localtime(&t);

				strftime(buf,127,"\n%a %d %b %Y %H:%M:%S\n\n",date);
				fputs(buf,fd);

				fprintf(fd,"Mem: there are still unfreed Memory Slots left!\n");
				print_table(fd);
				fprintf(fd,"Mem: freeing up all Slots!\n");
				fclose(fd);
			}
		}
	}
*/
	free_all();

	if (mem.table != NULL)
		FREE(mem.table);
	
	mem.table=NULL;
	mem.stats.slots=0;
}

void* mem_changeStr(void*  ptr, const char* file)
{
	int	 i;
	
	if (mem.table == NULL || ptr == NULL)
		return NULL;
		
	for (i=0;i < mem.stats.slots;i++)
	{
		if (mem.table[i].ptr == ptr) 
		{
			mem.table[i].file = (const char*)file;
			return ptr;
		}
	}
	return NULL;
}

void* mem_changeStr(void*  ptr, const AECHAR* file)
{
	int	 i;
	char _cBuf[100];

	if (mem.table == NULL || ptr == NULL)
		return NULL;
		
	for (i=0;i < mem.stats.slots;i++)
	{
		if (mem.table[i].ptr == ptr) 
		{
			mem.table[i].file = WSTRTOSTR(file, _cBuf, sizeof(char)*100);
			return ptr;
		}
	}
	return NULL;
}

