#pragma once

// ****************************************************************************
//  aDebug Class V1.0 BREW 1.0 (C) Kolja Hegelich [Charismatix]
//  converted from
//  aDebug Class V1.2 MIDP 1.0 (C) Andrea Middelhove
// ****************************************************************************
//
// aDebug() Constructor:
// Call the Constructor with the maximum Number of Lines of Debug Messages to
// store.
//
// aDebug.setLevel(int level)/aDebug.getLevel():
// Use setLevel(int) to set the Level of Debug Information you want to
// see, normally all Messages are shown. If a the Level of a new Message is
// higher than the set one, it will be ignored. Set a negative Level to 
// see all Debug Messages (Default Mode).
// getLevel() returns the current working Level.
//
// aDebug.add(int level,String message):
// Add a new Message, if the Level is higher than the actual Debug Level
// the Message will be ignored!
// 
// aDebug.setIgnore(boolean doIgnore)/aDebug.getIgnore():
// Easy way to stop adding new Messages to the Debug System, if set to 
// true all new Messages will be ignored. May be helpfull when scrolling
// thrue the stored Messages.
//
// aDebug.movePos(int posOffset):
// Can be used to change the current Startposition for the Debug Message
// printing. If posOffset is positiv the Messages printed with the
// aDebug.paint() Method will scroll posOffset Lines up, if it is negative
// they scroll posOffset down. Use 0 to position to the last Message
// added.
//
// aDebug.paint(Graphics g,int startX,int startY,int widthX,int widthY):
// Call the paint Method to get your Messages printed into your Graphics
// Object g. The Messages are printed into the Box build by startX, StartY
// widthX and widthY. All Messages are printed using the current Font with
// the current Color.
//
// Example:
// aDebug.setLevel(1);                     //set debug level to 1
// aDebug.add(0,"Error:...");              //add Message with Level 0
// aDebug.add(1,"Warning:...");            //add Message with Level 1
// aDebug.add(2,"Called:...");             //add Message with Level 2
// aDebug.add(3,"Info:...");               //add Message with Level 3
// aDebug.paint(myGraphics,0,0,100,100);   //print Messages in this Box
//
// Result:
// The Messages with a level higher than 1 will be ignored (not printed)
// because we set our Debug Level to 1, it's an easy way to control
// the number of messages dumped onto the screen...
//
// Info:
// You can generate multiple Instances of aDebug, all will work with the
// same Messagebuffer, only the first Instance defines the Number of
// Messages that can be stored.
#include "AEE.h"
#include "AEEGraphics.h"
//#include "AEEFont.h"
#include "AEEFile.h"
#include "AEEAppGen.h"
#include "AEEStdLib.h"        // AEE Stb Lib Services
#include "BrewString.h"
#include "BrewVector.h"

 class aDebug
{
public:
	AEEApplet* myMain;
	StringVector msgs;
	int level;
	int pos;
	boolean doIgnore;

	aDebug(void);
	~aDebug(void);
	void aDebug::freeRes();
	//this is only here for compatiblity reasons
	aDebug(int maxMessages);
	void create(AEEApplet* p_pMain);
 	void add(int lvl, String msg);
	//set the debug level
	void setLevel(int lvl);
	int getLevel();
	//move to a a sposition relative to our last one
	void movePos(int posOffset);
	//set ignore mode in which no messages will be added
	void setIgnore(boolean b);
	boolean getIgnore();
	//paint our current messages in console style
	void paint(IGraphics* p_IGraphics,int startX,int startY,int endX,int endY);
};
