#ifndef __AKEY_H__
#define __AKEY_H__

// **********************************************************************
//  aKey Class V1.0 BREW (C) Kolja Hegelich [Charismatix] 2006
//  converted from 
//  aKey Class V1.1 MIDP (C) Ralf Niebecker & Andrea Middelhove GbR 2004
// **********************************************************************
// aKey is an easy to use KeyMapper Class defining the Keys depending
// on the Device you are compiling your Application for.
// MIDP only defines KeyCodes for a standard set of Keys:
// KeyZero, KeyOne, KeyTwo, KeyThree, KeyFour, KeyFive, KeySix, KeySeven
// KeyEight, KeyNine, KeyAsterix and KeyNumber
// The KeyCodes for all other depend on the Manufacturer of the Device.
// This are: KeySoftLeft, KeySoftRight, KeySoftMid
// KeyUp, KeyDown, KeyLeft, KeyRight, KeyFire, KeyCall, KeyClear
// Not all Keys are available on each Device, you can detect this by
// checking the Value of the Key, if it's zero the Key can not be used.
// aKey is mapping the Direction Keys (Up/Down/Left/Right) and Fire to
// the DirectionPad of the Device if available and to the Standard Keys
// KeyTwo, KeyEight, KeyFour, KeySix and KeyFive.
//
// The Mapping for the Nokia 3650 is :
// Left: KeyTwo/KeyThree Right: KeyEight/KeyNine
// Up: KeyOne/KeyZero Down: KeyFive/KeySix
// Fire: KeyNumber/KeyAsterix
//
// The Mapping for the Siemens SX1/Nokia 7600 is :
// Left: KeyThree Right: KeyEight
// Up: KeyOne/KeySix Down: KeyFour/KeyNine
// Fire: KeyFive/KeyZero
// 
// Constructor: aKey(Canvas c) 
// Call the Constructor handing over your current Canvas Object.
//
// aKey.isUp(), aKey.isDown(), aKey.isLeft(), aKey.isRight(), aKey.isFire():
// Use this Functions to check if the given KeyCode is mapped to them.
// All Functions require a KeyCode and return true or false.
//
// int getGameAction(int keyCode):
// Behaves exactly like the Canvas.getGameAction(int keyCode) method.
// Use this one, it checks reliable for all Keys bound to GameActions.
//
// Example:
// aKey myKey=new aKey(myCanvas);
// ...
// public void keyPressed(int keyCode)
// {
//   //first check for standard keys
//   if (keyCode == myKey.KeyAsterix)
//   {
//     ...
//   }
//   else if (keyCode == myKey.KeyOne)
//   {
//     ...
//   }
//   // now check for game actions
//   else if (myKey.isUp(keyCode) == true)
//   {
//     ...
//   }
//   else if (myKey.isDown(keyCode) == true)
//   {
//     ...
//   }
// }
//
// Warning: First check all normal Keys before checking game related ones,
// the Reason is that the Key may be mapped to a game action on a Device
// like the Nokia 3650 causing that the Key will be interpreted as a game
// action and never reach the Code for the specific Key...
#include "AEEAppGen.h"



//typedef AEEApplet Main;

class aKey
{
public:
	aKey();
	AEEApplet* myMain;
	//device specific keys
	uint16 KeyUp       ;
	uint16 KeyDown     ;
	uint16 KeyLeft     ;
	uint16 KeyRight    ;
	uint16 KeyFire     ;
	uint16 KeyCall     ;
	uint16 KeyClear    ;
	uint16 KeySoftLeft ;
	uint16 KeySoftRight;
	uint16 KeySoftMid  ;

	//standard keys
	uint16 KeyZero    ;
	uint16 KeyOne     ;
	uint16 KeyTwo     ;
	uint16 KeyThree   ;
	uint16 KeyFour    ;
	uint16 KeyFive    ;
	uint16 KeySix     ;
	uint16 KeySeven   ;
	uint16 KeyEight   ;
	uint16 KeyNine    ;
	uint16 KeyAsterix ;
	uint16 KeyNumber  ;
	uint32 KeyState;

	void create();
	int  isUp( uint16 keyCode);
	int  isDown( uint16 keyCode);
	int  isLeft( uint16 keyCode);
	int  isRight( uint16 keyCode);
	int  isFire( uint16 keyCode);
	int  isCall( uint16 keyCode);
	int  isClear( uint16 keyCode);
	int  isSoftLeft( uint16 keyCode);
	int  isSoftRight( uint16 keyCode);
	int  isSoftMiddle( uint16 keyCode);
	int  availableFire();
	int  availableCall();
	int  availableClear();
	int  availableSoftLeft();
	int  availableSoftRight();
	int  availableSoftMiddle();
	uint16  getFire();
	uint16  getSoftLeft();
	uint16  getSoftRight();
	uint16  getGameAction( uint16 keyCode);
	uint16  updatePressedKey( uint16 keyCode);
	uint16  updateReleasedKey( uint16 keyCode);

};

#endif  //__AKEY_H__
