#pragma once

// ***************************************************************************
//  bText Class V1.0 MIDP 1.0 (C) Ralf Niebecker & Andrea Middelhove GbR 2005
// ***************************************************************************
// Can be used to generate multilanguage Applications.
//
// bText::Constructor(String name):
// Reads a File with the given Name and the extension .cfg from the JAR File,
// parses it and stores all strings line by line in a vector where the first
// String is interpreted as the Language name than a comma and than the wait
// message to be displayed while switching to that language and finally a comma
// and the locale of the language to be used for language detection. The language 
// selection screen will show them in the language names in the menu, the wait
// message can be read by using the getWaitMessage(id); method.
// For every line in the .cfg file a check is performed if the related textfile
// is available, the strings in the first line of the .cfg file relates to the 
// name1.dat file, the strings in the second line to name2.dat and so on.
// lines can be left empty in the config, which means that they will be ignored
// and there is no check performed for a corresponding .dat file.
// The dat files are text definition files, the standard Format of all 
// Definitions in the File is Key=Text
// Example: help=this is my help text!\nblah blah blah ...
// Whitespaces before and after the Text are stripped away, you have to escape
// them with \r (carriage return) \n (linefeed) \t (tab) and \s (space) if you
// need to use them in front or at the end of your Text.
//
// bText.setDomain():
// Switch to a new language, you can either hand over a id of the language
// which starts by 0 and goes up to the number of available languages - 1,
// or by using the name of the language used within the cfg file.
//
// String bText.get(String key):
// Get the Text that is stored for the given Key in the Hashtable.
//
// bText.cleanup():
// Cleanup all resources needed for the current active language.
//
// Examples:
//
// Config file example "t.cfg":
// English, Please wait ...,en
// Deutsch, Bitte warten ...,de
//
// Data file Example "t1.dat":
// help=this is my help
// quit=Exit
//
// Data file Example "t2.dat":
// help=Dies is meine Hilfe.
// quit=Beenden
//
// Example Code using the Textfile Example above:
// bText myText=new bText("/t")
//
// //set the first available language
// myText.setDomain(localLanguageIndex);
//
// //in your options code
// if (german == true)
// 	myText.setDomain("German");
// else
// 	myText.setDomain("English");
//
// //in your help screen code
// g.drawString(myText.get("help"),0,0,0);
//
// //in your Command code
// exitCommand=new Command(myText.get("quit"),Command.EXIT,1);

#include "BrewString.h"
#include "BrewVector.h"
#include "BrewHashtable.h"
#include "aeeappgen.h"


class bText
{
public:
	AEEApplet* myMain;
	String filename;
	WStringVector domains;
	int currentTextId;
	//support up to 15 languages
	Bint languageMapping;
	StringVector waitMessage;
	String localLanguage;
	int localLanguageIndex;
	WStringHashtable ht;
	String noTextFound;
	int l,c;
	int lastPosition;
	int lastPos;
	boolean isParseReady;
	boolean cleanit;
	AECHAR* m_cBuf;


public:
	bText(void);
	~bText(void);
	//check if modified-utf8 syntax is correct
	void checkModifiedUTF(byte* b, int size);
	void autodetectDomain();
	void create(AEEApplet* _pMain,String name);
	int parseCharArray( AECHAR* c, int size, boolean p_parseConfig=false, int pos=0);
	void parseByteArray(byte* b, boolean parseConfig);
	void parseConfig(WString s);
	void parse(WString s);
	void setDomain(String s);
	void setDomainID(int id);
	WString getDomain();
	WStringVector* getDomains();
	int getDomainNumber();
	boolean check(String key, WString* keyval);
	String getWaitMessage(int languageId);
	WString* get(String key, WString* keyval);
	WString get(String key);
	//cleanup all texts currently in memeory	
	void cleanup();
	void freeRes();
};
