

/*
 * Copyright (C) 2003 Radu Braniste (rbraniste@epicad.com)
 * All Rights Reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of the Author nor the names of its contributors
 *    may be used to endorse or promote products derived from this
 *    software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */
#ifndef _HashTable_h
#define _HashTable_h 
##pragma once


// Provide default hash functions for ints, longs and void*'s



template<class K> 
inline int hash(const K& k, int hashSize)
{
  int h = k.hash(hashSize);
  return h;
}

// Dd: Adding hash function for type "string".  Feel free to change
// this to make it a good one...
inline int hash(const String& s, int hashSize)
{
  int val = 0;
  for( unsigned int pos = 0; pos < s.length(); pos++ )
  {
    val += s.charAt(pos);
  }
  return val % hashSize;
}
/*
inline int hash(const BrewString<>& s, int hashSize)
{

    int        hval;  // hash value
    int        g;     // intermediate hash value
	const char* token =  s.toCharArray();

    hval = 0;

    for (const char* p = token; *p != '\0'; p++) {
        hval = (hval << 4) + (*p);

        if ((g = hval & 0xf0000000L) != 0) 
		{
            hval = hval ^ (g >> 24);
            hval = hval ^ g;
        }
    }

    return (hval % hashSize );
}*/

inline int hash(const char* s, int hashSize)
{
	int   g = 0;     // intermediate hash value
	int  hval = 0;
    for (const char* p = s; *p != '\0'; p++) {
        hval = (hval << 4) + (*p);
        if ((g = hval & 0xf0000000L) != 0) 
		{
            hval = hval ^ (g >> 24);
            hval = hval ^ g;
        }
    }

    return (hval % hashSize );
}

inline  int hash(const int& k, int hashSize)
{
  return (k^(3*hashSize+1))%hashSize;
}

/*inline  int hash(const unsigned long& k, int hashSize)
{
  return (int)(k^(3*hashSize+1))%hashSize;
} */  

typedef void* HashVoid;

inline  int hash(const HashVoid& k, int hashSize)
{
  return (int)(((long)k^(3*hashSize+1))%hashSize);
}

inline  int hash(const char& k, int hashSize)
{
	int s = k;
	if(k < 0)
	{
		s = 1024 - k;
	}
	return (s^(3*hashSize+1))%hashSize;
}

inline int hash(const AECHAR* s, int hashSize)
{
	int   g = 0;     // intermediate hash value
	int  hval = 0;
    for (const AECHAR* p = s; *p != '\0'; p++) {
        hval = (hval << 4) + (*p);
        if ((g = hval & 0xf0000000L) != 0) 
		{
            hval = hval ^ (g >> 24);
            hval = hval ^ g;
        }
    }

    return (hval % hashSize );
}

inline  int hash(const AECHAR& k, int hashSize)
{
	int s = k;
	if(k < 0)
	{
		s = 1024 - k;
	}
	return (s^(3*hashSize+1))%hashSize;
}
//template <class K, class V> class BrewHashtable;
template<class K, class V> class HashKey;


template<class K, class V > 
class BrewHashtable 
{
private:
	typedef HashKey<K, V> HKEY;
	typedef BrewHashtable<K, V> HTBL;
	const HTBL & 
	operator=( const HTBL &rhs   );
public:
	BrewHashtable() : table_(0), hashSize_(0), nElems_(0)
	{	}

	BrewHashtable(int hashSize):hashSize_(hashSize), nElems_(0)
	{
		if (hashSize_ == 0)
		{
			table_ = 0;
		}
		else 
		{
			table_=new HKEY*[hashSize_];
			for(int i=0;i<hashSize_;i++)
			{
				table_[i]=0;
			}
		}
		nElems_=0;
	}

	BrewHashtable(const HTBL& copy): hashSize_(copy.hashSize_), nElems_(copy.nElems_)
	{
		if (hashSize_ == 0)
		{
			table_ = 0;
		}
		else 
		{
			table_=new HKEY*[hashSize_];
			for(int i=0;i<hashSize_;i++)
			{
				if(copy.table_[i])
					table_[i]= new HKEY(*copy.table_[i], 1); // Deep copy
				else
					table_[i]=0;
			}
		}
	}

	~BrewHashtable()
	{
		if(table_){
			for(int i=0;i<hashSize_;i++)
			{
				HKEY* p=table_[i];
				while(p)
				{
					HKEY* prev=p;
					p=p->next;
					delete prev;
				}
			}
			delete[] table_;
		}
	}

	void put(const K& k, const V& d)  
	{
		nElems_++;
		if(3*nElems_ >= 2*hashSize_)
		{
			// Build a new table_... 
			rehash(2*hashSize_+1);
		}
		int h=hash(k, hashSize_);
		HKEY* bin=table_[h];
		HKEY* p=new HKEY(k, d, bin);
		table_[h]=p;
	}
    
	int get(const K& k, V& d)
	{
		if(table_)
		{
			int h=hash(k, hashSize_);
			for(HKEY* p=table_[h];p!=0;p=p->next)
			{
				if(p->key == k)
				{
					d = p->data;
					return 1;
				}
			}
		}
		return 0;
	}

	template<class T >
	void keys(T& bk)
	{
		if(table_)
		{
			bk.ensureCapacity(nElems_);
			for(int i=0;i<hashSize_;i++)
			{
				for(HKEY* p=table_[i];p!=0;p=p->next)
				{
					bk.append( p->key);
				}
			}
		}
	}

	template<class T>
	void elements(T& bk)
	{
		if(table_)
		{
			bk.ensureCapacity(nElems_);
			for(int i=0;i<hashSize_;i++)
			{
				for(HKEY* p=table_[i];p!=0;p=p->next)
				{
					bk.append( p->data);
				}
			}
		}
	}



	int remove(const K& k)
	{
		int count=0;
		if(table_)
		{
			int h=hash(k, hashSize_);
			HKEY* prev=0;
			HKEY* p=table_[h];
			while(p)
			{
				HKEY* next=p->next;
				if(p->key == k)
				{
					count++;
					nElems_--;
					if(prev)
					{
						prev->next=p->next;
					} 
					else 
					{
						table_[h]=p->next;
					}
					delete p;
				} 
				else 
				{
					prev=p;
				}
				p=next;
			}
		}
		return count;
	}

	void removeAll()
	{
		if(table_)
		{
			for(int i=0;i<hashSize_;i++)
			{
				HKEY* p=table_[i];
				while(p)
				{
					HKEY* next=p->next;
					delete p;
					p=next;
				}
				table_[i]=0;
			}
			nElems_=0;
			_FREE(table_);
		}
		_FREE(table_);//von mir
		this->hashSize_ = 0;
	}

	int size() const
	{
		return nElems_;
	}

	bool isEmpty() const
	{
		return nElems_==0;
	}

private:
	void rehash(int newsize)
	{
		HKEY** oldtab=table_;
		table_=new HKEY*[newsize];
		for(int ii=0;ii<newsize;ii++)
		{
			table_[ii]=0;
		}
		int oldsize=hashSize_;
		hashSize_=newsize;
		for(int i=0;i<oldsize;i++)
		{
			HKEY* p=oldtab[i];
			while(p)
			{
				HKEY* next=p->next;
				int h=hash(p->key, hashSize_);
				p->next=table_[h];
				table_[h]=p;
				p=next;
			}
		}
		if(oldtab) delete[] oldtab;
	}

	
private:
	HKEY** table_;
	int hashSize_;
	int nElems_;

};




template<class K, class V> 
class HashKey {
private:
	typedef HashKey<K, V> HKEY;
	HashKey()  : next(0)
	{}
	HashKey(const K& k, const V& d, HKEY* n): key(k), data(d), next(n)
	{}
	HashKey(const HKEY& copy, int deep=0)  : key(copy.key), data(copy.data),
				next((deep && copy.next)?new HKEY(*copy.next, 1):0)
	{}

private:
	friend class BrewHashtable<K, V>;
	K key;
	V data;
	HKEY* next;
};

typedef BrewHashtable<String, String> StringHashtable;
typedef BrewHashtable<String, WString> WStringHashtable;
typedef BrewHashtable<AECHAR, uint32> FontHashtable;







#endif

