// ************************************************************************
//  aScore Class V1.0 BREW 1.0 (C) Kolja Hegelich & Tiago Teixeira [Charismatix] 2006
//  converted from 
//  aScore Class V1.1 MIDP (C) Ralf Niebecker & Andrea Middelhove GbR 2004
// ************************************************************************
// aScore is a Class for handling HighScores, it sorts the Scores and
// stores them in a RecordStore. The Constructor defines the Name of the
// RecordStore and how many Scores you want to handle. Already stored Scores
// are read into Memory. Every Operation only takes place in the Memory,
// you have to call aScore.write() to update the RecordStore.
// 
// Constructor: aScore(String storeName, int maxScores)
//                     throws IllegalArgumentException:
// Init the needed Variables to store up to maxScores Scores, opens/creates
// the RecordStore with Name storeName and reads currently stored Scores
// into the Memory. Throws an IllegalArgumentException if maxScores is <= 0
// or if storeName == null.
//
// int checkPos(int score):
// Check if the given Score is high enough to be placed in the ScoreTable
// if yes the Position is returned if no 0 is returned. Should be used
// before asking the Player for the Name.
//
// void add(String name,int score):
// Add a Score to the HighScore Table, it will only be added if the Score
// is high enough. You have to call aScore.write() to save the changes to
// the RecordStore!
//
// void reset():
// Clear the HighScore Table. You have to call aScore.write() to save the
// changes to the RecordStore
//
// void write(boolean blocking):
// Write all changes to the RecordStore. Returns directly if nothing has
// been changed or if write() is already in progress in non-blocking Mode.
// If blocking is set to true the write() will return after the HighScore
// Table has been written to the RecordStore, else the Methode will
// return directly writing the Scores in a seperate Thread which destroys
// itself after the writing is finished. I recommend using the blocking
// mode, first Tests showed that most of the Devices are blocking anyway
// when writing into a RecordStore even if this happens in a Thread.
// Best way is to use write() as less as possible and only in Situations
// that allow stopping the whole action for up to 10 Seconds!(worst case?)
//
// boolean isWriting():
// Can be used to check if the writing of the Scores is still in progress
// when writing in non-blocking mode.
//
// int numScores():
// Returns the Number of currently stored Scores.
//
// String getName(int pos):
// Returns the Player Name at Position pos, where pos starts with 1 for
// the highest ranked Player and goes up to numScores().
//
// int getScore(int pos):
// Returns the Player Score at Position pos, where pos starts with 1 for
// the highest ranked Player and goes up to numScores().
//
// Complete integration Example:
//
// private aScore myScores=new aScore("TestScores",10);
//
// //add some standard score entries if none are available
// if (myScores.numScores() == 0)
// {
// 	myScores.add("Sandy",1000);
// 	myScores.add("Lucy",3000);
//      myScores.add("Tina",2000);
// 	myScores.write(true);
// }
//
// ...
//
// //check if a new score is high enough to be added
// int pos=myScores.checkPos(newScore);
// if (pos != 0)
// {
//	System.out.println("Congratulation! You placed Position " + pos + ", insert your Name");
// 
// 	...
//
// 	myScores.add(playerName,newScore);
// 	myScores.write(true);
// }
// else
// 	System.out.println("Sorry your Score is to low to be added!");
//
// ...
//
// //print HighScore Table
// for (int i=1;i <= myScores.numScores();i++)
// 	System.out.println(,"Pos:" + i + " Score:" + myScores.getScore(i) + " Name:" + myScores.getName(i));

##if !defined(_ASCORE_H)
##define _ASCORE_H
##include "AEEStdLib.h"        // AEE Stb Lib Services
##include "PlatformSpecific.h"
##include "BrewString.h"
##include "BrewVector.h"
##include "AEEAppGen.h"
##include "aUtils.h"

#define MAX_LINE 20

struct OS {
	char Name[MAX_LINE];
	int Score;
};

class aScore
{
private:
	//our recordstore
	char* myStoreName;
	byte changed;
	//buffer for names and scores
	StringVector Names;
	int* Scores;
	int m_iMaxScores;

	//we need our utils to read/write the data
	AEEApplet* myMain;
public:
	aScore();
	void aScoreCreate(AEEApplet* main, char* storeName, int maxScores);

	int read ( void );
	//the blocking parameter is not in use anymore, it's here to provide backward compatiblity
	void write ( );

	void add (String name, int score );

	int checkPos(int score);

	String getName(int pos);

	int getScore(int pos);

	void reset( void );

	int numScores ( void );
	//stay compatible to older versions
	boolean isWriting ( void );
	void freeRes();
};
void test ();
byte* OStobyte ( OS* s, byte * ret, int elements );
##endif // !defined(_ASCORE_H)
