#ifndef __ATEXTBOX_H__
#define __ATEXTBOX_H__
##pragma once
// ***********************************************************************************
//  aTextBox Class V1.0 BREW 1.0 (C) Kolja Hegelich [Charismatix] 2006
//  converted from 
//  aTextBox Class V1.6 MIDP 1.0 (C) Ralf Niebecker & Andrea Middelhove GbR 2004/2005
// ***********************************************************************************
// This Class takes all added Messages and formats them to generate a
// formatted output into an area on the screen. The output can be scrolled
// line by line or pagewise. You can jump to the start of the text or to the
// end of it. aTextBox is using the standard Fonts of the Device for the
// rendering of the output.
//
// aTextBox.addLine(String msg):
// Add the Line to the TextBox, use %i %u and %b to switch italic, underlined and
// bold on/off use %% to add a % character and \n to force a new line.
//
// aTextBox.add(String msg, int mode):
// Add the String msg to the TextBox, mode can be a combination of the
// following Constants:
// aTextBox.BOLD, aTextBox.UNDERLINED, aTextBox.ITALIC, aTextBox.APPEND
// The Constants BOLD/UNDERLINED/ITALIC define the Font Style to use.
// APPEND forces the Message to be appended to the last one added,
// they will be printed in the same Line if possible. You are allowed to
// mix the Constants using OR like aTextBox.APPEND|aTextBox.BOLD.
//
// aTextBox.clear():
// Clears all Strings that have been added to the TextBox.
//
// aTextBox.prepare(int startX, int startY, int endX, int endY, int mode):
// After adding all the Text you have to call this methode to prerender
// the Text into the formatted output. startX, startY, endX and endY
// define the area on the screen where the text will be rendered when
// calling the aTextBox.paint() methode. mode can be a combination of
// the following Constants:
// aTextBox.SMALL, aTextBox.MEDIUM, aTextBox.LARGE (Font Size)
// aTextBox.SYSTEM, aTextBox.MONOSPACE, aTextBox.PROPORTIONAL (Font Face)
// aTextBox.DONOTWRAP (WordWrap off)
// You are not allowed to use more than one of the Font Size/Face Constants
// at the same time. Valid mode example: aTextBox.PROPORTIONAL|aTextBox.SMALL
//
// aTextBox.movePos(int posOffset):
// Move posOffset lines up/down, where negative values scoll up positiv down.
//
// aTextBox.setHome() / aTextBox.setEnd():
// Jump to the beginning/end of the Text.
//
// boolean aTextBox.isHome() / boolean aTextBox.isEnd():
// Returns true if the condition is fulfilled else false.
//
// aTextPos.pageUp() / aTextPos.pageDown():
// Jump to next/previous Page.
//
// int aTextBox.getPos():
// Returns the line position of the text printed in the first line of the TextBox.
// May be used for an own ScrollBar.
//
// int aTextBox.getLines():
// Returns the number of lines that can be displayed in the TextBox at the same time.
// May be used for an own ScrollBar.
//
// int aTextBox.getTotalLines():
// Returns the total number of lines generated by aTextBox.prepare().
// May be used for an own ScrollBar.
//
// aTextBox.paint(Graphics g, boolean center, boolean scrollbar):
// Paints the TextBox into your Graphics object. Set center to true to generate
// centerprinted output else the generated output is left-aligned.
// If scrollbar is true a scrollbar will be painted right beneath the textarea
// it is 5 pixel width and as high as the textbox, it will only be painted if
// there is more text available as fit into the textbox.
//
// Example:
// myTextBox=new aTextBox();
//
// //now add some text
// myTextBox.addLine("%b%uScrollTest%u%b\n is demonstrating the%i Charismatix Development Environment%i (%bCDE%b).");
// myTextBox.addLine("\n%iThat's it...%i");
// 
// //now prepare the text for printing
// myTextBox.prepare(8,8,ScreenWidth-8,ScreenHeight-8,aTextBox.MEDIUM|aTextBox.SYSTEM);
//
// ...
// 
// //somewhere in your paint methode
// myTextBox.paint(g,true,true);
##include "AEE.h"
##include "AEEGraphics.h"
##include "AEEFile.h"
##include "AEEAppGen.h"
##include "AEEStdLib.h"        // AEE Stb Lib Services
##include "BrewString.h"
##include "BrewVector.h"

//standard settings
##if BREWVERSION > 0x1
##define TEXTBOX_ITALIC AEE_FONT_ITALIC
##else
##define TEXTBOX_ITALIC AEE_FONT_NORMAL
##endif
//typedef AEEApplet Main;

//needed for storing the text lines

class aTextBox
{
public:
	class aEntry
	{
	public:
		WString text;
		int mode;
		int len;
		int line;
	};

	typedef BrewVector<aEntry> VEntry;

	aTextBox();

	//reference to our main object where we get the screenwidth and screenheight values from
	AEEApplet* myMain;
	//storage the text/lines
	String test;
	VEntry myText;
//	int myTextEntries;
	VEntry myLines;
//	int myLinesEntries;

	//font symetrics
	int fontSize;
	int fontFace;
	int fontHeight;

	//positions
	int textPos;
	int numLines;
	int showLines;

	//screen dimensiones
	int startX;
	int startY;
	int endX;
	int endY;
	int offsetY;

	//can be mixed
	long PLAIN;
	int UNDERLINED;
	long BOLD;
	long ITALIC;

	//use one of
	long MEDIUM;
	long SMALL;
	long LARGE;

	//use one of
	int SYSTEM;
	int MONOSPACE;
	int PROPORTIONAL;

	//add to last text
	int APPEND;

	//word wrap
	int DONOTWRAP;
	int paintOffsetX;
	int paintOffsetY;

	const aTextBox&	operator=( const aTextBox &p_TB );
	void create( AEEApplet* main);
	int addElement( aEntry p_Entry);
	void add( WString msg, int mode);
	void clear();
	void freeRes();
	void setPaintOffsetX( int offx);
	void setPaintOffsetY( int offy);
	void movePos( int posOffset);
	void setHome();
	void setEnd();
	void pageDown();
	void pageUp();
	boolean isHome();
	boolean isEnd();
	int getPos();
	int getLines();
	int getTotalLines();
	void addLine( WString msg);
	void prepare( int sX, int sY, int eX, int eY, int mode);
	void paint( IGraphics* p_IGraphics, boolean center, boolean scrollbar);

	WStringVector getTextLines();
	
};

#endif  //__ATEXTBOX_H__
